#!#usr#bin#env python

##########################################################################*                       
######	                       CAMERA MODULE
######
######    Federico B. 2003 - dragonscraft@yahoo.it
######
################################# IMPORTS ################################*

import os
from math import sin, cos
################################# MODULE #################################*

class CVector3(object):

  def __init__(self,x,y,z):
    self.x = x
    self.y = y
    self.z = z
	
################################# CCAMERA ################################*
######
###### This is the class constructor
######
################################# CCAMERA ################################*
class CCamera(object):
  
  def __init__(self):
    vZero = CVector3(0.0, 0.0, 0.0)
    vView = CVector3(0.0, 1.0, 0.5)
    vUp   = CVector3(0.0, 0.0, 0.0)
    
    self.m_vPosition = vZero      # Init the position to zero
    self.m_vView = vView	  # Init the view to a std starting view
    self.m_vUpVector = vUp	  # Init the UpVector
	
  ############################## POSITION CAMERA ########################*
  ######
  ###### This function sets the camera's position and view and up vector.
  ######
  ############################## POSITION CAMERA ########################*	
  def PositionCamera( self, positionX, positionY, positionZ,
                            viewX,     viewY,     viewZ,
                            upVectorX, upVectorY, upVectorZ):
							
    vPosition	= CVector3(positionX, positionY, positionZ)
    vView	= CVector3(viewX, viewY, viewZ)
    vUpVector	= CVector3(upVectorX, upVectorY, upVectorZ)

    # The code above just makes it cleaner to set the variables.
    # Otherwise we would have to set each variable x y and z.

    self.m_vPosition = vPosition		# Assign the position
    self.m_vView     = vView     		# Assign the view
    self.m_vUpVector = vUpVector		# Assign the up vector
	
  ############################# MOVE CAMERA ############################*
  ######
  ###### This will move the camera forward or backward depending on the speed
  ######
  ############################ MOVE CAMERA #############################*
  def MoveCamera(self, speed):
  
    vVector = CVector3(0.0,0.0,0.0)			# Init a vector for our view


    # Get our view vector (The direciton we are facing)
    vVector.x = self.m_vView.x - self.m_vPosition.x   # This gets the direction of the X	
    vVector.y = self.m_vView.y - self.m_vPosition.y   # This gets the direction of the Y
    vVector.z = self.m_vView.z - self.m_vPosition.z   # This gets the direction of the Z

    self.m_vPosition.x += vVector.x * speed	      # Add our acceleration to our position's X
    self.m_vPosition.z += vVector.z * speed	      # Add our acceleration to our position's Z
    self.m_vView.x += vVector.x * speed		      # Add our acceleration to our view's X
    self.m_vView.z += vVector.z * speed		      # Add our acceleration to our view's Z
    
  ############################## ROTATE VIEW #############################*
  ######
  ###### This rotates the view around the position using an axis-angle rotation
  ######
  ############################## ROTATE VIEW ##############################*

  def RotateView(self, angle, x, y, z):

	vNewView = CVector3(0.0,0.0,0.0)
	vView    = CVector3(0.0,0.0,0.0)	

	# Get our view vector (The direciton we are facing)
	vView.x = self.m_vView.x - self.m_vPosition.x		# This gets the direction of the X	
	vView.y = self.m_vView.y - self.m_vPosition.y		# This gets the direction of the Y
	vView.z = self.m_vView.z - self.m_vPosition.z		# This gets the direction of the Z

	# Now that we have a view vector stored in the CVector3 "vView", we can rotate it.
	# This function will be called when we want to move the camera LEFT and RIGHT.
	# So, we need to rotate our view around our position.  Think about it like this:
	# Say we we are standing in one place looking forward.  We are looking in a
	# direction right?  Well, if we rotate our head LEFT or RIGHT, our view changes.
	# Our position does not change... just what we are looking at.  That is why
	# we are just changing m_vView because that is where we are looking.  We want
	# to rotate m_vView around m_vPosition in a circular fashion to simulate this.
	# To rotate something, we want to use sine and cosine.  We need to include
	# math.h in order to use the sin() and cos() functions.
	#
	# To do our camera rotations from now on, we will be using Axis-Angle
	# rotations.  I must warn you that the equations to calculate this
	# rotation isn't intuitive, but isn't a lot of code.  An axis-angle rotation
	# allows us to rotation a point in space around an arbitrary axis.  That means
	# we can take our view point (m_vView) and rotate around our position 
	# (m_vPosition), in any direction.  This is going to come in handy when we
	# get to moving the camera's view by the mouse.  Since the explanation behind
	# the math is larger than I want to write, I suggest checking our Paul Bourke's
	# page for an decent explanation of the math involved:  
	# http:##astronomy.swin.edu.au#~pbourke#geometry#rotate#

	# Calculate the sine and cosine of the angle once
	cosTheta = (float(cos(angle)))
	sinTheta = (float(sin(angle)))

	# Find the new x position for the new rotated point
	vNewView.x  = (cosTheta + (1 - cosTheta) * x * x)	* vView.x
	vNewView.x += ((1 - cosTheta) * x * y - z * sinTheta)	* vView.y
	vNewView.x += ((1 - cosTheta) * x * z + y * sinTheta)	* vView.z

	# Find the new y position for the new rotated point
	vNewView.y  = ((1 - cosTheta) * x * y + z * sinTheta)	* vView.x
	vNewView.y += (cosTheta + (1 - cosTheta) * y * y)	* vView.y
	vNewView.y += ((1 - cosTheta) * y * z - x * sinTheta)	* vView.z

	# Find the new z position for the new rotated point
	vNewView.z  = ((1 - cosTheta) * x * z - y * sinTheta)	* vView.x
	vNewView.z += ((1 - cosTheta) * y * z + x * sinTheta)	* vView.y
	vNewView.z += (cosTheta + (1 - cosTheta) * z * z)	* vView.z

	# Now we just add the newly rotated vector to our position to set
	# our new rotated view of our camera.
	self.m_vView.x = self.m_vPosition.x + vNewView.x
	self.m_vView.y = self.m_vPosition.y + vNewView.y
	self.m_vView.z = self.m_vPosition.z + vNewView.z
 


	
