#!/usr/bin/env python

##########################################################################*                       
######	                    GAME TUTORIAL 07 MODULE
######
######    Federico B. 2003 - dragonscraft@yahoo.it
######
################################# IMPORTS ################################*

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame
from pygame.locals import *
import pygame, pygame.key, pygame.display, pygame.mouse

############################# LOCAL IMPORTS ##############################*

from Camera import *

################################# MODULE #################################*

g_Camera = CCamera()

def resize((width, height)):
    if height==0:
        height=1
    glViewport(0, 0, width, height)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(45, 1.0*width/height, 1, 150.0)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

def init():    
    glShadeModel(GL_SMOOTH)
    glClearColor(0.0, 0.0, 0.0, 0.0)
    # Below we call our init function for our camera.  We pass in the
    # starting position of our last tutorial where we are looking dead
    # on at the color triangle.  I added .5 to the Y since the last tutorial.
    # Remember, we only need to init this stuff ONCE, then we never call this again.
			  # Position     View         Up Vector
    g_Camera.PositionCamera(0, 1.5, 6,   0, 1.5, 5,   0, 1, 0)
    glEnable(GL_DEPTH_TEST)

def draw():
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)
    glLoadIdentity()
    # Now instead of passing in hard coded values, we give gluLookAt()
    # the values that our global camera has.  Now we can check if we
    # hit the LEFT RIGHT UP and DOWN arrow keys, then update the camera.
    # It will automatically update right here.  Cool huh?

    # Give openGL our camera position, then camera view, then camera up vector
    gluLookAt(g_Camera.m_vPosition.x, g_Camera.m_vPosition.y, g_Camera.m_vPosition.z,	
              g_Camera.m_vView.x,     g_Camera.m_vView.y,     g_Camera.m_vView.z,	
              g_Camera.m_vUpVector.x, g_Camera.m_vUpVector.y, g_Camera.m_vUpVector.z)

    glBegin(GL_QUADS)				# This is our BEGIN to draw			
    for i in xrange(0,100,5):
	glColor3f(1, 1, 0)			# Make this corner YELLOW
	glVertex3f(-10 + i/5,   i,  10 - i/5)	# Here is the bottom left corner of the QUAD
	glColor3f(1, 0, 0);			# Make this corner RED
	glVertex3f(-10 + i/5,   i, -10 + i/5)	# Here is the top left corner of the QUAD			
	glColor3f(0, 1, 1);			# Make the left vertex CYAN
	glVertex3f(10 - i/5,    i, -10 + i/5)   # Here is the top right corner of the QUAD
	glColor3f(0, 0, 1);			# Make the right vertex BLUE
	glVertex3f(10 - i/5,    i,  10 - i/5)	# Here is the bottom right corner of the QUAD
    glEnd()	              

def handleKeyPress(key):
  kSpeed = 0.03
    
  if key==K_ESCAPE:
    return 0   
  elif key==K_UP:
    g_Camera.MoveCamera(kSpeed)
  elif key==K_DOWN:
    g_Camera.MoveCamera(-kSpeed)  
  elif key==K_LEFT: 
    g_Camera.RotateView(kSpeed, 0, 1, 0)
  elif key==K_RIGHT: 
    g_Camera.RotateView(-kSpeed, 0, 1, 0)
  return 1

def main():

  video_flags = OPENGL|DOUBLEBUF|RESIZABLE
    
  pygame.init()
  pygame.display.set_mode((640,480), video_flags)
  pygame.key.set_repeat(10,10) 

  resize((640,480))
  init()
  frames = 0
  done = 0
  ticks = pygame.time.get_ticks() 
  while not done:
    while 1:
      event = pygame.event.poll()
      if event.type == NOEVENT:
        break
      if event.type == KEYDOWN:
        if handleKeyPress(event.key) == 0:
          done = 1
      if event.type == QUIT:
        done = 1
    #Loop the scene
    draw()
    # Here we check for mouse and key movements every frame
    g_Camera.SetViewByMouse(640,480,pygame.mouse)  # Move the camera by the mouse
    pygame.display.flip()
    frames = frames+1

  print "fps:  %d" % ((frames*1000)/(pygame.time.get_ticks()-ticks))


if __name__ == '__main__': main()
