#!#usr#bin#env python
#############################################################################################*
#############################################################################################*                       
######	                       CAMERA MODULE
######
######    Federico B. 2003 - dragonscraft@yahoo.it
######
################################# IMPORTS ###################################################*
#############################################################################################*


import os
from math import sin, cos, sqrt
################################# MODULE ####################################################*

class CVector3(object):

  def __init__(self,x,y,z):
    self.x = x
    self.y = y
    self.z = z
    
###################################### CROSS ################################################*
######
###### This returns a perpendicular vector from 2 given vectors by 
###### taking the cross product.
######
###################################### CROSS ################################################*
def Cross(vVector1, vVector2):

  vNormal = CVector3(0.0,0.0,0.0)
	
  # The X value for the vector is:  (V1.y * V2.z) - (V1.z * V2.y)													## Get the X value
  vNormal.x = ((vVector1.y * vVector2.z) - (vVector1.z * vVector2.y))
															
  # The Y value for the vector is:  (V1.z * V2.x) - (V1.x * V2.z)
  vNormal.y = ((vVector1.z * vVector2.x) - (vVector1.x * vVector2.z))
															
  # The Z value for the vector is:  (V1.x * V2.y) - (V1.y * V2.x)
  vNormal.z = ((vVector1.x * vVector2.y) - (vVector1.y * vVector2.x))

  # Return the cross product
  return vNormal									 

####################################### MAGNITUDE ###########################################*
#####
##### This returns the magnitude of a vector
#####
####################################### MAGNITUDE ###########################################*
def Magnitude(vNormal):

  return (float(sqrt( (vNormal.x * vNormal.x) + 
					  (vNormal.y * vNormal.y) + 
					  (vNormal.z * vNormal.z) )))

####################################### NORMALIZE ###########################################*
#####
#####	This returns a normalize vector (A vector exactly of length 1)
#####
####################################### NORMALIZE ###########################################*
def Normalize(vVector):

  ## Get the magnitude of our normal
  magnitude = Magnitude(vVector)				

  vVector.x /= magnitude		
  vVector.y /= magnitude		
  vVector.z /= magnitude		
		
  ## Finally, return our normalized vector
  return vVector										

####################################### OPERATIONS ##########################################*
#####
##### theese are others varius operations, they returns vectors
#####
####################################### OPERATIONS ##########################################*  
def Add(v1, v2):
  return CVector3(v1.x+v2.x, v1.y+v2.y, v1.z+v2.z)

def Subtract(v1, v2):
  return CVector3(v1.x-v2.x, v1.y-v2.y, v1.z-v2.z)

def Multiply(v1, m):
  return CVector3(v1.x*m, v1.y*m, v1.z*m)

def Divide( v1, m):
  return CVector3(v1.x/m, v1.y/m, v1.z/m)    
	
################################# CCAMERA ###################################################*
######
###### This is the class constructor
######
################################# CCAMERA ###################################################*
class CCamera(object):
  
  def __init__(self):
    vZero = CVector3(0.0, 0.0, 0.0)
    vView = CVector3(0.0, 1.0, 0.5)
    vUp   = CVector3(0.0, 0.0, 0.0)
    
    self.m_vPosition = vZero      # Init the position to zero
    self.m_vView = vView	  # Init the view to a std starting view
    self.m_vUpVector = vUp	  # Init the UpVector
	
  ############################## POSITION CAMERA #############################*
  ######
  ###### This function sets the camera's position and view and up vector.
  ######
  ############################## POSITION CAMERA #############################*	
  def PositionCamera( self, positionX, positionY, positionZ,
                            viewX,     viewY,     viewZ,
                            upVectorX, upVectorY, upVectorZ):
							
    vPosition	= CVector3(positionX, positionY, positionZ)
    vView	= CVector3(viewX, viewY, viewZ)
    vUpVector	= CVector3(upVectorX, upVectorY, upVectorZ)

    # The code above just makes it cleaner to set the variables.
    # Otherwise we would have to set each variable x y and z.

    self.m_vPosition = vPosition		# Assign the position
    self.m_vView     = vView     		# Assign the view
    self.m_vUpVector = vUpVector		# Assign the up vector
	
  ############################# MOVE CAMERA ###################################*
  ######
  ###### This will move the camera forward or backward depending on the speed
  ######
  ############################ MOVE CAMERA ####################################*
  def MoveCamera(self, speed):
  
    vVector = CVector3(0.0,0.0,0.0)			# Init a vector for our view


    # Get our view vector (The direciton we are facing)
    vVector.x = self.m_vView.x - self.m_vPosition.x   # This gets the direction of the X	
    vVector.y = self.m_vView.y - self.m_vPosition.y   # This gets the direction of the Y
    vVector.z = self.m_vView.z - self.m_vPosition.z   # This gets the direction of the Z

    self.m_vPosition.x += vVector.x * speed	      # Add our acceleration to our position's X
    self.m_vPosition.z += vVector.z * speed	      # Add our acceleration to our position's Z
    self.m_vView.x += vVector.x * speed		      # Add our acceleration to our view's X
    self.m_vView.z += vVector.z * speed		      # Add our acceleration to our view's Z
    
  ############################## ROTATE VIEW ##################################*
  ######
  ###### This rotates the view around the position using an axis-angle rotation
  ######
  ############################## ROTATE VIEW ##################################*
  def RotateView(self, angle, x, y, z):

	vNewView = CVector3(0.0,0.0,0.0)
	vView    = CVector3(0.0,0.0,0.0)	

	# Get our view vector (The direciton we are facing)
	vView.x = self.m_vView.x - self.m_vPosition.x		# This gets the direction of the X	
	vView.y = self.m_vView.y - self.m_vPosition.y		# This gets the direction of the Y
	vView.z = self.m_vView.z - self.m_vPosition.z		# This gets the direction of the Z

	# Calculate the sine and cosine of the angle once
	cosTheta = (float(cos(angle)))
	sinTheta = (float(sin(angle)))

	# Find the new x position for the new rotated point
	vNewView.x  = (cosTheta + (1 - cosTheta) * x * x)	* vView.x
	vNewView.x += ((1 - cosTheta) * x * y - z * sinTheta)	* vView.y
	vNewView.x += ((1 - cosTheta) * x * z + y * sinTheta)	* vView.z

	# Find the new y position for the new rotated point
	vNewView.y  = ((1 - cosTheta) * x * y + z * sinTheta)	* vView.x
	vNewView.y += (cosTheta + (1 - cosTheta) * y * y)	* vView.y
	vNewView.y += ((1 - cosTheta) * y * z - x * sinTheta)	* vView.z

	# Find the new z position for the new rotated point
	vNewView.z  = ((1 - cosTheta) * x * z - y * sinTheta)	* vView.x
	vNewView.z += ((1 - cosTheta) * y * z + x * sinTheta)	* vView.y
	vNewView.z += (cosTheta + (1 - cosTheta) * z * z)	* vView.z

	# Now we just add the newly rotated vector to our position to set
	# our new rotated view of our camera.
	self.m_vView.x = self.m_vPosition.x + vNewView.x
	self.m_vView.y = self.m_vPosition.y + vNewView.y
	self.m_vView.z = self.m_vPosition.z + vNewView.z
	
  ############################## ROTATE AROUND POINT ##########################*
  #####
  #####	This rotates the position around a given point
  #####
  ############################## ROTATE AROUND POINT ##########################*
  def RotateAroundPoint(self, vCenter, angle, x, y, z):

	vNewPosition = CVector3(0.0,0.0,0.0)			
	# To rotate our position around a point, what we need to do is find
	# a vector from our position to the center point we will be rotating around.
	# Once we get this vector, then we rotate it along the specified axis with
	# the specified degree.  Finally the new vector is added center point that we
	# rotated around (vCenter) to become our new position.  That's all it takes.

	# Get the vVector from our position to the center we are rotating around
	vPos = Subtract(self.m_vPosition , vCenter)

	# Calculate the sine and cosine of the angle once
	cosTheta = (float(cos(angle)))
	sinTheta = (float(sin(angle)))

	# Find the new x position for the new rotated point
	vNewPosition.x  = (cosTheta + (1 - cosTheta) * x * x)		* vPos.x
	vNewPosition.x += ((1 - cosTheta) * x * y - z * sinTheta)	* vPos.y
	vNewPosition.x += ((1 - cosTheta) * x * z + y * sinTheta)	* vPos.z

	# Find the new y position for the new rotated point
	vNewPosition.y  = ((1 - cosTheta) * x * y + z * sinTheta)	* vPos.x
	vNewPosition.y += (cosTheta + (1 - cosTheta) * y * y)		* vPos.y
	vNewPosition.y += ((1 - cosTheta) * y * z - x * sinTheta)	* vPos.z

	# Find the new z position for the new rotated point
	vNewPosition.z  = ((1 - cosTheta) * x * z - y * sinTheta)	* vPos.x
	vNewPosition.z += ((1 - cosTheta) * y * z + x * sinTheta)	* vPos.y
	vNewPosition.z += (cosTheta + (1 - cosTheta) * z * z)		* vPos.z

	# Now we just add the newly rotated vector to our position to set
	# our new rotated position of our camera.
	self.m_vPosition = Add(vCenter , vNewPosition)
 


	
