#!#usr#bin#env python

##########################################################################*                       
######	                    GAME TUTORIAL 09 MODULE
######
######    Federico B. 2003 - dragonscraft@yahoo.it
######
################################# IMPORTS ################################*

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame
from pygame.locals import *
import pygame, pygame.key, pygame.display

############################# LOCAL IMPORTS ##############################*

from Camera import *

################################# MODULE #################################*

g_Camera = CCamera()

def resize((width, height)):
    if height==0:
        height=1
    glViewport(0, 0, width, height)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(45, 1.0*width/height, 1, 150.0)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

def init():    
    glShadeModel(GL_SMOOTH)
    glClearColor(0.0, 0.0, 0.0, 0.0)
    # Below we call our init function for our camera.  We pass in the
    # starting position of our last tutorial where we are looking dead
    # on at the color triangle.  I added .5 to the Y since the last tutorial.
    # Remember, we only need to init this stuff ONCE, then we never call this again.
			  # Position     View         Up Vector
    g_Camera.PositionCamera(0, 1.5, 6,   0, 1.5, 0,   0, 1, 0)
    glEnable(GL_DEPTH_TEST)

################################# CREATE PYRAMID ################################*
#####
#####	This creates a pyramid with the center being (X, Y, Z).
#####   The width and height depend on the WIDTH and HEIGHT passed in
#####
################################# CREATE PYRAMID ################################*
def CreatePyramid( x, y, z, width, height):

    ## Below we create a pyramid (hence CreatePyramid() :).  This entails
    ## 4 triangles for the sides and one QUAD for the bottom.  We could have done
    ## 2 triangles instead of a QUAD but it's less code and seemed appropriate.
    ## We also assign some different colors to each vertex to add better visibility.
    ## The pyramid will be centered around (x, y, z).  This code is also used in the
    ## Lighting and Fog tutorial on our site at www.GameTutorials.com.

    ## Start rendering the 4 triangles for the sides
    glBegin(GL_TRIANGLES)					
    ## These vertices create the Back Side
    glColor3f(1, 0, 0) 
    glVertex3f(x, y + height, z)			## Top point
    glColor3f(0, 1, 1) 
    glVertex3f(x - width, y - height, z - width)	## Bottom left point
    glColor3f(1, 0, 1) 
    glVertex3f(x + width, y - height, z - width)	## Bottom right point

    ## These vertices create the Front Side
    glColor3f(1, 0, 0)
    glVertex3f(x, y + height, z)			## Top point
    glColor3f(0, 1, 1)
    glVertex3f(x + width, y - height, z + width)	## Bottom right point
    glColor3f(1, 0, 1)
    glVertex3f(x - width, y - height, z + width)	## Bottom left point

    ## These vertices create the Front Left Side
    glColor3f(1, 0, 0)
    glVertex3f(x, y + height, z)			## Top point
    glColor3f(1, 0, 1)
    glVertex3f(x - width, y - height, z + width)	## Front bottom point
    glColor3f(0, 1, 1)
    glVertex3f(x - width, y - height, z - width)	## Bottom back point

    ## These vertices create the Front right Side
    glColor3f(1, 0, 0)
    glVertex3f(x, y + height, z)			## Top point
    glColor3f(1, 0, 1)
    glVertex3f(x + width, y - height, z - width)	## Bottom back point
    glColor3f(0, 1, 1)
    glVertex3f(x + width, y - height, z + width)	## Front bottom point				
    glEnd()

    ## Now render the bottom of our pyramid
    glBegin(GL_QUADS)
    ## These vertices create the bottom of the pyramid
    glColor3f(0, 0, 1)
    glVertex3f(x - width, y - height, z + width);	## Front left point
    glColor3f(0, 0, 1)
    glVertex3f(x + width, y - height, z + width);	## Front right point
    glColor3f(0, 0, 1)
    glVertex3f(x + width, y - height, z - width);	## Back right point
    glColor3f(0, 0, 1)
    glVertex3f(x - width, y - height, z - width);	## Back left point
    glEnd() 
	
################################# DRAW 3D GRID ################################*
#####
#####   This function create a simple grid to get a better view of the animation
#####
################################# DRAW 3D GRID ################################*
def Draw3DSGrid():

    ## This function was added to give a better feeling of moving around.
    ## A black background just doesn't give it to ya :)  We just draw 100
    ## green lines vertical and horizontal along the X and Z axis.

    ## Turn the lines GREEN
    glColor3ub(0, 255, 0);

    ## Draw a 1x1 grid along the X and Z axis'
    for i in xrange(-50, 50, 1):

      ## Start drawing some lines
      glBegin(GL_LINES)
      ## Do the horizontal lines (along the X)
      glVertex3f(-50, 0, i)
      glVertex3f(50, 0, i)

      ## Do the vertical lines (along the Z)
      glVertex3f(i, 0, -50)
      glVertex3f(i, 0, 50)

      ## Stop drawing lines
      glEnd()
	   
    
def draw():
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)
    glLoadIdentity()
    # Now instead of passing in hard coded values, we give gluLookAt()
    # the values that our global camera has.  Now we can check if we
    # hit the LEFT RIGHT UP and DOWN arrow keys, then update the camera.
    # It will automatically update right here.  Cool huh?

    # Give openGL our camera position, then camera view, then camera up vector
    gluLookAt(g_Camera.m_vPosition.x, g_Camera.m_vPosition.y, g_Camera.m_vPosition.z,	
              g_Camera.m_vView.x,     g_Camera.m_vView.y,     g_Camera.m_vView.z,	
              g_Camera.m_vUpVector.x, g_Camera.m_vUpVector.y, g_Camera.m_vUpVector.z)

    Draw3DSGrid()
    
    # Below we create 4 pyramids which will just be objects in the world
    # to orient us.  Nothing special, but I figured I should add them since
    # it was only 4 lines of extra code and they act as pillars.

    # Create a pyramid at (-6, 3, 6) with a width and height of 1 and 6
    CreatePyramid(-6, 3, 6, 1, 6)

    # Create a pyramid at (-6, 3, 6) with a width and height of 1 and 6
    CreatePyramid(6, 3, 6, 1, 6)

    # Create a pyramid at (-6, 3, 6) with a width and height of 1 and 6
    CreatePyramid(6, 3, -6, 1, 6)

    # Create a pyramid at (-6, 3, 6) with a width and height of 1 and 6
    CreatePyramid(-6, 3, -6, 1, 6)

    # Translate the object represented as our character to our view position.
    # This way we will always be looking at the object and it will follow us
    # where ever we go.
    glTranslatef(g_Camera.m_vView.x, 0, g_Camera.m_vView.z)

    # Now we create our object.  It's simply 2 pyramids with one turned upside down.
    # This will always be in the center of our screen because we keep setting it's
    # position to be where we are looking.

    # Create a pyramid at (0, 2, 0) with a width and height of 1
    CreatePyramid(0, 2, 0, 1, 1)

    # Before we create the other pyramid that makes up our object, 
    # we rotate it 180 degrees around the X axis which turns it upside down.
    glRotatef(180, 1, 0, 0)

    # Create a pyramid at (0, -1, 0) with a width and height of 1
    CreatePyramid(0, -1, 0, 1, 1)
     
def handleKeyPress(key):
  kSpeed = 0.03
    
  if key==K_ESCAPE:
    return 0   
  elif key==K_UP:
    g_Camera.MoveCamera(kSpeed)
  elif key==K_DOWN:
    g_Camera.MoveCamera(-kSpeed)  
  elif key==K_LEFT: 
    g_Camera.RotateAroundPoint(g_Camera.m_vView, kSpeed, 0, 1, 0)	
  elif key==K_RIGHT: 
    g_Camera.RotateAroundPoint(g_Camera.m_vView, -kSpeed, 0, 1, 0)	
  return 1

def main():

  video_flags = OPENGL|DOUBLEBUF|RESIZABLE
    
  pygame.init()
  pygame.display.set_mode((640,480), video_flags)
  pygame.key.set_repeat(10,10) 

  resize((640,480))
  init()
  frames = 0
  done = 0
  ticks = pygame.time.get_ticks() 
  while not done:
    while 1:
      event = pygame.event.poll()
      if event.type == NOEVENT:
        break
      if event.type == KEYDOWN:
        if handleKeyPress(event.key) == 0:
          done = 1
      if event.type == QUIT:
        done = 1
    #Loop the scene
    draw()
    pygame.display.flip()
    frames = frames+1

  print "fps:  %d" % ((frames*1000)/(pygame.time.get_ticks()-ticks))


if __name__ == '__main__': main()
