#!#usr#bin#env python
####################################################################################*
####################################################################################*                       
######	                       CAMERA MODULE
######
######    Federico B. 2003 - dragonscraft@yahoo.it
######
################################# IMPORTS ##########################################*
####################################################################################*


import os
from math import sin, cos, sqrt
from OpenGL.GL import *
from OpenGL.GLU import *
import pygame
from pygame.locals import *
import pygame, pygame.key, pygame.mouse

################################# MODULE ###########################################*

class CVector3(object):

  def __init__(self,x,y,z):
    self.x = x
    self.y = y
    self.z = z
    
###################################### CROSS #######################################*
######
###### This returns a perpendicular vector from 2 given vectors by 
###### taking the cross product.
######
###################################### CROSS #######################################*
def Cross(vVector1, vVector2):

  vNormal = CVector3(0.0,0.0,0.0)
	
  # The X value for the vector is:  (V1.y * V2.z) - (V1.z * V2.y)													## Get the X value
  vNormal.x = ((vVector1.y * vVector2.z) - (vVector1.z * vVector2.y))															
  # The Y value for the vector is:  (V1.z * V2.x) - (V1.x * V2.z)
  vNormal.y = ((vVector1.z * vVector2.x) - (vVector1.x * vVector2.z))															
  # The Z value for the vector is:  (V1.x * V2.y) - (V1.y * V2.x)
  vNormal.z = ((vVector1.x * vVector2.y) - (vVector1.y * vVector2.x))
  # Return the cross product
  return vNormal									 

####################################### MAGNITUDE ###################################*
#####
##### This returns the magnitude of a vector
#####
####################################### MAGNITUDE ###################################*
def Magnitude(vNormal):

  return (float(sqrt( (vNormal.x * vNormal.x) + 
					  (vNormal.y * vNormal.y) + 
					  (vNormal.z * vNormal.z) )))

####################################### NORMALIZE ###################################*
#####
#####	This returns a normalize vector (A vector exactly of length 1)
#####
####################################### NORMALIZE ###################################*
def Normalize(vVector):

  ## Get the magnitude of our normal
  magnitude = Magnitude(vVector)				
  vVector.x /= magnitude		
  vVector.y /= magnitude		
  vVector.z /= magnitude				
  ## Finally, return our normalized vector
  return vVector										

####################################### OPERATIONS ###################################*
#####
##### theese are others varius operations, they returns vectors
#####
####################################### OPERATIONS ###################################*  
def Add(v1, v2):
  return CVector3(v1.x+v2.x, v1.y+v2.y, v1.z+v2.z)

def Subtract(v1, v2):
  return CVector3(v1.x-v2.x, v1.y-v2.y, v1.z-v2.z)

def Multiply(v1, m):
  return CVector3(v1.x*m, v1.y*m, v1.z*m)

def Divide( v1, m):
  return CVector3(v1.x/m, v1.y/m, v1.z/m)    
	
################################# CCAMERA ############################################*
######
###### This is the class constructor
######
################################# CCAMERA ############################################*
class CCamera(object):
  
  def __init__(self):
    vZero = CVector3(0.0, 0.0, 0.0)
    vView = CVector3(0.0, 1.0, 0.5)
    vUp   = CVector3(0.0, 0.0, 0.0)
    
    self.m_vPosition = vZero      # Init the position to zero
    self.m_vView = vView	  # Init the view to a std starting view
    self.m_vUpVector = vUp	  # Init the UpVector
	
  ############################## POSITION CAMERA ############################*
  ######
  ###### This function sets the camera's position and view and up vector.
  ######
  ############################## POSITION CAMERA ############################*	
  def PositionCamera( self, positionX, positionY, positionZ,
                            viewX,     viewY,     viewZ,
                            upVectorX, upVectorY, upVectorZ):
							
    vPosition	= CVector3(positionX, positionY, positionZ)
    vView	= CVector3(viewX, viewY, viewZ)
    vUpVector	= CVector3(upVectorX, upVectorY, upVectorZ)
    # The code above just makes it cleaner to set the variables.
    # Otherwise we would have to set each variable x y and z.
    self.m_vPosition = vPosition		# Assign the position
    self.m_vView     = vView     		# Assign the view
    self.m_vUpVector = vUpVector		# Assign the up vector
	
  ############################ MOVE CAMERA ##################################*
  ######
  ###### This will move the camera forward or backward depending on the speed
  ######
  ############################ MOVE CAMERA ##################################*
  def MoveCamera(self, speed):
  
    vVector = CVector3(0.0,0.0,0.0)			# Init a vector for our view
    # Get our view vector (The direciton we are facing)
    vVector.x = self.m_vView.x - self.m_vPosition.x   # This gets the direction of the X	
    vVector.y = self.m_vView.y - self.m_vPosition.y   # This gets the direction of the Y
    vVector.z = self.m_vView.z - self.m_vPosition.z   # This gets the direction of the Z

    self.m_vPosition.x += vVector.x * speed	      # Add our acceleration to our position's X
    self.m_vPosition.z += vVector.z * speed	      # Add our acceleration to our position's Z
    self.m_vView.x += vVector.x * speed		      # Add our acceleration to our view's X
    self.m_vView.z += vVector.z * speed		      # Add our acceleration to our view's Z
    
  ############################## ROTATE VIEW ##################################*
  ######
  ###### This rotates the view around the position using an axis-angle rotation
  ######
  ############################## ROTATE VIEW ##################################*
  def RotateView(self, angle, x, y, z):

	vNewView = CVector3(0.0,0.0,0.0)
	vView    = CVector3(0.0,0.0,0.0)	
	# Get our view vector (The direciton we are facing)
	vView.x = self.m_vView.x - self.m_vPosition.x		# This gets the direction of the X	
	vView.y = self.m_vView.y - self.m_vPosition.y		# This gets the direction of the Y
	vView.z = self.m_vView.z - self.m_vPosition.z		# This gets the direction of the Z
	# Calculate the sine and cosine of the angle once
	cosTheta = (float(cos(angle)))
	sinTheta = (float(sin(angle)))
	# Find the new x position for the new rotated point
	vNewView.x  = (cosTheta + (1 - cosTheta) * x * x)	* vView.x
	vNewView.x += ((1 - cosTheta) * x * y - z * sinTheta)	* vView.y
	vNewView.x += ((1 - cosTheta) * x * z + y * sinTheta)	* vView.z
	# Find the new y position for the new rotated point
	vNewView.y  = ((1 - cosTheta) * x * y + z * sinTheta)	* vView.x
	vNewView.y += (cosTheta + (1 - cosTheta) * y * y)	* vView.y
	vNewView.y += ((1 - cosTheta) * y * z - x * sinTheta)	* vView.z
	# Find the new z position for the new rotated point
	vNewView.z  = ((1 - cosTheta) * x * z - y * sinTheta)	* vView.x
	vNewView.z += ((1 - cosTheta) * y * z + x * sinTheta)	* vView.y
	vNewView.z += (cosTheta + (1 - cosTheta) * z * z)	* vView.z
	# Now we just add the newly rotated vector to our position to set
	# our new rotated view of our camera.
	self.m_vView.x = self.m_vPosition.x + vNewView.x
	self.m_vView.y = self.m_vPosition.y + vNewView.y
	self.m_vView.z = self.m_vPosition.z + vNewView.z
	
  ############################## ROTATE AROUND POINT ##########################*
  #####
  #####	This rotates the position around a given point
  #####
  ############################## ROTATE AROUND POINT ##########################*
  def RotateAroundPoint(self, vCenter, angle, x, y, z):

	vNewPosition = CVector3(0.0,0.0,0.0)			
	# Get the vVector from our position to the center we are rotating around
	vPos = Subtract(self.m_vPosition , vCenter)
	# Calculate the sine and cosine of the angle once
	cosTheta = (float(cos(angle)))
	sinTheta = (float(sin(angle)))
	# Find the new x position for the new rotated point
	vNewPosition.x  = (cosTheta + (1 - cosTheta) * x * x)		* vPos.x
	vNewPosition.x += ((1 - cosTheta) * x * y - z * sinTheta)	* vPos.y
	vNewPosition.x += ((1 - cosTheta) * x * z + y * sinTheta)	* vPos.z
	# Find the new y position for the new rotated point
	vNewPosition.y  = ((1 - cosTheta) * x * y + z * sinTheta)	* vPos.x
	vNewPosition.y += (cosTheta + (1 - cosTheta) * y * y)		* vPos.y
	vNewPosition.y += ((1 - cosTheta) * y * z - x * sinTheta)	* vPos.z
	# Find the new z position for the new rotated point
	vNewPosition.z  = ((1 - cosTheta) * x * z - y * sinTheta)	* vPos.x
	vNewPosition.z += ((1 - cosTheta) * y * z + x * sinTheta)	* vPos.y
	vNewPosition.z += (cosTheta + (1 - cosTheta) * z * z)		* vPos.z
	# Now we just add the newly rotated vector to our position to set
	# our new rotated position of our camera.
	self.m_vPosition = Add(vCenter , vNewPosition)
	
  ################################# SET VIEW BY MOUSE ################################*
  #####
  #####	This allows us to look around using the mouse, like in most first person games.
  #####
  ################################# SET VIEW BY MOUSE ################################*
  def SetViewByMouse(self, mouseObj, screenWidth, screenHeight):

    middleX = screenWidth  >> 1			# This is a binary shift to get half the width
    middleY = screenHeight >> 1			# This is a binary shift to get half the height
    angleY = 0.0						# This is the direction for looking up or down
    angleZ = 0.0						# This will be the value we need to rotate around the Y axis (Left and Right)
    currentRotX = 0.0
       
  # Get the mouse's current X,Y position
    mouseX,mouseY = mouseObj.get_pos()
  
  # Now that we got the mouse position, we want to put the mouse position
  # back at the middle of the screen.  We pass in half of our screen width and height.
  # The >> operator is a binary shift.  So, we are shifting our width and height 
  # to the right by 1.  If you do the binary math, it is the same thing and dividing by 2, 
  # but extremely faster.  The reason why we put the cursor in the middle of the 
  # screen each time is we can can get a delta (difference) of how far we move 
  # each frame, so we know how much to rotate the camera.

  # If our cursor is still in the middle, we never moved... so don't update the screen
    if( (mouseX == middleX) & (mouseY == middleY) ):
      return

  # Set the mouse position to the middle of our window
    mouseObj.set_pos(middleX, middleY)                            

  # Now we need to get the direction (or VECTOR) that the mouse moved.
  # To do this, it's a simple subtraction.  Just take the middle point,
  # and subtract the new point from it:  VECTOR = P1 - P2;   with P1
  # being the middle point (400, 300) and P2 being the new mouse location.
  # Look below at the *Quick Notes* for more information and examples.
  # After we get the X and Y delta (or direction), I divide by 1000 to bring the number
  # down a bit, otherwise the camera would move lightning fast!

  # Get the direction the mouse moved in, but bring the number down to a reasonable amount
    angleY = (float( middleX - mouseX ) / 1000.0 )       
    angleZ = (float( middleY - mouseY ) / 1000.0 )       

  # Here we keep track of the current rotation (for up and down) so that
  # we can restrict the camera from doing a full 360 loop.
    currentRotX -= angleZ  

  # If the current rotation (in radians) is greater than 1.0, we want to cap it.
    if(currentRotX > 1.0):
      currentRotX = 1.0
  # Check if the rotation is below -1.0, if so we want to make sure it doesn't continue
    elif(currentRotX < -1.0):
      currentRotX = -1.0
  # Otherwise, we can rotate the view around our position
    else:

    # To find the axis we need to rotate around for up and down
    # movements, we need to get a perpendicular vector from the
    # camera's view vector and up vector.  This will be the axis.
    # Before using the axis, it's a good idea to normalize it first.
      vAxis = Cross(Subtract(self.m_vView, self.m_vPosition), self.m_vUpVector)
      vAxis = Normalize(vAxis)

    # Rotate around our perpendicular axis and along the y-axis
      self.RotateView(angleZ, vAxis.x, vAxis.y, vAxis.z)
      self.RotateView(angleY, 0, 1, 0)
	
  ################################# STRAFE CAMERA ###############################*
  #####
  #####	This strafes the camera left and right depending on the speed (-#+)
  #####
  ################################# STRAFE CAMERA ###############################*
  def StrafeCamera(self, speed):

    # Strafing is quite simple if you understand what the cross product is.
    # If you have 2 vectors (say the up vVector and the view vVector) you can
    # use the cross product formula to get a vVector that is 90 degrees from the 2 vectors.
    # For a better explanation on how this works, check out the OpenGL "Normals" tutorial at our site.
    # In our new Update() function, we set the strafing vector (m_vStrafe).  Due
    # to the fact that we need this vector for many things including the strafing
    # movement and camera rotation (up and down), we just calculate it once.
    #
    # Like our MoveCamera() function, we add the strafing vector to our current position 
    # and view.  It's as simple as that.  It has already been calculated in Update().
			
    # Add the strafe vector to our position
    self.m_vPosition.x += self.m_vStrafe.x * speed
    self.m_vPosition.z += self.m_vStrafe.z * speed

    # Add the strafe vector to our view
    self.m_vView.x += self.m_vStrafe.x * speed
    self.m_vView.z += self.m_vStrafe.z * speed

  # The next 3 functions were added to our camera class.  The less code in 
  # Main.cpp the better.

  ############################ CHECK FOR MOVEMENT ############################*
  #####
  #####	This function handles the input faster than in the WinProc()
  #####
  ############################ CHECK FOR MOVEMENT ############################*
  def CheckForMovement(self, key):
    kSpeed = 0.1
  
    # Check if we hit the Up arrow or the 'w' key
    if(key==K_UP ):				
    # Move our camera forward by a positive SPEED
      self.MoveCamera(kSpeed)				

    # Check if we hit the Down arrow or the 's' key
    elif(key==K_DOWN ):			
      # Move our camera backward by a negative SPEED
      self.MoveCamera(-kSpeed)				

    # Check if we hit the Left arrow or the 'a' key
    elif(key==K_LEFT ):			
      # Strafe the camera left
      self.StrafeCamera(-kSpeed)

    # Check if we hit the Right arrow or the 'd' key
    elif(key==K_RIGHT ):		
      # Strafe the camera right
      self.StrafeCamera(kSpeed)

  ################################# UPDATE ################################*
  #####
  #####	This updates the camera's view and strafe vector
  #####
  ################################# UPDATE ################################*
  def Update(self, mouse, screen_w,screen_h):

	# Below we calculate the strafe vector every time we update
	# the camera.  This is because many functions use it so we might
	# as well calculate it only once.  

	# Initialize a variable for the cross product result
	vCross = Cross(Subtract(self.m_vView , self.m_vPosition), self.m_vUpVector)

	# Normalize the strafe vector
	self.m_vStrafe = Normalize(vCross)

	# Move the camera's view by the mouse
	self.SetViewByMouse(mouse, screen_w,screen_h)

	# This checks to see if the keyboard was pressed
	#self.CheckForMovement(key)

  ################################# LOOK ################################*
  #####
  #####	This updates the camera according to the 
  #####
  ################################# LOOK ################################*
  def Look(self):

	# Give openGL our camera position, then camera view, then camera up vector
	gluLookAt(self.m_vPosition.x, self.m_vPosition.y, self.m_vPosition.z,	
		  self.m_vView.x,	 self.m_vView.y,     self.m_vView.z,	
		  self.m_vUpVector.x, self.m_vUpVector.y, self.m_vUpVector.z)




	
