#!#usr#bin#env python

##########################################################################*                       
######	                    GAME TUTORIAL 09 MODULE
######
######    Federico B. 2003 - dragonscraft@yahoo.it
######
################################# IMPORTS ################################*

from OpenGL.GL import *
from OpenGL.GLU import *
from math import cos, sin 
import pygame
from pygame.locals import *
import pygame, pygame.key, pygame.display, pygame.mouse

############################# LOCAL IMPORTS ##############################*

from Camera import *

################################# MODULE #################################*

g_Camera = CCamera()

def resize((width, height)):
    if height==0:
        height=1
    glViewport(0, 0, width, height)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(45, 1.0*width/height, 1, 150.0)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

def init():    
    glShadeModel(GL_SMOOTH)
    glClearColor(0.0, 0.0, 0.0, 0.0)
    # Below we call our init function for our camera.  We pass in the
    # starting position of our last tutorial where we are looking dead
    # on at the color triangle.  I added .5 to the Y since the last tutorial.
    # Remember, we only need to init this stuff ONCE, then we never call this again.
			  # Position     View         Up Vector
    g_Camera.PositionCamera(0, 1.5, 6,   0, 1.5, 0,   0, 1, 0)
    glEnable(GL_DEPTH_TEST)

################################# CREATE PYRAMID ################################*
#####
#####	This creates a pyramid with the center being (X, Y, Z).
#####   The width and height depend on the WIDTH and HEIGHT passed in
#####
################################# CREATE PYRAMID ################################*
def CreatePyramid( x, y, z, width, height):

    ## Below we create a pyramid (hence CreatePyramid() :).  This entails
    ## 4 triangles for the sides and one QUAD for the bottom.  We could have done
    ## 2 triangles instead of a QUAD but it's less code and seemed appropriate.
    ## We also assign some different colors to each vertex to add better visibility.
    ## The pyramid will be centered around (x, y, z).  This code is also used in the
    ## Lighting and Fog tutorial on our site at www.GameTutorials.com.

    ## Start rendering the 4 triangles for the sides
    glBegin(GL_TRIANGLES)					
    ## These vertices create the Back Side
    glColor3f(1, 0, 0) 
    glVertex3f(x, y + height, z)			## Top point
    glColor3f(0, 1, 1) 
    glVertex3f(x - width, y - height, z - width)	## Bottom left point
    glColor3f(1, 0, 1) 
    glVertex3f(x + width, y - height, z - width)	## Bottom right point

    ## These vertices create the Front Side
    glColor3f(1, 0, 0)
    glVertex3f(x, y + height, z)			## Top point
    glColor3f(0, 1, 1)
    glVertex3f(x + width, y - height, z + width)	## Bottom right point
    glColor3f(1, 0, 1)
    glVertex3f(x - width, y - height, z + width)	## Bottom left point

    ## These vertices create the Front Left Side
    glColor3f(1, 0, 0)
    glVertex3f(x, y + height, z)			## Top point
    glColor3f(1, 0, 1)
    glVertex3f(x - width, y - height, z + width)	## Front bottom point
    glColor3f(0, 1, 1)
    glVertex3f(x - width, y - height, z - width)	## Bottom back point

    ## These vertices create the Front right Side
    glColor3f(1, 0, 0)
    glVertex3f(x, y + height, z)			## Top point
    glColor3f(1, 0, 1)
    glVertex3f(x + width, y - height, z - width)	## Bottom back point
    glColor3f(0, 1, 1)
    glVertex3f(x + width, y - height, z + width)	## Front bottom point				
    glEnd()

    ## Now render the bottom of our pyramid
    glBegin(GL_QUADS)
    ## These vertices create the bottom of the pyramid
    glColor3f(0, 0, 1)
    glVertex3f(x - width, y - height, z + width);	## Front left point
    glColor3f(0, 0, 1)
    glVertex3f(x + width, y - height, z + width);	## Front right point
    glColor3f(0, 0, 1)
    glVertex3f(x + width, y - height, z - width);	## Back right point
    glColor3f(0, 0, 1)
    glVertex3f(x - width, y - height, z - width);	## Back left point
    glEnd() 
	
################################# DRAW 3D GRID ################################*
#####
#####   This function create a simple grid to get a better view of the animation
#####
################################# DRAW 3D GRID ################################*
def Draw3DSGrid():

    ## This function was added to give a better feeling of moving around.
    ## A black background just doesn't give it to ya :)  We just draw 100
    ## green lines vertical and horizontal along the X and Z axis.

    ## Turn the lines GREEN
    glColor3ub(0, 255, 0);

    ## Draw a 1x1 grid along the X and Z axis'
    for i in xrange(-50, 50, 1):

      ## Start drawing some lines
      glBegin(GL_LINES)
      ## Do the horizontal lines (along the X)
      glVertex3f(-50, 0, i)
      glVertex3f(50, 0, i)

      ## Do the vertical lines (along the Z)
      glVertex3f(i, 0, -50)
      glVertex3f(i, 0, 50)

      ## Stop drawing lines
      glEnd()
	   
################################# DRAW SPIRAL TOWERS ################################*
#####
#####	This draws a spiral of pyramids to maneuver around
#####
################################# DRAW SPIRAL TOWERS ################################*
def DrawSpiralTowers():

    PI = 3.14159							# Create a constant for PI
    kIncrease = (PI / 16.0)*4					# This is the angle we increase by in radians
    kMaxRotation = (PI * 6)*4					# This is 1080 degrees of rotation in radians (3 circles)
    radius = 40								# We start with a radius of 40 and decrease towards the center

    # Keep looping until we go past the max degrees of rotation (which is 3 full rotations)
    for degree in xrange(0,kMaxRotation,1):

      # Here we use polar coordinates for the rotations, but we swap the y with the z
      x = (float)(radius * cos(degree))			# Get the x position for the current rotation and radius
      z = (float)(radius * sin(degree))			# Get the z position for the current rotation and radius

      # Create a pyramid for every degree in our spiral with a width of 1 and height of 3 
      CreatePyramid(x, 3, z, 1, 3)
		
      # Decrease the radius by the constant amount so the pyramids spirals towards the center
      radius -= 40 / (kMaxRotation / kIncrease)
         
def draw():
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)
    glLoadIdentity()
    # Instead of calling gluLookAt(), we just call our Look() function from
    # the camera object.  This makes it more object oriented, along with
    # cleaning up RenderScene().  If we were building an engine, you would
    # not want to ever explicitely call a specific graphics API.  This allows
    # you to not have to change client code, but only the code underneath
    # when a new API is desired for rendering, or a code addition/bug fix.
    g_Camera.Look()

    # Draw a grid so we can get a good idea of movement around the world		
    Draw3DSGrid()
	
    # Draw the pyramids that spiral in to the center
    DrawSpiralTowers()
     
def handleKeyPress(key):
  global camera_key
    
  if key==K_ESCAPE:
    return 0
  g_Camera.CheckForMovement(key)
  return 1

def main():
  
  w,h = 648,480
  video_flags = OPENGL|DOUBLEBUF|RESIZABLE
    
  pygame.init()
  pygame.display.set_mode((w,h), video_flags)
  pygame.key.set_repeat(10,10) 

  resize((w,h))
  init()
  frames = 0
  done = 0
  ticks = pygame.time.get_ticks() 
  while not done:
    while 1:
      event = pygame.event.poll()
      if event.type == NOEVENT:
        break
      if event.type == KEYDOWN:
        if handleKeyPress(event.key) == 0:
          done = 1
      if event.type == QUIT:
        done = 1
    #Loop the scene
    draw()
    
    # We now just need to call Update() from the camera object to
    # deal with the input and updating.   
    g_Camera.Update(pygame.mouse,w,h)			## Update the camera information
    
    pygame.display.flip()
    frames = frames+1

  print "fps:  %d" % ((frames*1000)/(pygame.time.get_ticks()-ticks))


if __name__ == '__main__': main()
