#!/usr/bin/env python
####################################################################################*
####################################################################################*                       
######	                       CAMERA MODULE
######
######    Federico B. 2003 - dragonscraft@yahoo.it
######
################################# IMPORTS ##########################################*
####################################################################################*


import os
from math import sin, cos, sqrt
from OpenGL.GL import *
from OpenGL.GLU import *
import pygame
from pygame.locals import *
import pygame, pygame.key, pygame.mouse, pygame.time

################################# MODULE ###########################################*

g_FrameInterval = 0.0
framesPerSecond  = 0     ## This will store our fps
lastTime  = 0     ## This will hold the time from the last frame
frameTime = 0          ## This stores the last frame's time


################################# CALCULATE FRAME RATE ################################*
#####
#####   This function calculates the frame rate and time intervals between frames
#####
################################# CALCULATE FRAME RATE ################################*
def CalculateFrameRate():
    global g_FrameInterval, lastTime, frameTime, framesPerSecond

    currentTime = pygame.time.get_ticks()              
    ## Here we store the elapsed time between the current and last frame,
    ## then keep the current frame in our static variable for the next frame.
    g_FrameInterval = float((currentTime - frameTime)/1000.0)
    print g_FrameInterval
    frameTime = currentTime
    ## Increase the frame counter
    framesPerSecond+=1
    ## Now we want to subtract the current time by the last time that was stored
    ## to see if the time elapsed has been over a second, which means we found our FPS.
    if( currentTime - lastTime > 1000 ):    
      ## Here we set the lastTime to the currentTime
      lastTime = currentTime        
      ## Copy the frames per second into a string to display in the window title bar
      strFrameRate = "Current Frames Per Second: %d" % (int(framesPerSecond))
      ## Set the window title bar to our string
      pygame.display.set_caption(strFrameRate,"GameTutorials")
      ## Reset the frames per second
      framesPerSecond = 0

###################################### CVECTOR3 #####################################*
######
###### This  is the vector class
######
###################################### CVECTO #######################################*
class CVector3(object):

  def __init__(self,x,y,z):
    self.x = x
    self.y = y
    self.z = z    
        
###################################### CROSS #######################################*
######
###### This returns a perpendicular vector from 2 given vectors by 
###### taking the cross product.
######
###################################### CROSS #######################################*
def Cross(vVector1, vVector2):

  vNormal = CVector3(0.0,0.0,0.0)
	
  # The X value for the vector is:  (V1.y * V2.z) - (V1.z * V2.y)													## Get the X value
  vNormal.x = ((vVector1.y * vVector2.z) - (vVector1.z * vVector2.y))															
  # The Y value for the vector is:  (V1.z * V2.x) - (V1.x * V2.z)
  vNormal.y = ((vVector1.z * vVector2.x) - (vVector1.x * vVector2.z))															
  # The Z value for the vector is:  (V1.x * V2.y) - (V1.y * V2.x)
  vNormal.z = ((vVector1.x * vVector2.y) - (vVector1.y * vVector2.x))
  # Return the cross product
  return vNormal									 

####################################### MAGNITUDE ###################################*
#####
##### This returns the magnitude of a vector
#####
####################################### MAGNITUDE ###################################*
def Magnitude(vNormal):

  return (float(sqrt( (vNormal.x * vNormal.x) + 
					  (vNormal.y * vNormal.y) + 
					  (vNormal.z * vNormal.z) )))

####################################### NORMALIZE ###################################*
#####
#####	This returns a normalize vector (A vector exactly of length 1)
#####
####################################### NORMALIZE ###################################*
def Normalize(vVector):

  ## Get the magnitude of our normal
  magnitude = Magnitude(vVector)				
  vVector.x /= magnitude		
  vVector.y /= magnitude		
  vVector.z /= magnitude				
  ## Finally, return our normalized vector
  return vVector										

####################################### OPERATIONS ###################################*
#####
##### theese are others varius operations, they returns vectors
#####
####################################### OPERATIONS ###################################*  
def Add(v1, v2):
  return CVector3(v1.x+v2.x, v1.y+v2.y, v1.z+v2.z)

def Subtract(v1, v2):
  return CVector3(v1.x-v2.x, v1.y-v2.y, v1.z-v2.z)

def Multiply(v1, m):
  return CVector3(v1.x*m, v1.y*m, v1.z*m)

def Divide( v1, m):
  return CVector3(v1.x/m, v1.y/m, v1.z/m)    
	
################################# CCAMERA ############################################*
######
###### This is the class constructor
######
################################# CCAMERA ############################################*
class CCamera(object):
  
  def __init__(self):
    vZero = CVector3(0.0, 0.0, 0.0)
    vView = CVector3(0.0, 1.0, 0.5)
    vUp   = CVector3(0.0, 0.0, 0.0)
    
    self.m_vPosition = vZero      # Init the position to zero
    self.m_vView = vView	  # Init the view to a std starting view
    self.m_vUpVector = vUp	  # Init the UpVector
	
  ############################## POSITION CAMERA ############################*
  ######
  ###### This function sets the camera's position and view and up vector.
  ######
  ############################## POSITION CAMERA ############################*	
  def PositionCamera( self, positionX, positionY, positionZ,
                            viewX,     viewY,     viewZ,
                            upVectorX, upVectorY, upVectorZ):
							
    vPosition	= CVector3(positionX, positionY, positionZ)
    vView	= CVector3(viewX, viewY, viewZ)
    vUpVector	= CVector3(upVectorX, upVectorY, upVectorZ)
    # The code above just makes it cleaner to set the variables.
    # Otherwise we would have to set each variable x y and z.
    self.m_vPosition = vPosition		# Assign the position
    self.m_vView     = vView     		# Assign the view
    self.m_vUpVector = vUpVector		# Assign the up vector
	
  ############################ MOVE CAMERA ##################################*
  ######
  ###### This will move the camera forward or backward depending on the speed
  ######
  ############################ MOVE CAMERA ##################################*
  def MoveCamera(self, speed):
  
    vVector = CVector3(0.0,0.0,0.0)			# Init a vector for our view
    # Get our view vector (The direciton we are facing)
    vVector.x = self.m_vView.x - self.m_vPosition.x   # This gets the direction of the X	
    vVector.y = self.m_vView.y - self.m_vPosition.y   # This gets the direction of the Y
    vVector.z = self.m_vView.z - self.m_vPosition.z   # This gets the direction of the Z
    self.m_vPosition.x += vVector.x * speed	      # Add our acceleration to our position's X
    self.m_vPosition.z += vVector.z * speed	      # Add our acceleration to our position's Z
    self.m_vView.x += vVector.x * speed		      # Add our acceleration to our view's X
    self.m_vView.z += vVector.z * speed		      # Add our acceleration to our view's Z
    
  ############################## ROTATE VIEW ##################################*
  ######
  ###### This rotates the view around the position using an axis-angle rotation
  ######
  ############################## ROTATE VIEW ##################################*
  def RotateView(self, angle, x, y, z):

	vNewView = CVector3(0.0,0.0,0.0)
	vView    = CVector3(0.0,0.0,0.0)	
	# Get our view vector (The direciton we are facing)
	vView.x = self.m_vView.x - self.m_vPosition.x		# This gets the direction of the X	
	vView.y = self.m_vView.y - self.m_vPosition.y		# This gets the direction of the Y
	vView.z = self.m_vView.z - self.m_vPosition.z		# This gets the direction of the Z
	# Calculate the sine and cosine of the angle once
	cosTheta = (float(cos(angle)))
	sinTheta = (float(sin(angle)))
	# Find the new x position for the new rotated point
	vNewView.x  = (cosTheta + (1 - cosTheta) * x * x)	* vView.x
	vNewView.x += ((1 - cosTheta) * x * y - z * sinTheta)	* vView.y
	vNewView.x += ((1 - cosTheta) * x * z + y * sinTheta)	* vView.z
	# Find the new y position for the new rotated point
	vNewView.y  = ((1 - cosTheta) * x * y + z * sinTheta)	* vView.x
	vNewView.y += (cosTheta + (1 - cosTheta) * y * y)	* vView.y
	vNewView.y += ((1 - cosTheta) * y * z - x * sinTheta)	* vView.z
	# Find the new z position for the new rotated point
	vNewView.z  = ((1 - cosTheta) * x * z - y * sinTheta)	* vView.x
	vNewView.z += ((1 - cosTheta) * y * z + x * sinTheta)	* vView.y
	vNewView.z += (cosTheta + (1 - cosTheta) * z * z)	* vView.z
	# Now we just add the newly rotated vector to our position to set
	# our new rotated view of our camera.
	self.m_vView.x = self.m_vPosition.x + vNewView.x
	self.m_vView.y = self.m_vPosition.y + vNewView.y
	self.m_vView.z = self.m_vPosition.z + vNewView.z
	
  ############################## ROTATE AROUND POINT ##########################*
  #####
  #####	This rotates the position around a given point
  #####
  ############################## ROTATE AROUND POINT ##########################*
  def RotateAroundPoint(self, vCenter, angle, x, y, z):

	vNewPosition = CVector3(0.0,0.0,0.0)			
	# Get the vVector from our position to the center we are rotating around
	vPos = Subtract(self.m_vPosition , vCenter)
	# Calculate the sine and cosine of the angle once
	cosTheta = (float(cos(angle)))
	sinTheta = (float(sin(angle)))
	# Find the new x position for the new rotated point
	vNewPosition.x  = (cosTheta + (1 - cosTheta) * x * x)		* vPos.x
	vNewPosition.x += ((1 - cosTheta) * x * y - z * sinTheta)	* vPos.y
	vNewPosition.x += ((1 - cosTheta) * x * z + y * sinTheta)	* vPos.z
	# Find the new y position for the new rotated point
	vNewPosition.y  = ((1 - cosTheta) * x * y + z * sinTheta)	* vPos.x
	vNewPosition.y += (cosTheta + (1 - cosTheta) * y * y)		* vPos.y
	vNewPosition.y += ((1 - cosTheta) * y * z - x * sinTheta)	* vPos.z
	# Find the new z position for the new rotated point
	vNewPosition.z  = ((1 - cosTheta) * x * z - y * sinTheta)	* vPos.x
	vNewPosition.z += ((1 - cosTheta) * y * z + x * sinTheta)	* vPos.y
	vNewPosition.z += (cosTheta + (1 - cosTheta) * z * z)		* vPos.z
	# Now we just add the newly rotated vector to our position to set
	# our new rotated position of our camera.
	self.m_vPosition = Add(vCenter , vNewPosition)
	
  ################################# SET VIEW BY MOUSE ################################*
  #####
  #####	This allows us to look around using the mouse, like in most first person games.
  #####
  ################################# SET VIEW BY MOUSE ################################*
  def SetViewByMouse(self, screenWidth, screenHeight, mouseObj):

    middleX = screenWidth  >> 1			# This is a binary shift to get half the width
    middleY = screenHeight >> 1			# This is a binary shift to get half the height
    angleY = 0.0						# This is the direction for looking up or down
    angleZ = 0.0						# This will be the value we need to rotate around the Y axis (Left and Right)
    currentRotX = 0.0
       
  # Get the mouse's current X,Y position
    mouseX,mouseY = mouseObj.get_pos()
  # If our cursor is still in the middle, we never moved... so don't update the screen
    if( (mouseX == middleX) & (mouseY == middleY) ):
      return
  # Set the mouse position to the middle of our window
    mouseObj.set_pos(middleX, middleY)                            
  # Get the direction the mouse moved in, but bring the number down to a reasonable amount
    angleY = (float( middleX - mouseX ) / 1000.0 )       
    angleZ = (float( middleY - mouseY ) / 1000.0 )       
  # Here we keep track of the current rotation (for up and down) so that
  # we can restrict the camera from doing a full 360 loop.
    currentRotX -= angleZ  
  # If the current rotation (in radians) is greater than 1.0, we want to cap it.
    if(currentRotX > 1.0):
      currentRotX = 1.0
  # Check if the rotation is below -1.0, if so we want to make sure it doesn't continue
    elif(currentRotX < -1.0):
      currentRotX = -1.0
  # Otherwise, we can rotate the view around our position
    else:
      vAxis = Cross(Subtract(self.m_vView, self.m_vPosition), self.m_vUpVector)
      vAxis = Normalize(vAxis)
    # Rotate around our perpendicular axis and along the y-axis
      self.RotateView(angleZ, vAxis.x, vAxis.y, vAxis.z)
      self.RotateView(angleY, 0, 1, 0)
	
  ################################# STRAFE CAMERA ###############################*
  #####
  #####	This strafes the camera left and right depending on the speed (-#+)
  #####
  ################################# STRAFE CAMERA ###############################*
  def StrafeCamera(self, speed):
			
    # Add the strafe vector to our position
    self.m_vPosition.x += self.m_vStrafe.x * speed
    self.m_vPosition.z += self.m_vStrafe.z * speed
    # Add the strafe vector to our view
    self.m_vView.x += self.m_vStrafe.x * speed
    self.m_vView.z += self.m_vStrafe.z * speed

  # The next 3 functions were added to our camera class.  The less code in 
  # Main.cpp the better.

  ############################ CHECK FOR MOVEMENT ############################*
  #####
  #####	This function handles the input faster than in the WinProc()
  #####
  ############################ CHECK FOR MOVEMENT ############################*
  def CheckForMovement(self, key):
    global g_FrameInterval
    
    kSpeed = 15.0
    speed = (kSpeed * g_FrameInterval)
  
    # Check if we hit the Up arrow or the 'w' key
    if(key==K_UP ):				
    # Move our camera forward by a positive SPEED
      self.MoveCamera(speed)				
    # Check if we hit the Down arrow or the 's' key
    elif(key==K_DOWN ):			
      # Move our camera backward by a negative SPEED
      self.MoveCamera(-speed)				
    # Check if we hit the Left arrow or the 'a' key
    elif(key==K_LEFT ):			
      # Strafe the camera left
      self.StrafeCamera(-speed)
    # Check if we hit the Right arrow or the 'd' key
    elif(key==K_RIGHT ):		
      # Strafe the camera right
      self.StrafeCamera(speed)

  ################################# UPDATE ################################*
  #####
  #####	This updates the camera's view and strafe vector
  #####
  ################################# UPDATE ################################*
  def Update(self, screen_w,screen_h, mouse):

    # Initialize a variable for the cross product result
    vCross = Cross(Subtract(self.m_vView , self.m_vPosition), self.m_vUpVector)
    # Normalize the strafe vector
    self.m_vStrafe = Normalize(vCross)
    # Move the camera's view by the mouse
    self.SetViewByMouse(screen_w, screen_h, mouse)
    # This checks to see if the keyboard was pressed
    #self.CheckForMovement(key)
    	
    # We stick this function in our camera source file so that we can move
    # time base movement and camera functionality together between tutorials
    # relatively easyly.     
    # We calculate our frame rate and set our frame interval for time based movement
    CalculateFrameRate()

  ################################# LOOK ################################*
  #####
  #####	This updates the camera according to the 
  #####
  ################################# LOOK ################################*
  def Look(self):

	# Give openGL our camera position, then camera view, then camera up vector
	gluLookAt(self.m_vPosition.x, self.m_vPosition.y, self.m_vPosition.z,	
		  self.m_vView.x,	 self.m_vView.y,     self.m_vView.z,	
		  self.m_vUpVector.x, self.m_vUpVector.y, self.m_vUpVector.z)




	
