#!/usr/bin/env python

##########################################################################*                       
######	                    GAME TUTORIAL 21 MODULE
######
######    Federico B. 2003 - dragonscraft@yahoo.it
######
################################# IMPORTS ################################*

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame
from pygame.locals import *
import pygame, pygame.key, pygame.display, pygame.mouse

############################# LOCAL IMPORTS ##############################*

from Camera import *

################################# MODULE #################################*

BACK_ID  = 0                          # The texture ID for the back side of the cube
FRONT_ID = 1                          # The texture ID for the front side of the cube
BOTTOM_ID= 2                          # The texture ID for the bottom side of the cube
TOP_ID   = 3                          # The texture ID for the top side of the cube
LEFT_ID  = 4                          # The texture ID for the left side of the cube
RIGHT_ID = 5                          # The texture ID for the right side of the cube

g_Texture=[]

g_Camera = CCamera()

#//////////////////////////////// CREATE SKY BOX \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
#////
#////    This creates a box centered around X Y Z.  Instead of colors at
#////   each vertices there is texture maps for each side of the box to 
#////   create the illusion of the sky box.  You can set the width, height and length
#////   of the sky box to create different perspectives for different sky box textures.
#////
#//////////////////////////////// CREATE SKY BOX \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

def CreateSkyBox( x, y, z, width, height, length):
    global g_Texture

    # This is the most important function of this tutorial.  This function
    # used to just create a silly colored cube in the RotateCube tutorial,
    # but now it creates something beautiful.  You'll notice we added
    # some more parameters to the function.  This way we can change the perspective
    # of the sky box.  It doesn't really look good if it's a perfect cube.  Some
    # textures look better at different ratios.  We assign the sky box textures
    # to each side of the box creating the illusion of a detailed 3D world.
    # You will notice I had to change the texture coordinates for every one
    # to be flipped correctly.  Also, depending on your culling, the vertex
    # order might need to be changed around.  I don't use culling in this tutorial
    # so it will work fine here, but be sure to remember this if you do.

    # Bind the BACK texture of the sky map to the BACK side of the cube
    glBindTexture(GL_TEXTURE_2D, g_Texture[BACK_ID])

    # Since we want the sky box to be centered around X, Y, and Z for ease,
    # we do a little math to accomplish this.  We just change the X, Y and Z
    # to perform this task.  If we just minus half the width, height and length
    # from x, y and z it will give us the desired result.  Now when we create the
    # box it will center it around (x, y, z)

    # This centers the sky box around (x, y, z)
    x = x - width  / 2
    y = y - height / 2
    z = z - length / 2

    # Start drawing the side as a QUAD
    glBegin(GL_QUADS)        
        
    # Assign the texture coordinates and vertices for the BACK Side
    glTexCoord2f(0.0, 0.0)
    glVertex3f(x,  y, z)
    glTexCoord2f(0.0, 1.0)
    glVertex3f(x,  y + height, z)
    glTexCoord2f(1.0, 1.0)
    glVertex3f(x + width, y + height, z) 
    glTexCoord2f(1.0, 0.0)
    glVertex3f(x + width, y, z)
    glEnd()

    # Bind the FRONT texture of the sky map to the FRONT side of the box
    glBindTexture(GL_TEXTURE_2D, g_Texture[FRONT_ID])

    # Start drawing the side as a QUAD
    glBegin(GL_QUADS)    
    
    # Assign the texture coordinates and vertices for the FRONT Side
    glTexCoord2f(1.0, 0.0)
    glVertex3f(x, y, z + length)
    glTexCoord2f(1.0, 1.0)
    glVertex3f(x, y + height, z + length)
    glTexCoord2f(0.0, 1.0)
    glVertex3f(x + width, y + height, z + length) 
    glTexCoord2f(0.0, 0.0)
    glVertex3f(x + width, y, z + length)
    glEnd()

    # Bind the BOTTOM texture of the sky map to the BOTTOM side of the box
    glBindTexture(GL_TEXTURE_2D, g_Texture[BOTTOM_ID])

    # Start drawing the side as a QUAD
    glBegin(GL_QUADS)        
    
    # Assign the texture coordinates and vertices for the BOTTOM Side
    glTexCoord2f(1.0, 0.0)
    glVertex3f(x,  y,  z)
    glTexCoord2f(1.0, 1.0)
    glVertex3f(x, y, z + length)
    glTexCoord2f(0.0, 1.0)
    glVertex3f(x + width, y, z + length)
    glTexCoord2f(0.0, 0.0)
    glVertex3f(x + width, y, z)
    glEnd()

    # Bind the TOP texture of the sky map to the TOP side of the box
    glBindTexture(GL_TEXTURE_2D, g_Texture[TOP_ID])

    # Start drawing the side as a QUAD
    glBegin(GL_QUADS);        
        
    # Assign the texture coordinates and vertices for the TOP Side
    glTexCoord2f(1.0, 1.0)
    glVertex3f(x,  y + height, z)
    glTexCoord2f(1.0, 0.0)
    glVertex3f(x, y + height, z + length)
    glTexCoord2f(0.0, 0.0)
    glVertex3f(x + width, y + height, z + length) 
    glTexCoord2f(0.0, 1.0)
    glVertex3f(x + width, y + height, z)
    glEnd()

    # Bind the LEFT texture of the sky map to the LEFT side of the box
    glBindTexture(GL_TEXTURE_2D, g_Texture[LEFT_ID])

    # Start drawing the side as a QUAD
    glBegin(GL_QUADS)       
        
    # Assign the texture coordinates and vertices for the LEFT Side
    glTexCoord2f(1.0, 0.0)
    glVertex3f(x, y, z)
    glTexCoord2f(0.0, 0.0)
    glVertex3f(x,y, z + length)
    glTexCoord2f(0.0, 1.0)
    glVertex3f(x, y + height, z + length) 
    glTexCoord2f(1.0, 1.0) 
    glVertex3f(x, y + height,  z)
    glEnd()

    # Bind the RIGHT texture of the sky map to the RIGHT side of the box
    glBindTexture(GL_TEXTURE_2D, g_Texture[RIGHT_ID])

    # Start drawing the side as a QUAD
    glBegin(GL_QUADS)        

    # Assign the texture coordinates and vertices for the RIGHT Side
    glTexCoord2f(0.0, 0.0)
    glVertex3f(x + width, y, z)
    glTexCoord2f(1.0, 0.0)
    glVertex3f(x + width, y,  z + length)
    glTexCoord2f(1.0, 1.0)
    glVertex3f(x + width, y + height, z + length)
    glTexCoord2f(0.0, 1.0)
    glVertex3f(x + width, y + height, z)
    glEnd()
    
def loadtextures():
  global g_Texture
  texturefile=[]
  textureSurface=[]
  textureData=[]
    
  texturefile.append(os.path.join('data','Back.bmp'))
  texturefile.append(os.path.join('data','Front.bmp'))
  texturefile.append(os.path.join('data','Bottom.bmp'))
  texturefile.append(os.path.join('data','Top.bmp'))
  texturefile.append(os.path.join('data','Left.bmp'))
  texturefile.append(os.path.join('data','Right.bmp'))
    
  for image_n in range(6):								
    textureSurface.append(pygame.image.load(texturefile[image_n]))
    textureData.append(pygame.image.tostring(textureSurface[image_n], "RGBX", 1))
  
  g_Texture = glGenTextures(6)
 
  for texture_n in range(6):
    glBindTexture(GL_TEXTURE_2D, g_Texture[texture_n])

    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST )
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR )
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE)
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP)
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP)  
    
    gluBuild2DMipmaps( GL_TEXTURE_2D, GL_RGBA, textureSurface[texture_n].get_width(),
                       textureSurface[texture_n].get_height(),GL_RGBA, GL_UNSIGNED_BYTE,
                       textureData[texture_n])         
 
def resize((width, height)):
    if height==0:
        height=1
    glViewport(0, 0, width, height)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(45, 1.0*width/height, 0.5, 600.0)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

def init():

    glClearColor(0.0, 0.0, 0.0, 0.0)
    glEnable(GL_TEXTURE_2D)
    glEnable(GL_DEPTH_TEST)
    loadtextures()  
    # Below we call our init function for our camera.  We pass in the
    # starting position of our last tutorial where we are looking dead
    # on at the color triangle.  I added .5 to the Y since the last tutorial.
    # Remember, we only need to init this stuff ONCE, then we never call this again.
			  # Position     View         Up Vector
    g_Camera.PositionCamera(0, 0, 0,   0, 0, 6,   0, 1, 0)

def draw():
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)
    glLoadIdentity()
    # Now instead of passing in hard coded values, we give gluLookAt()
    # the values that our global camera has.  Now we can check if we
    # hit the LEFT RIGHT UP and DOWN arrow keys, then update the camera.
    # It will automatically update right here.  Cool huh?

    # Give openGL our camera position, then camera view, then camera up vector
    g_Camera.Look()
              
    # Here we create our sky box.  It will be centered around (0, 0, 0) with
    # a width of 400, height of 200 and length of 400.  You might wonder why we
    # don't make it a perfect cube?  Well, if we make the height half the size
    # of the width and length it looks better.  Try putting the height to 400 and
    # you will notice that everything looks really close up.  You will want to tweak
    # this ratio most likely for every sky maps you use.  Some look better at different
    # perspectives and ratios.  Try changing the height to 100.  It looks more flat
    # and at a greater distance.  

    # Draw the sky box centered at (0, 0, 0)
    CreateSkyBox(0, 0, 0, 400, 200, 400)           

def handleKeyPress(key):
    
  if key==K_ESCAPE:
    return 0
  g_Camera.CheckForMovement(key)
  return 1

def main():

  video_flags = OPENGL|DOUBLEBUF|RESIZABLE
    
  pygame.init()
  pygame.display.set_mode((640,480), video_flags)
  pygame.key.set_repeat(10,10) 

  resize((640,480))
  init()
  frames = 0
  done = 0
  ticks = pygame.time.get_ticks() 
  while not done:
    while 1:
      event = pygame.event.poll()
      if event.type == NOEVENT:
        break
      if event.type == KEYDOWN:
        if handleKeyPress(event.key) == 0:
          done = 1
      if event.type == QUIT:
        done = 1
    #Loop the scene
    draw()
    # Here we check for mouse and key movements every frame
    g_Camera.Update(640,480,pygame.mouse)  # Move the camera by the mouse
    pygame.display.flip()
    frames = frames+1

  print "fps:  %d" % ((frames*1000)/(pygame.time.get_ticks()-ticks))


if __name__ == '__main__': main()
