#!/usr/bin/env python

##########################################################################*                       
######	                    GAME TUTORIAL 28 MODULE
######
######    Federico B. 2003 - dragonscraft@yahoo.it
######
################################# IMPORTS ################################*

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame
from pygame.locals import *
import pygame, pygame.key, pygame.display, pygame.mouse

############################# LOCAL IMPORTS ##############################*

from Math3D import *
################################# MODULE #################################*

				    # Left Point             Right Point             Top point
vTriangle = [CVector3(-1.0, 0.0, 0.0), CVector3(1.0, 0.0, 0.0),	CVector3(0.0, 1.0, 0.0)]

def resize((width, height)):
    if height==0:
        height=1
    glViewport(0, 0, width, height)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(45, 1.0*width/height, 1.0, 150.0)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

def init():
    glClearColor(0.0, 0.0, 0.0, 0.0)
    glEnable(GL_TEXTURE_2D)	
    glEnable(GL_DEPTH_TEST)

def draw():
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)  # Clear The Screen And The Depth Buffer
    glLoadIdentity()									# Reset The matrix
	
    # We changed the position of the camera so we could better see the normal at an angle. 
    # This positions the camera off to the left a bit and in front of the triangle.
    # We move our view up a bit so fully see the triangle instead looking down

	    # 	    Position          View	     Up Vector
    gluLookAt(-2.5, 0, -0.5,  0, 0.5, 0,   0, 1, 0)	# This determines where the camera's position and view is
	
    # Below we draw the triangle like normal, except we pass in our vTriangle structure.
    # This is so we can move the triangle around and now just have it constant.
    # As we move the triangle around, the "normal" will follow it perfectly.

    glBegin (GL_TRIANGLES)								# This is our BEGIN to draw

    glColor3ub(255, 0, 0)							# Make the Left vertex RED
    glVertex3f(vTriangle[0].x, vTriangle[0].y, vTriangle[0].z)

    glColor3ub(0, 255, 0)							# Make the Right vertex GREEN
    glVertex3f(vTriangle[1].x, vTriangle[1].y, vTriangle[1].z)

    glColor3ub(0, 0, 255)							# Make the Top vertex BLUE
    glVertex3f(vTriangle[2].x, vTriangle[2].y, vTriangle[2].z)
    glEnd()											# This is the END of drawing

    # Now that we displayed the triangle, let's display a line coming from the
    # triangle to show you visually what the normal looks like.  Remember, the
    # normal does NOT have a position, just a direction, but just to help us visualize it,
    # we will draw the normal coming from the middle of the triangle.

    # Get the normal of our triangle by passing in our array or points.
    vNormal = Normal(vTriangle)

    # Now, just for show, let's find the center of the triangle.
    # We know that the middle X is zero, so we don't need to calculate the X value.
    vCenter = CVector3(0.0,0.0,0.0)

    # To get the Y value, we just divide the top point's Y value by 2, since we start at 0.
    # Do the same thing for the Z value, just divide the Z value of the top point by 2
    # because we start out at 0 on the Z axis, so only the top of the triangle will be moving.
    # If we weren't so conveniently positioned we would have to do something more modular.
    # To find the center of an object you just add up all the X Y and Z values singularly,
    # then divide each one by the number of vertices, and that gives you the center.
    # IE, after you get the total:  total.x /= totalVertices;  total.y /= totalVertices; total.z /= totalVertices;
    # I decided not to make another function for this for simplicity reasons, but you should :)

    vCenter.y = float(vTriangle[2].y / 2.0 )				# Get the Y value for the center of the triangle
    vCenter.z = float(vTriangle[2].z / 2.0 )					# Get the Z value for the center of the triangle.

    # Now let's draw the line coming from the center of our triangle.
    # The first point will be the center point of the triangle that we just calculated.
    # The next point will be the center point + the normal of the triangle.

    glBegin (GL_LINES)							# This is our BEGIN to draw

    glColor3ub(255, 255, 0)						# Make the normal line YELLOW
    glVertex3f(vCenter.x, vCenter.y, vCenter.z)	# Let's draw the normal centered on the triangle
												# Draw the normal of the polygon from the center of the polygon to better see it
    glVertex3f(vCenter.x + vNormal.x, vCenter.y + vNormal.y, vCenter.z + vNormal.z)
    glEnd()										# This is the END of drawing
    
def handleKeyPress(key):
  global vTriangle
    
  if key==K_ESCAPE:
    return 0
  if key==K_RIGHT:
  	vTriangle[2].z += 0.01				# Move the top of the triangle forward
	vTriangle[2].y += 0.01				# Move the top of the triangle up
  if key==K_LEFT:
  	vTriangle[2].z -= 0.01				# Move the top of the triangle forward
	vTriangle[2].y -= 0.01				# Move the top of the triangle up
  return 1

def main():

  video_flags = OPENGL|DOUBLEBUF|RESIZABLE
    
  pygame.init()
  pygame.display.set_mode((640,480), video_flags)
  pygame.key.set_repeat(10,10) 

  resize((640,480))
  init()
  frames = 0
  done = 0
  ticks = pygame.time.get_ticks() 
  while not done:
    while 1:
      event = pygame.event.poll()
      if event.type == NOEVENT:
        break
      if event.type == KEYDOWN:
        if handleKeyPress(event.key) == 0:
          done = 1
      if event.type == QUIT:
        done = 1
    #Loop the scene
    draw()
    # Here we check for mouse and key movements every frame
    pygame.display.flip()
    frames = frames+1

  print "fps:  %d" % ((frames*1000)/(pygame.time.get_ticks()-ticks))


if __name__ == '__main__': main()
