#!/usr/bin/env python
####################################################################################*
####################################################################################*                       
######	                       MATH3D MODULE
######
######    Federico B. 2003 - dragonscraft@yahoo.it
######
################################# IMPORTS ##########################################*
####################################################################################*

import os
from math import sin, cos, sqrt
from OpenGL.GL import *
from OpenGL.GLU import *
import pygame
from pygame.locals import *
import pygame, pygame.key, pygame.mouse, pygame.time

###################################### CVECTOR3 #####################################*
######
###### This  is the vector class
######
###################################### CVECTOR#######################################*
class CVector3(object):

  def __init__(self,x,y,z):
    self.x = x
    self.y = y
    self.z = z    
        
###################################### CROSS #######################################*
######
###### This returns a perpendicular vector from 2 given vectors by 
###### taking the cross product.
######
###################################### CROSS #######################################*
def Cross(vVector1, vVector2):

  vNormal = CVector3(0.0,0.0,0.0)
	
  # The X value for the vector is:  (V1.y * V2.z) - (V1.z * V2.y)													## Get the X value
  vNormal.x = ((vVector1.y * vVector2.z) - (vVector1.z * vVector2.y))															
  # The Y value for the vector is:  (V1.z * V2.x) - (V1.x * V2.z)
  vNormal.y = ((vVector1.z * vVector2.x) - (vVector1.x * vVector2.z))															
  # The Z value for the vector is:  (V1.x * V2.y) - (V1.y * V2.x)
  vNormal.z = ((vVector1.x * vVector2.y) - (vVector1.y * vVector2.x))
  # Return the cross product
  return vNormal	
  
####################################### VECTOR ###########################################*
#####
#####	This returns a vector between 2 points
#####
####################################### VECTOR ###########################################*

def Vector(vPoint1, vPoint2):

		vVector = CVector3(0,0,0)					# Initialize our variable to zero

		# In order to get a vector from 2 points (a direction) we need to
		# subtract the second point from the first point.

		vVector.x = vPoint1.x - vPoint2.x			# Get the X value of our new vector
		vVector.y = vPoint1.y - vPoint2.y			# Get the Y value of our new vector
		vVector.z = vPoint1.z - vPoint2.z			# Get the Z value of our new vector

		# Now that we have our new vector between the 2 points, we will return it.

		return vVector								# Return our new vector								 

####################################### MAGNITUDE ###################################*
#####
##### This returns the magnitude of a vector
#####
####################################### MAGNITUDE ###################################*
def Magnitude(vNormal):

  return (float(sqrt( (vNormal.x * vNormal.x) + 
					  (vNormal.y * vNormal.y) + 
					  (vNormal.z * vNormal.z) )))

####################################### NORMALIZE ###################################*
#####
#####	This returns a normalize vector (A vector exactly of length 1)
#####
####################################### NORMALIZE ###################################*
def Normalize(vVector):

  ## Get the magnitude of our normal
  magnitude = Magnitude(vVector)				
  vVector.x /= magnitude		
  vVector.y /= magnitude		
  vVector.z /= magnitude				
  ## Finally, return our normalized vector
  return vVector										

####################################### NORMAL #####################################*
#####
#####	This returns the normal of a polygon (The direction the polygon is facing)
#####
####################################### NORMAL #####################################*

def Normal(vTriangle):					# You might want to error check to make sure it's valid
									    # Get 2 vectors from the polygon
	vVector1 = Vector(vTriangle[2], vTriangle[0])
	vVector2 = Vector(vTriangle[1], vTriangle[0])

	# We Chose to get the vectors surrounding the first point of the polygon.
	# We could have chosen to get 2 other sides of the triangle, but we chose these 2.
	# Now that we have the 2 side vectors, we will take their cross product.
	# (*NOTE*) It is important that pass in the vector of the bottom side of the triangle
	# first, and then pass in the vector of the left side second.  If we switch them,
	# it will turn the normal the opposite way.  Try it, switch them like this: Cross(vVector2, vVector1);
	# Like I said before, it's important to ALWAYS work in the same direction.  In our case,
	# we chose that we always work counter-clockwise.

	vNormal = Cross(vVector1, vVector2)

	# Now that we have the direction of the normal, we want to do one last thing.
	# Right now, it's an unknown length, it is probably pretty long in length.
	# We want to do something which gives the normal a length of 1.  This is called
	# normalizing.  To do this we divide the normal by it's magnitude.  Well how do we
	# find it's magnitude? We use this equation: magnitude = sqrt(x^2 + y^2 + z^2)

	vNormal = Normalize(vNormal)						# Use our function we created to normalize the normal (Makes it a length of one)

	# Now return the normalized normal
	# (*NOTE*) If you want to understand what normalizing our normal does, comment out
	# the above line.  Then you can see how long the normal is before we normalize it.
	# I strongly recommend this.  And remember, it doesn't matter how long the normal is,
	# (of course besides (0, 0, 0)), if we normalize it, it will always be of length 1 afterwards.

	return vNormal										# Return our normal at our desired length
  
####################################### OPERATIONS ###################################*
#####
##### theese are others varius operations, they returns vectors
#####
####################################### OPERATIONS ###################################*  
def Add(v1, v2):
  return CVector3(v1.x+v2.x, v1.y+v2.y, v1.z+v2.z)

def Subtract(v1, v2):
  return CVector3(v1.x-v2.x, v1.y-v2.y, v1.z-v2.z)

def Multiply(v1, m):
  return CVector3(v1.x*m, v1.y*m, v1.z*m)

def Divide( v1, m):
  return CVector3(v1.x/m, v1.y/m, v1.z/m)    