#!/usr/bin/env python
####################################################################################*
####################################################################################*                       
######	                       MATH3D MODULE
######
######    Federico B. 2003 - dragonscraft@yahoo.it
######
################################# IMPORTS ##########################################*
####################################################################################*

import os
from math import sin, cos, sqrt
from OpenGL.GL import *
from OpenGL.GLU import *
import pygame
from pygame.locals import *
import pygame, pygame.key, pygame.mouse, pygame.time

###################################### CVECTOR3 #####################################*
######
###### This  is the vector class
######
###################################### CVECTOR#######################################*
class CVector3(object):

  def __init__(self,x,y,z):
    self.x = x
    self.y = y
    self.z = z    
        
###################################### CROSS #######################################*
######
###### This returns a perpendicular vector from 2 given vectors by 
###### taking the cross product.
######
###################################### CROSS #######################################*
def Cross(vVector1, vVector2):

  vNormal = CVector3(0.0,0.0,0.0)
	
  # The X value for the vector is:  (V1.y * V2.z) - (V1.z * V2.y)													## Get the X value
  vNormal.x = ((vVector1.y * vVector2.z) - (vVector1.z * vVector2.y))															
  # The Y value for the vector is:  (V1.z * V2.x) - (V1.x * V2.z)
  vNormal.y = ((vVector1.z * vVector2.x) - (vVector1.x * vVector2.z))															
  # The Z value for the vector is:  (V1.x * V2.y) - (V1.y * V2.x)
  vNormal.z = ((vVector1.x * vVector2.y) - (vVector1.y * vVector2.x))
  # Return the cross product
  return vNormal	
  
####################################### VECTOR ###########################################*
#####
#####	This returns a vector between 2 points
#####
####################################### VECTOR ###########################################*

def Vector(vPoint1, vPoint2):

		vVector = CVector3(0,0,0)					# Initialize our variable to zero

		# In order to get a vector from 2 points (a direction) we need to
		# subtract the second point from the first point.

		vVector.x = vPoint1.x - vPoint2.x			# Get the X value of our new vector
		vVector.y = vPoint1.y - vPoint2.y			# Get the Y value of our new vector
		vVector.z = vPoint1.z - vPoint2.z			# Get the Z value of our new vector

		# Now that we have our new vector between the 2 points, we will return it.

		return vVector								# Return our new vector								 

####################################### MAGNITUDE ###################################*
#####
##### This returns the magnitude of a vector
#####
####################################### MAGNITUDE ###################################*
def Magnitude(vNormal):

  return (float(sqrt( (vNormal.x * vNormal.x) + 
					  (vNormal.y * vNormal.y) + 
					  (vNormal.z * vNormal.z) )))

####################################### NORMALIZE ###################################*
#####
#####	This returns a normalize vector (A vector exactly of length 1)
#####
####################################### NORMALIZE ###################################*
def Normalize(vVector):

  ## Get the magnitude of our normal
  magnitude = Magnitude(vVector)				
  vVector.x /= magnitude		
  vVector.y /= magnitude		
  vVector.z /= magnitude				
  ## Finally, return our normalized vector
  return vVector										

####################################### NORMAL #####################################*
#####
#####	This returns the normal of a polygon (The direction the polygon is facing)
#####
####################################### NORMAL #####################################*
def Normal(vTriangle):					# You might want to error check to make sure it's valid
									    # Get 2 vectors from the polygon
	vVector1 = Vector(vTriangle[2], vTriangle[0])
	vVector2 = Vector(vTriangle[1], vTriangle[0])

	vNormal = Cross(vVector1, vVector2)
	vNormal = Normalize(vNormal)						# Use our function we created to normalize the normal (Makes it a length of one)
	return vNormal										# Return our normal at our desired length
 
#################################### PLANE DISTANCE ######################################*
#####
#####	This returns the distance between a plane and the origin
#####
#################################### PLANE DISTANCE ######################################*									
def PlaneDistance(Normal, Point):
	
	distance = 0.0									# This variable holds the distance from the plane tot he origin

	# Use the plane equation to find the distance (Ax + By + Cz + D = 0)  We want to find D.
	# For more information about the plane equation, read about it in the function below (IntersectedPlane())
	# Basically, A B C is the X Y Z value of our normal and the x y z is our x y z of our point.
	# D is the distance from the origin.  So, we need to move this equation around to find D.
	# We come up with D = -(Ax + By + Cz)
													# Basically, the negated dot product of the normal of the plane and the point. (More about the dot product in another tutorial)
	distance = - ((Normal.x * Point.x) + (Normal.y * Point.y) + (Normal.z * Point.z))

	return distance									# Return the distance

################################### INTERSECTED PLANE ######################################*
#####
#####	This checks to see if a line intersects a plane
#####
################################### INTERSECTED PLANE ######################################*
											
def IntersectedPlane(vTriangle, vLine):
	distance1=0.0
	distance2=0.0		# The distances from the 2 points of the line from the plane
			
	vNormal = Normal(vTriangle)			# We need to get the normal of our plane to go any further

	# Now that we have the normal, we need to calculate the distance our triangle is from the origin.
	# Since we would have the same triangle, but -10 down the z axis, we need to know
	# how far our plane is to the origin.  The origin is (0, 0, 0), so we need to find
	# the shortest distance our plane is from (0, 0, 0).  This way we can test the collision.
	# The direction the plane is facing is important (We know this by the normal), but it's
	# also important WHERE that plane is in our 3D space.  I hope this makes sense.

	# We created a function to calculate the distance for us.  All we need is the normal
	# of the plane, and then ANY point located on that plane.  Well, we have 3 points.  Each
	# point of the triangle is on the plane, so we just pass in one of our points.  It doesn't
	# matter which one, so we will just pass in the first one.  We get a single value back.
	# That is the distance.  Just like our normalized normal is of length 1, our distance
	# is a single value too.  It's like if you were to measure something with a ruler,
	# you don't measure it according to the X Y and Z of our world, you just want ONE number.

	originDistance = PlaneDistance(vNormal, vTriangle[0])

	# Now the next step is simple, but hard to understand at first.  What we need to
	# do is get the distance of EACH point from out plane.  Above we got the distance of the
	# plane to the point (0, 0, 0) which happens to be the origin, now we need to get a distance
	# for each point.  If the distance is a negative number, then the point is BEHIND the plane.
	# If the distance is positive, then the point is in front of the plane.  Basically, if the
	# line collides with the plane, there should be a negative and positive distance.  make sense?
	# If the line pierces the plane, it will have a negative distance and a positive distance,
	# meaning that a point will be on one side of the plane, and one point on the other.  But we
	# will do the check after this, first we need to get the distance of each point to the plane.

	# Now, we need to use something called the plane equation to get the distance from each point.
	# Here is the plane Equation:  (Ax + By + Cz + D = The distance from the plane)
	# If "The distance from the plane" is 0, that means that the point is ON the plane, which all the polygon points should be.
	# A, B and C is the Normal's X Y and Z values.  x y and z is the Point's x y and z values.
	# "the Point" meaning one of the points of our line.  D is the distance that the plane
	# is from the origin.  We just calculated that and stored it in "originDistance".
	# Let's fill in the equation with our data:

	# Get the distance from point1 from the plane using: Ax + By + Cz + D = (The distance from the plane)

	distance1 = ((vNormal.x * vLine[0].x)  +				# Ax +
		     (vNormal.y * vLine[0].y)  +				# Bx +
		     (vNormal.z * vLine[0].z)) + originDistance	# Cz + D

	# We just got the first distance from the first point to the plane, now let's get the second.
	
	# Get the distance from point2 from the plane using Ax + By + Cz + D = (The distance from the plane)
	
	distance2 = ((vNormal.x * vLine[1].x)  +				# Ax +
		     (vNormal.y * vLine[1].y)  +				# Bx +
		     (vNormal.z * vLine[1].z)) + originDistance # Cz + D

	# Ok, we should have 2 distances from the plane, from each point of our line.
	# Remember what I said about an intersection?  If one is negative and one is positive,
	# that means that they are both on either side of the plane.  So, all we need to do
	# is multiply the 2 distances together, and if the result is less than 0, we intersection.
	# This works because, any number times a negative number is always negative, IE (-1 * 1 = -1)
	# If they are both positive or negative values then it will be above zero.

	if(distance1 * distance2 >= 0):		# Check to see if both point's distances are both negative or both positive
	   return 0						# Return false if each point has the same sign.  -1 and 1 would mean each point is on either side of the plane.  -1 -2 or 3 4 wouldn't...
					
	return 1							# The line intersected the plane, Return TRUE
	
	 
####################################### OPERATIONS ###################################*
#####
##### theese are others varius operations, they returns vectors
#####
####################################### OPERATIONS ###################################*  
def Add(v1, v2):
  return CVector3(v1.x+v2.x, v1.y+v2.y, v1.z+v2.z)

def Subtract(v1, v2):
  return CVector3(v1.x-v2.x, v1.y-v2.y, v1.z-v2.z)

def Multiply(v1, m):
  return CVector3(v1.x*m, v1.y*m, v1.z*m)

def Divide( v1, m):
  return CVector3(v1.x/m, v1.y/m, v1.z/m)    
