#!/usr/bin/env python

##########################################################################*                       
######	                    GAME TUTORIAL 29 MODULE
######
######    Federico B. 2003 - dragonscraft@yahoo.it
######
################################# IMPORTS ################################*

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame
from pygame.locals import *
import pygame, pygame.key, pygame.display, pygame.mouse

############################# LOCAL IMPORTS ##############################*

from Math3D import *
################################# MODULE #################################*

				    # Left Point             Right Point             Top point
vTriangle = [CVector3(-1.0, 0.0, 0.0), CVector3(1.0, 0.0, 0.0),	CVector3(0.0, 1.0, 0.0)]

vLine     = [CVector3(0, 0.5, -0.5), CVector3(0, 0.5, 0.5)]

def resize((width, height)):
    if height==0:
        height=1
    glViewport(0, 0, width, height)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(45, 1.0*width/height, .1, 150.0)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

def init():
    glClearColor(0.0, 0.0, 0.0, 0.0)
    glEnable(GL_TEXTURE_2D)	
    glEnable(GL_DEPTH_TEST)

def draw():
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)  # Clear The Screen And The Depth Buffer
    glLoadIdentity()									# Reset The matrix
	
    # We changed the position of the camera so we could better see the normal at an angle. 
    # This positions the camera off to the left a bit and in front of the triangle.
    # We move our view up a bit so fully see the triangle instead looking down

	    # 	    Position          View	     Up Vector
    gluLookAt(-2.5, 0.5, 0.5,  0, 0.5, 0,   0, 1, 0)	# This determines where the camera's position and view is
	
    # Below we give OpenGL the 3 vertices of our triangle.  Once again, we put them
    # into an array of CVector3 structures so we could dynamically move it around screen.

    glBegin(GL_TRIANGLES)								# This is our BEGIN to draw

    glColor3ub(255, 0, 0)							# Make the Left vertex RED
    glVertex3f(vTriangle[0].x, vTriangle[0].y, vTriangle[0].z)

    glColor3ub(0, 255, 0)							# Make the Right vertex GREEN
    glVertex3f(vTriangle[1].x, vTriangle[1].y, vTriangle[1].z)

    glColor3ub(0, 0, 255)							# Make the Top vertex BLUE
    glVertex3f(vTriangle[2].x, vTriangle[2].y, vTriangle[2].z)
    glEnd()											# This is the END of drawing

    # Below we use our function we just wrote to see if the plane of the
    # triangle and the line intersect.  It will return true if that's the case.

    bCollided = IntersectedPolygon(vTriangle, vLine, 3)

    # Below we draw the line that the polygon will be colliding with.
    # We will check to see if the line collides with the polygons plane, and if it does, 
    # we will turn the line green to show when it is intersecting the plane.

    glBegin (GL_LINES)								# This is our BEGIN to draw

    if(bCollided):									# If we collided, change the color of the line to illustrate this.
      glColor3ub(0, 255, 0)				# Make the line GREEN if we collided with the triangle's plane
    else:
      glColor3ub(255, 255, 0)				# Make the line YELLOW if we didn't collide

    glVertex3f(vLine[0].x, vLine[0].y, vLine[0].z)	# Let's draw the normal centered on the triangle
													# Draw the normal of the polygon from the center of the polygon to better see it
    glVertex3f(vLine[1].x, vLine[1].y, vLine[1].z)	
    glEnd()										    # This is the END of drawing
    
def handleKeyPress(key):
  global vTriangle
    
  if key==K_ESCAPE:
    return 0
  if key==K_UP:				            # Check if we hit the UP ARROW key.
	vTriangle[0].x += 0.01				# Move the left point of the triangle to the left
	vTriangle[1].x += 0.01				# Move the right point of the triangle to the left
	vTriangle[2].x += 0.01				# Move the top point of the triangle to the left
  if key==K_DOWN:				        # Check if we hit the DOWN ARROW key.
	vTriangle[0].x -= 0.01				# Move the left point of the triangle to the left
	vTriangle[1].x -= 0.01				# Move the right point of the triangle to the left
	vTriangle[2].x -= 0.01				# Move the top point of the triangle to the left		
  if key==K_RIGHT:
  	vTriangle[0].z += 0.01				# Move the left point of the triangle forward
	vTriangle[1].z += 0.01				# Move the right point of the triangle forward
	vTriangle[2].z += 0.01				# Move the top point of the triangle forward
  if key==K_LEFT:
  	vTriangle[0].z -= 0.01				# Move the left point of the triangle back
	vTriangle[1].z -= 0.01				# Move the right point of the triangle back
	vTriangle[2].z -= 0.01				# Move the top point of the triangle back
  return 1

def main():

  video_flags = OPENGL|DOUBLEBUF|RESIZABLE
    
  pygame.init()
  pygame.display.set_mode((640,480), video_flags)
  pygame.key.set_repeat(10,10) 

  resize((640,480))
  init()
  frames = 0
  done = 0
  ticks = pygame.time.get_ticks() 
  while not done:
    while 1:
      event = pygame.event.poll()
      if event.type == NOEVENT:
        break
      if event.type == KEYDOWN:
        if handleKeyPress(event.key) == 0:
          done = 1
      if event.type == QUIT:
        done = 1
    #Loop the scene
    draw()
    # Here we check for mouse and key movements every frame
    pygame.display.flip()
    frames = frames+1

  print "fps:  %d" % ((frames*1000)/(pygame.time.get_ticks()-ticks))


if __name__ == '__main__': main()
