#!/usr/bin/env python
####################################################################################*
####################################################################################*                       
######	                       MATH3D MODULE
######
######    Federico B. 2003 - dragonscraft@yahoo.it
######
################################# IMPORTS ##########################################*
####################################################################################*

import os
from math import sin, cos, sqrt
from OpenGL.GL import *
from OpenGL.GLU import *
import pygame
from pygame.locals import *
import pygame, pygame.key, pygame.mouse, pygame.time
from math import cos, acos, sin, asin
from math import pi as PI

###################################### CVECTOR3 #####################################*
######
###### This  is the vector class
######
###################################### CVECTOR#######################################*
class CVector3(object):

  def __init__(self,x,y,z):
    self.x = x
    self.y = y
    self.z = z    
        
###################################### CROSS #######################################*
######
###### This returns a perpendicular vector from 2 given vectors by 
###### taking the cross product.
######
###################################### CROSS #######################################*
def Cross(vVector1, vVector2):

  vNormal = CVector3(0.0,0.0,0.0)
	
  # The X value for the vector is:  (V1.y * V2.z) - (V1.z * V2.y)													## Get the X value
  vNormal.x = ((vVector1.y * vVector2.z) - (vVector1.z * vVector2.y))															
  # The Y value for the vector is:  (V1.z * V2.x) - (V1.x * V2.z)
  vNormal.y = ((vVector1.z * vVector2.x) - (vVector1.x * vVector2.z))															
  # The Z value for the vector is:  (V1.x * V2.y) - (V1.y * V2.x)
  vNormal.z = ((vVector1.x * vVector2.y) - (vVector1.y * vVector2.x))
  # Return the cross product
  return vNormal	
  
####################################### VECTOR ###########################################*
#####
#####	This returns a vector between 2 points
#####
####################################### VECTOR ###########################################*

def Vector(vPoint1, vPoint2):

		vVector = CVector3(0,0,0)					# Initialize our variable to zero

		# In order to get a vector from 2 points (a direction) we need to
		# subtract the second point from the first point.

		vVector.x = vPoint1.x - vPoint2.x			# Get the X value of our new vector
		vVector.y = vPoint1.y - vPoint2.y			# Get the Y value of our new vector
		vVector.z = vPoint1.z - vPoint2.z			# Get the Z value of our new vector

		# Now that we have our new vector between the 2 points, we will return it.

		return vVector								# Return our new vector								 

####################################### MAGNITUDE ###################################*
#####
##### This returns the magnitude of a vector
#####
####################################### MAGNITUDE ###################################*
def Magnitude(vNormal):

  return (float(sqrt( (vNormal.x * vNormal.x) + 
					  (vNormal.y * vNormal.y) + 
					  (vNormal.z * vNormal.z) )))

####################################### NORMALIZE ###################################*
#####
#####	This returns a normalize vector (A vector exactly of length 1)
#####
####################################### NORMALIZE ###################################*
def Normalize(vVector):

  ## Get the magnitude of our normal
  magnitude = Magnitude(vVector)				
  vVector.x /= magnitude		
  vVector.y /= magnitude		
  vVector.z /= magnitude				
  ## Finally, return our normalized vector
  return vVector										

####################################### NORMAL #####################################*
#####
#####	This returns the normal of a polygon (The direction the polygon is facing)
#####
####################################### NORMAL #####################################*
def Normal(vTriangle):					# You might want to error check to make sure it's valid
									    # Get 2 vectors from the polygon
	vVector1 = Vector(vTriangle[2], vTriangle[0])
	vVector2 = Vector(vTriangle[1], vTriangle[0])

	vNormal = Cross(vVector1, vVector2)
	vNormal = Normalize(vNormal)						# Use our function we created to normalize the normal (Makes it a length of one)
	return vNormal										# Return our normal at our desired length
 
#################################### PLANE DISTANCE ######################################*
#####
#####	This returns the distance between a plane and the origin
#####
#################################### PLANE DISTANCE ######################################*									
def PlaneDistance(Normal, Point):
	
	distance = 0.0									# This variable holds the distance from the plane tot he origin
													# Basically, the negated dot product of the normal of the plane and the point. (More about the dot product in another tutorial)
	distance = - ((Normal.x * Point.x) + (Normal.y * Point.y) + (Normal.z * Point.z))

	return distance									# Return the distance

################################### INTERSECTED PLANE ######################################*
#####
#####	This checks to see if a line intersects a plane
#####
################################### INTERSECTED PLANE ######################################*
											
def IntersectedPlane(vPoly, vLine, vNormal, originDistance):

	distance1=0.0
	distance2=0.0						# The distances from the 2 points of the line from the plane
			
	vNormal = Normal(vPoly)				# We need to get the normal of our plane to go any further

	# Let's find the distance our plane is from the origin.  We can find this value
	# from the normal to the plane (polygon) and any point that lies on that plane (Any vertice)
	originDistance = PlaneDistance(vNormal, vPoly[0])

	# Get the distance from point1 from the plane using: Ax + By + Cz + D = (The distance from the plane)

	distance1 = ((vNormal.x * vLine[0].x)  +				# Ax +
		         (vNormal.y * vLine[0].y)  +				# Bx +
				 (vNormal.z * vLine[0].z)) + originDistance	# Cz + D
	
	# Get the distance from point2 from the plane using Ax + By + Cz + D = (The distance from the plane)
	
	distance2 = ((vNormal.x * vLine[1].x)  +				# Ax +
		         (vNormal.y * vLine[1].y)  +				# Bx +
				 (vNormal.z * vLine[1].z)) + originDistance	# Cz + D

	# Now that we have 2 distances from the plane, if we times them together we either
	# get a positive or negative number.  If it's a negative number, that means we collided!
	# This is because the 2 points must be on either side of the plane (IE. -1 * 1 = -1).

	if(distance1 * distance2 >= 0):			# Check to see if both point's distances are both negative or both positive
	   return 0						# Return false if each point has the same sign.  -1 and 1 would mean each point is on either side of the plane.  -1 -2 or 3 4 wouldn't...
					
	return 1							# The line intersected the plane, Return TRUE

################################### DOT ######################################*
#####
#####	This computers the dot product of 2 vectors
#####
################################### DOT ######################################*
def Dot(vVector1, vVector2):

	# The dot product is this equation: V1.V2 = (V1.x * V2.x  +  V1.y * V2.y  +  V1.z * V2.z)
	# In math terms, it looks like this:  V1.V2 = ||V1|| ||V2|| cos(theta)
	# The '.' means DOT.   The || || is magnitude.  So the magnitude of V1 times the magnitude
	# of V2 times the cosine of the angle.  It seems confusing now, but it will become more clear.
	# This function is used for a ton of things, which we will cover in other tutorials.
	# For this tutorial, we use it to compute the angle between 2 vectors.  If the vectors
	# are normalize, the dot product returns the cosine of the angle between the 2 vectors.
	# What does that mean? Well, it doesn't return the actual angle, it returns the value of:
	# cos(angle).	Well, what if we want to get the actual angle?  Then we use the arc cosine.
	# There is more on this in the below function AngleBetweenVectors().  Let's give some
	# applications of using the dot product.  How would you tell if the angle between the
	# 2 vectors is perpendicular (90 degrees)?  Well, if we normalize the vectors we can
	# get rid of the ||V1|| * ||V2|| in front, which just leaves us with:  cos(theta).
	# If a vector is normalize, it's magnitude is 1, so it would be: 1 * 1 * cos(theta) , 
	# which is pointless, so we discard that part of the equation.  So, What is the cosine of 90?
	# If you punch it in your calculator you will find that it's 0.  So that means
	# if the dot product of 2 angles is 0, then they are perpendicular.  What we did in
	# our mind is take the arc cosine of 0, which is 90 (or PI/2 in radians).  More on this below.

			 #    (V1.x * V2.x        +        V1.y * V2.y        +        V1.z * V2.z)
	return ( (vVector1.x * vVector2.x) + (vVector1.y * vVector2.y) + (vVector1.z * vVector2.z) )
		
################################### ANGLE BETWEEN VECTORS ######################################*
#####
#####	This checks to see if a point is inside the ranges of a polygon
#####
################################### ANGLE BETWEEN VECTORS ######################################*
def AngleBetweenVectors(Vector1, Vector2):
							
	## Remember, above we said that the Dot Product of returns the cosine of the angle
	## between 2 vectors?  Well, that is assuming they are unit vectors (normalize vectors).
	## So, if we don't have a unit vector, then instead of just saying  arcCos(DotProduct(A, B))
	## We need to divide the dot product by the magnitude of the 2 vectors multiplied by each other.
	## Here is the equation:   arc cosine of (V . W # || V || * || W || )
	## the || V || means the magnitude of V.  This then cancels out the magnitudes dot product magnitudes.
	## But basically, if you have normalize vectors already, you can forget about the magnitude part.

	## Get the dot product of the vectors
	dotProduct = Dot(Vector1, Vector2)	

	## Get the product of both of the vectors magnitudes
	vectorsMagnitude = Magnitude(Vector1) * Magnitude(Vector2) 

	## Get the arc cosine of the (dotProduct / vectorsMagnitude) which is the angle in RADIANS.
	## (IE.   PI#2 radians = 90 degrees      PI radians = 180 degrees    2*PI radians = 360 degrees)
	## To convert radians to degress use this equation:   radians * (PI / 180)
	## TO convert degrees to radians use this equation:   degrees * (180 / PI)
        try:
	  angle = acos( dotProduct / vectorsMagnitude )
        except (ValueError):
          return 0
	## Here we make sure that the angle is not a -1.#IND0000000 number, which means indefinate.
	## acos() thinks it's funny when it returns -1.#IND0000000.  If we don't do this check,
	## our collision results will sometimes say we are colliding when we aren't.  I found this
	## out the hard way after MANY hours and already wrong written tutorials :)  Usually
	## this value is found when the dot product and the maginitude are the same value.
	## We want to return 0 when this happens.
	
	# This is quite a big problem, python doesnt' menage nans..
	#if((angle)==Nan):
	#  return 0
	
	## Return the angle in radians
	return( angle )
	print angle
	
################################### INTERSECTION POINT ######################################*
#####
#####	This returns the intersection point of the line that intersects the plane
#####
################################### INTERSECTION POINT ######################################*											
def IntersectionPoint(vNormal, vLine, distance):
  
	vPoint = CVector3(0.0,0.0,0.0)
	vLineDir = CVector3(0.0,0.0,0.0) 		## Variables to hold the point and the line's direction
	Numerator = 0.0
	Denominator = 0.0
	dist = 0.0

	## Here comes the confusing part.  We need to find the 3D point that is actually
	## on the plane.  Here are some steps to do that:
	
	## 1)  First we need to get the vector of our line, Then normalize it so it's a length of 1
	vLineDir = Vector(vLine[1], vLine[0])		## Get the Vector of the line
	vLineDir = Normalize(vLineDir)				## Normalize the lines vector


	## 2) Use the plane equation (distance = Ax + By + Cz + D) to find the distance from one of our points to the plane.
	##    Here I just chose a arbitrary point as the point to find that distance.  You notice we negate that
	##    distance.  We negate the distance because we want to eventually go BACKWARDS from our point to the plane.
	##    By doing this is will basically bring us back to the plane to find our intersection point.
	Numerator = - (vNormal.x * vLine[0].x +		## Use the plane equation with the normal and the line
				   vNormal.y * vLine[0].y +
				   vNormal.z * vLine[0].z + distance)

	## 3) If we take the dot product between our line vector and the normal of the polygon,
	##    this will give us the cosine of the angle between the 2 (since they are both normalized - length 1).
	##    We will then divide our Numerator by this value to find the offset towards the plane from our arbitrary point.
	Denominator = Dot(vNormal, vLineDir)		## Get the dot product of the line's vector and the normal of the plane
				  
	## Since we are using division, we need to make sure we don't get a divide by zero error
	## If we do get a 0, that means that there are INFINATE points because the the line is
	## on the plane (the normal is perpendicular to the line - (Normal.Vector = 0)).  
	## In this case, we should just return any point on the line.

	if( Denominator == 0):					## Check so we don't divide by zero
	  return vLine[0]						## Return an arbitrary point on the line

	## We divide the (distance from the point to the plane) by (the dot product)
	## to get the distance (dist) that we need to move from our arbitrary point.  We need
	## to then times this distance (dist) by our line's vector (direction).  When you times
	## a scalar (single number) by a vector you move along that vector.  That is what we are
	## doing.  We are moving from our arbitrary point we chose from the line BACK to the plane
	## along the lines vector.  It seems logical to just get the numerator, which is the distance
	## from the point to the line, and then just move back that much along the line's vector.
	## Well, the distance from the plane means the SHORTEST distance.  What about in the case that
	## the line is almost parallel with the polygon, but doesn't actually intersect it until half
	## way down the line's length.  The distance from the plane is short, but the distance from
	## the actual intersection point is pretty long.  If we divide the distance by the dot product
	## of our line vector and the normal of the plane, we get the correct length.  Cool huh?

	dist = Numerator / Denominator				## Divide to get the multiplying (percentage) factor
	
	## Now, like we said above, we times the dist by the vector, then add our arbitrary point.
	## This essentially moves the point along the vector to a certain distance.  This now gives
	## us the intersection point.  Yay!

	vPoint.x = (float(vLine[0].x + (vLineDir.x * dist)))
	vPoint.y = (float(vLine[0].y + (vLineDir.y * dist)))
	vPoint.z = (float(vLine[0].z + (vLineDir.z * dist)))

	return vPoint								## Return the intersection point

################################### INSIDE POLYGON ######################################*
#####
#####	This checks to see if a point is inside the ranges of a polygon
#####
################################### INSIDE POLYGON ######################################*

def InsidePolygon(vIntersection, Poly, verticeCount):

	MATCH_FACTOR = 0.9999		## Used to cover up the error in floating point
	Angle = 0.0					## Initialize the angle
	vA=CVector3(0.0,0.0,0.0) 
	vB=CVector3(0.0,0.0,0.0) 					## Create temp vectors
	
	## Just because we intersected the plane, doesn't mean we were anywhere near the polygon.
	## This functions checks our intersection point to make sure it is inside of the polygon.
	## This is another tough function to grasp at first, but let me try and explain.
	## It's a brilliant method really, what it does is create triangles within the polygon
	## from the intersection point.  It then adds up the inner angle of each of those triangles.
	## If the angles together add up to 360 degrees (or 2 * PI in radians) then we are inside!
	## If the angle is under that value, we must be outside of polygon.  To further
	## understand why this works, take a pencil and draw a perfect triangle.  Draw a dot in
	## the middle of the triangle.  Now, from that dot, draw a line to each of the vertices.
	## Now, we have 3 triangles within that triangle right?  Now, we know that if we add up
	## all of the angles in a triangle we get 360 right?  Well, that is kinda what we are doing,
	## but the inverse of that.  Say your triangle is an isosceles triangle, so add up the angles
	## and you will get 360 degree angles.  90 + 90 + 90 is 360.

	for i in xrange(0,verticeCount,1):		## Go in a circle to each vertex and get the angle between
	
		vA = Vector(Poly[i], vIntersection)	## Subtract the intersection point from the current vertex
												## Subtract the point from the next vertex
		vB = Vector(Poly[int((i + 1) % verticeCount)], vIntersection)
												
		Angle += AngleBetweenVectors(vA, vB)	## Find the angle between the 2 vectors and add them all up as we go along

	## Now that we have the total angles added up, we need to check if they add up to 360 degrees.
	## Since we are using the dot product, we are working in radians, so we check if the angles
	## equals 2*PI.  We defined PI in 3DMath.h.  You will notice that we use a MATCH_FACTOR
	## in conjunction with our desired degree.  This is because of the inaccuracy when working
	## with floating point numbers.  It usually won't always be perfectly 2 * PI, so we need
	## to use a little twiddling.  I use .9999, but you can change this to fit your own desired accuracy.
												
	if(Angle >= (MATCH_FACTOR * (2.0 * PI)) ):	## If the angle is greater than 2 PI, (360 degrees)
		return 1							## The point is inside of the polygon
		
	return 0								## If you get here, it obviously wasn't inside the polygon, so Return FALSE

################################### INTERSECTED POLYGON ######################################*
#####
#####	This checks if a line is intersecting a polygon
#####
################################### INTERSECTED POLYGON ######################################*

def IntersectedPolygon(vPoly, vLine, verticeCount):

	vNormal = CVector3(0.0,0.0,0.0)
	originDistance = 0.0
	
	## First we check to see if our line intersected the plane.  If this isn't true
	## there is no need to go on, so return false immediately.
	## We pass in address of vNormal and originDistance so we only calculate it once

									 ## Reference   ## Reference
	if( not IntersectedPlane( vPoly, vLine, vNormal, originDistance)):
		return 0

	## Now that we have our normal and distance passed back from IntersectedPlane(), 
	## we can use it to calculate the intersection point.  The intersection point
	## is the point that actually is ON the plane.  It is between the line.  We need
	## this point test next, if we are inside the polygon.  To get the I-Point, we
	## give our function the normal of the plan, the points of the line, and the originDistance.
	
	# Python seems suffer of the same problem than java
	# Java doesn't allow pass backs (let's recalculate them)
	vNormal = Normal(vPoly)
	originDistance = PlaneDistance(vNormal, vPoly[0])
	
	vIntersection = IntersectionPoint(vNormal, vLine, originDistance)

	## Now that we have the intersection point, we need to test if it's inside the polygon.
	## To do this, we pass in :
	## (our intersection point, the polygon, and the number of vertices our polygon has)

	if(InsidePolygon(vIntersection, vPoly, verticeCount)):
		return 1							## We collided!	  Return success


	## If we get here, we must have NOT collided

	return 0								## There was no collision, so return false
			 
####################################### OPERATIONS ###################################*
#####
##### theese are others varius operations, they returns vectors
#####
####################################### OPERATIONS ###################################*  
def Add(v1, v2):
  return CVector3(v1.x+v2.x, v1.y+v2.y, v1.z+v2.z)

def Subtract(v1, v2):
  return CVector3(v1.x-v2.x, v1.y-v2.y, v1.z-v2.z)

def Multiply(v1, m):
  return CVector3(v1.x*m, v1.y*m, v1.z*m)

def Divide( v1, m):
  return CVector3(v1.x/m, v1.y/m, v1.z/m)    
