#!/usr/bin/env python

##########################################################################*                       
######	                    GAME TUTORIAL 31 MODULE
######
######    Federico B. 2003 - dragonscraft@yahoo.it
######
################################# IMPORTS ################################*

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame
from pygame.locals import *
import pygame, pygame.key, pygame.display, pygame.mouse

############################# LOCAL IMPORTS ##############################*

from Math3D import *
################################# MODULE #################################*

				    # Left Point             Right Point             Top point

g_vLine = [CVector3(0.0,0.0,0.0),CVector3(0.0,0.0,0.0)]
g_vPosition = CVector3(0.0,0.0,0.0)
g_rotateY = 0.0

def resize((width, height)):
    if height==0:
        height=1
    glViewport(0, 0, width, height)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(45, 1.0*width/height, .1, 150.0)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

def init():
    global g_vLine, g_vPosition
    
    glClearColor(0.0, 0.0, 0.0, 0.0)

    glEnable(GL_TEXTURE_2D)
    glEnable(GL_DEPTH_TEST)	
 
    g_vLine[0] = CVector3(-1,  0,   0)
    g_vLine[1] = CVector3( 1,  0,   0)

    # Initialize our position of the sphere, which will be used as the given point
    g_vPosition = CVector3(0, 0.5, 0)

    # Here, we turn on a light and enable lighting.  We don't need to
    # set anything else for lighting because we will just take the defaults.
    # We also want color, so we turn that on too.  This gives the spheres a bit
    # of dimension.

    glEnable(GL_LIGHT0)								# Turn on a light with defaults set
    glEnable(GL_LIGHTING)							# Turn on lighting
    glEnable(GL_COLOR_MATERIAL)						# Allow color

def draw():
    global g_vrotatY, g_vLine, g_Position

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)  # Clear The Screen And The Depth Buffer
    glLoadIdentity()
    
    # Set our camera to the left a bit for a nice side view of the left hand side

    #     Position   View	    Up Vector
    gluLookAt(0, 1, -2,  0, 0, 0,   0, 1, 0)	# This determines where the camera's position and view is

    # Rotate the camera around our world for a 360 degree view of what is going on.
    # Yes I know... we are actually rotating the whole world, not the camera :)
    glRotatef(g_rotateY, 0, 1, 0)

    # We need a radius for our spheres, so let's create it here with 0.05f
    radius  = 0.05
    radius2 = 0.01
		
    # This sets our line width to %250, since 1 is %100 for normal line width
    glLineWidth(2.5)

    # Turn of lighting so our line isn't effected by it
    glDisable(GL_LIGHTING)
	
    # Draw the line segment
    glBegin (GL_LINES)							# This is our BEGIN to draw

    glColor3ub(255, 0, 0)						# Make the Left vertex RED
    glVertex3f(g_vLine[0].x, g_vLine[0].y, g_vLine[0].z)

    glColor3ub(0, 255, 0)						# Make the Right vertex GREEN
    glVertex3f(g_vLine[1].x, g_vLine[1].y, g_vLine[1].z)
    glEnd()										# This is the END of drawing

    glEnable(GL_LIGHTING)						# Turn lighting back on

    # Instead of calculating the sphere ourselves, we are going to use quadrics.
    # Check out the tutorial on quadrics if this is confusing for you.

    # Allocate a quadric object to use as a sphere
    pObj = gluNewQuadric()				# Get a Quadric off the stack

    # Below we create a new matrix so that when we translate the first sphere,
    # it doesn't effect the other sphere that we need to translate.

    glPushMatrix()										# Push on a new matrix

    # Move the sphere to it's center position
    glTranslatef(g_vPosition.x, g_vPosition.y, g_vPosition.z)
		
    # Draw the quadric as a sphere with the radius of .05 and a 15 by 15 detail.
    # To increase the detail of the sphere, just increase the 2 last parameters.
    gluSphere(pObj, radius, 15, 15)
	
    glPopMatrix()										# Pop off the current matrix

    # Now we need to find the closest point on the line.  We pass in the end points
    # of the line segment, and the point we want to use, which is the sphere's center position.
    # This returns the closest point to use for the center of the purple sphere.
    vClosestPoint = ClosestPointOnLine(g_vLine[0], g_vLine[1], g_vPosition)

    # Move the closest point sphere to it's center position
    glTranslatef(vClosestPoint.x, vClosestPoint.y, vClosestPoint.z)

    # Turn the line segment sphere purple
    glColor3f(0.8, 0, 1)

    # Draw the quadric as a sphere with the radius of .01 and a 15 by 15 detail.
    gluSphere(pObj, radius2, 15, 15)

    #gluDeleteQuadric(pObj)
    
def handleKeyPress(key):
  global g_vPosition, g_rotateY
    
  if key==K_ESCAPE:
    return 0
  if key==K_UP:				            # Check if we hit the UP ARROW key.
    g_vPosition.y += 0.01				# Move the sphere up
  if key==K_DOWN:				        # Check if we hit the DOWN ARROW key.
	g_vPosition.y -= 0.01			    # Move the sphere up	
  if key==K_RIGHT:
    g_vPosition.x -= 0.01
  if key==K_LEFT:
    g_vPosition.x += 0.01
  if key==K_F1:				
	g_rotateY -= 2						# Rotate the camera left								
  if key==K_F2:							# Check if we hit the F2 key.
	g_rotateY += 2;						# Rotate the camera right
  if key==K_F3:							# Check if we hit the F3 key.
	g_vPosition.z -= 0.01				# Move the sphere in front of the line
  if key==K_F4:							# Check if we hit the F4 key.
	g_vPosition.z += 0.01				# Move the sphere in front of the line
  return 1

def main():

  video_flags = OPENGL|DOUBLEBUF|RESIZABLE
    
  pygame.init()
  pygame.display.set_mode((640,480), video_flags)
  pygame.key.set_repeat(10,10) 

  resize((640,480))
  init()
  frames = 0
  done = 0
  ticks = pygame.time.get_ticks() 
  while not done:
    while 1:
      event = pygame.event.poll()
      if event.type == NOEVENT:
        break
      if event.type == KEYDOWN:
        if handleKeyPress(event.key) == 0:
          done = 1
      if event.type == QUIT:
        done = 1
    #Loop the scene
    draw()
    # Here we check for mouse and key movements every frame
    pygame.display.flip()
    frames = frames+1

  print "fps:  %d" % ((frames*1000)/(pygame.time.get_ticks()-ticks))


if __name__ == '__main__': main()
