#!/usr/bin/env python
####################################################################################*
####################################################################################*                       
######	                       MATH3D MODULE
######
######    Federico B. 2003 - dragonscraft@yahoo.it
######
################################# IMPORTS ##########################################*
####################################################################################*

import os
from math import sin, cos, sqrt
from OpenGL.GL import *
from OpenGL.GLU import *
import pygame
from pygame.locals import *
import pygame, pygame.key, pygame.mouse, pygame.time
from math import cos, acos, sin, asin
from math import pi as PI

###################################### CVECTOR3 #####################################*
######
###### This  is the vector class
######
###################################### CVECTOR#######################################*
class CVector3(object):

    def __init__(self,x,y,z):
      self.x = x
      self.y = y
      self.z = z    
        
###################################### CROSS #######################################*
######
###### This returns a perpendicular vector from 2 given vectors by 
###### taking the cross product.
######
###################################### CROSS #######################################*
def Cross(vVector1, vVector2):

    vNormal = CVector3(0.0,0.0,0.0)
	
    # The X value for the vector is:  (V1.y * V2.z) - (V1.z * V2.y)													## Get the X value
    vNormal.x = ((vVector1.y * vVector2.z) - (vVector1.z * vVector2.y))															
    # The Y value for the vector is:  (V1.z * V2.x) - (V1.x * V2.z)
    vNormal.y = ((vVector1.z * vVector2.x) - (vVector1.x * vVector2.z))															
    # The Z value for the vector is:  (V1.x * V2.y) - (V1.y * V2.x)
    vNormal.z = ((vVector1.x * vVector2.y) - (vVector1.y * vVector2.x))
    # Return the cross product
    return vNormal	
  
####################################### VECTOR ###########################################*
#####
#####	This returns a vector between 2 points
#####
####################################### VECTOR ###########################################*

def Vector(vPoint1, vPoint2):

    vVector = CVector3(0,0,0)				# Initialize our variable to zero

    # In order to get a vector from 2 points (a direction) we need to
    # subtract the second point from the first point.

    vVector.x = vPoint1.x - vPoint2.x			# Get the X value of our new vector
    vVector.y = vPoint1.y - vPoint2.y			# Get the Y value of our new vector
    vVector.z = vPoint1.z - vPoint2.z			# Get the Z value of our new vector

    # Now that we have our new vector between the 2 points, we will return it.

    return vVector					# Return our new vector								 

####################################### MAGNITUDE ###################################*
#####
##### This returns the magnitude of a vector
#####
####################################### MAGNITUDE ###################################*
def Magnitude(vNormal):

    return (float(sqrt( (vNormal.x * vNormal.x) + 
		        (vNormal.y * vNormal.y) + 
		        (vNormal.z * vNormal.z) )))

####################################### NORMALIZE ###################################*
#####
#####	This returns a normalize vector (A vector exactly of length 1)
#####
####################################### NORMALIZE ###################################*
def Normalize(vVector):

    ## Get the magnitude of our normal
    magnitude = Magnitude(vVector)				
    vVector.x /= magnitude		
    vVector.y /= magnitude		
    vVector.z /= magnitude				
    ## Finally, return our normalized vector
    return vVector										

####################################### NORMAL #####################################*
#####
#####	This returns the normal of a polygon (The direction the polygon is facing)
#####
####################################### NORMAL #####################################*
def Normal(vTriangle):					# You might want to error check to make sure it's valid
							# Get 2 vectors from the polygon
    vVector1 = Vector(vTriangle[2], vTriangle[0])
    vVector2 = Vector(vTriangle[1], vTriangle[0])

    vNormal = Cross(vVector1, vVector2)
    vNormal = Normalize(vNormal)			# Use our function we created to normalize the normal (Makes it a length of one)
    return vNormal					# Return our normal at our desired length
 
#################################### PLANE DISTANCE ######################################*
#####
#####	This returns the distance between a plane and the origin
#####
#################################### PLANE DISTANCE ######################################*									
def PlaneDistance(Normal, Point):
	
    distance = 0.0					# This variable holds the distance from the plane tot he origin
													# Basically, the negated dot product of the normal of the plane and the point. (More about the dot product in another tutorial)
    distance = - ((Normal.x * Point.x) + (Normal.y * Point.y) + (Normal.z * Point.z))

    return distance					# Return the distance

################################### INTERSECTED PLANE ######################################*
#####
#####	This checks to see if a line intersects a plane
#####
################################### INTERSECTED PLANE ######################################*
											
def IntersectedPlane(vPoly, vLine, vNormal, originDistance):

    distance1=0.0
    distance2=0.0					# The distances from the 2 points of the line from the plane
			
    vNormal = Normal(vPoly)				# We need to get the normal of our plane to go any further

    # Let's find the distance our plane is from the origin.  We can find this value
    # from the normal to the plane (polygon) and any point that lies on that plane (Any vertice)
    originDistance = PlaneDistance(vNormal, vPoly[0])

    # Get the distance from point1 from the plane using: Ax + By + Cz + D = (The distance from the plane)

    distance1 = ((vNormal.x * vLine[0].x)  +			# Ax +
		(vNormal.y * vLine[0].y)  +			# Bx +
		(vNormal.z * vLine[0].z)) + originDistance	# Cz + D
	
    # Get the distance from point2 from the plane using Ax + By + Cz + D = (The distance from the plane)
	
    distance2 = ((vNormal.x * vLine[1].x)  +			# Ax +
		(vNormal.y * vLine[1].y)  +			# Bx +
		(vNormal.z * vLine[1].z)) + originDistance	# Cz + D

    # Now that we have 2 distances from the plane, if we times them together we either
    # get a positive or negative number.  If it's a negative number, that means we collided!
    # This is because the 2 points must be on either side of the plane (IE. -1 * 1 = -1).

    if(distance1 * distance2 >= 0):			# Check to see if both point's distances are both negative or both positive
	return 0					# Return false if each point has the same sign.  -1 and 1 would mean each point is on either side of the plane.  -1 -2 or 3 4 wouldn't...
					
    return 1						# The line intersected the plane, Return TRUE

################################### DOT ######################################*
#####
#####	This computers the dot product of 2 vectors
#####
################################### DOT ######################################*
def Dot(vVector1, vVector2):
    
	 #    (V1.x * V2.x        +        V1.y * V2.y        +        V1.z * V2.z)
    return ( (vVector1.x * vVector2.x) + (vVector1.y * vVector2.y) + (vVector1.z * vVector2.z) )
		
################################### ANGLE BETWEEN VECTORS ######################################*
#####
#####	This checks to see if a point is inside the ranges of a polygon
#####
################################### ANGLE BETWEEN VECTORS ######################################*
def AngleBetweenVectors(Vector1, Vector2):
    
    ## Get the dot product of the vectors
    dotProduct = Dot(Vector1, Vector2)	

    ## Get the product of both of the vectors magnitudes
    vectorsMagnitude = Magnitude(Vector1) * Magnitude(Vector2)
    # This is quite a big problem, python doesnt' menage nans..
    #if((angle)==Nan):
    #  return 0 so when angle is a nan we make an exception
    try:
      angle = acos( dotProduct / vectorsMagnitude )
    except (ValueError):
      return 0
	
    ## Return the angle in radians
    return( angle )
	
################################### INTERSECTION POINT ######################################*
#####
#####	This returns the intersection point of the line that intersects the plane
#####
################################### INTERSECTION POINT ######################################*											
def IntersectionPoint(vNormal, vLine, distance):
  
    vPoint = CVector3(0.0,0.0,0.0)
    vLineDir = CVector3(0.0,0.0,0.0) 		## Variables to hold the point and the line's direction
    Numerator = 0.0
    Denominator = 0.0
    dist = 0.0

    # Here comes the confusing part.  We need to find the 3D point that is actually
    # on the plane.  Here are some steps to do that:
	
    # 1)  First we need to get the vector of our line, Then normalize it so it's a length of 1
    vLineDir = Vector(vLine[1], vLine[0])		## Get the Vector of the line
    vLineDir = Normalize(vLineDir)			## Normalize the lines vector

    Numerator = - (vNormal.x * vLine[0].x +		## Use the plane equation with the normal and the line
		   vNormal.y * vLine[0].y +
		   vNormal.z * vLine[0].z + distance)

    Denominator = Dot(vNormal, vLineDir)		## Get the dot product of the line's vector and the normal of the plane
				  
    if( Denominator == 0):				## Check so we don't divide by zero
      return vLine[0]					## Return an arbitrary point on the line

    dist = Numerator / Denominator			## Divide to get the multiplying (percentage) factor
	
    vPoint.x = (float(vLine[0].x + (vLineDir.x * dist)))
    vPoint.y = (float(vLine[0].y + (vLineDir.y * dist)))
    vPoint.z = (float(vLine[0].z + (vLineDir.z * dist)))

    return vPoint				## Return the intersection point

################################### INSIDE POLYGON ######################################*
#####
#####	This checks to see if a point is inside the ranges of a polygon
#####
################################### INSIDE POLYGON ######################################*

def InsidePolygon(vIntersection, Poly, verticeCount):

    MATCH_FACTOR = 0.9999		## Used to cover up the error in floating point
    Angle = 0.0					## Initialize the angle
    vA=CVector3(0.0,0.0,0.0) 
    vB=CVector3(0.0,0.0,0.0) 					## Create temp vectors
	
    for i in xrange(0,verticeCount,1):		## Go in a circle to each vertex and get the angle between
	
      vA = Vector(Poly[i], vIntersection)	## Subtract the intersection point from the current vertex
												## Subtract the point from the next vertex
      vB = Vector(Poly[int((i + 1) % verticeCount)], vIntersection)
												
      Angle += AngleBetweenVectors(vA, vB)	## Find the angle between the 2 vectors and add them all up as we go along
											
    if(Angle >= (MATCH_FACTOR * (2.0 * PI)) ):	## If the angle is greater than 2 PI, (360 degrees)
      return 1							## The point is inside of the polygon
		
    return 0								## If you get here, it obviously wasn't inside the polygon, so Return FALSE

################################### INTERSECTED POLYGON ######################################*
#####
#####	This checks if a line is intersecting a polygon
#####
################################### INTERSECTED POLYGON ######################################*
def IntersectedPolygon(vPoly, vLine, verticeCount):

    vNormal = CVector3(0.0,0.0,0.0)
    originDistance = 0.0

    if( not IntersectedPlane( vPoly, vLine, vNormal, originDistance)):
      return 0	
    # Python seems suffer of the same problem than java
    # Java doesn't allow pass backs (let's recalculate them)
    vNormal = Normal(vPoly)
    originDistance = PlaneDistance(vNormal, vPoly[0])
	
    vIntersection = IntersectionPoint(vNormal, vLine, originDistance)

    ## Now that we have the intersection point, we need to test if it's inside the polygon.
    ## To do this, we pass in :
    ## (our intersection point, the polygon, and the number of vertices our polygon has)

    if(InsidePolygon(vIntersection, vPoly, verticeCount)):
      return 1							## We collided!	  Return success


    ## If we get here, we must have NOT collided

    return 0								## There was no collision, so return false

################################### DISTANCE #######################################*
#####
#####	This returns the distance between 2 3D points
#####
################################### DISTANCE #######################################*
def Distance( vPoint1, vPoint2):

    # This is the classic formula used in beginning algebra to return the
    # distance between 2 points.  Since it's 3D, we just add the z dimension:
    # 
    # Distance = sqrt(  (P2.x - P1.x)^2 + (P2.y - P1.y)^2 + (P2.z - P1.z)^2 )
    #
    distance = sqrt( (vPoint2.x - vPoint1.x) * (vPoint2.x - vPoint1.x) +
		    (vPoint2.y - vPoint1.y) * (vPoint2.y - vPoint1.y) +
		    (vPoint2.z - vPoint1.z) * (vPoint2.z - vPoint1.z) )

    # Return the distance between the 2 points
    return (float(distance))

############################## CLOSET POINT ON LINE ###################################*
#####
#####	This returns the point on the line vA_vB that is closest to the point vPoint
#####
############################## CLOSET POINT ON LINE ##################################*
def ClosestPointOnLine( vA, vB, vPoint):

    ## This function takes a line segment, vA to vB, then a point, vPoint.
    ## We want to find the closet point on the line segment to our vPoint
    ## out in space.  Either it is going to be one of the end points of the line,
    ## or it is going to be somewhere between vA and vB.  This is a important
    ## function when dealing with collision detection.

    ## Here is how it works, it's a bit confusing at first, so you will need
    ## to contemplate it a bit.  First, we want to grab a vector from "vA" to the point.
    ## Then we want to get a normalized vector from "vA" to "vB".  We don't need the
    ## full length of the line segment vector, we just want a direction.  That is why 
    ## we normalize it.  Remember, this is important because we are going to be using 
    ## the dot product coming up next.  So, now we have 2 vectors that form a pseudo corner
    ## of a triangle on the plane of the line segment and the point.
    ##
    ## Next, we want to find the distance or "magnitude" of the line segment.  This is
    ## done with a simple distance formula.  Then we use the dot "vVector2" with "vVector1".
    ## By using the dot product, we can essentially project vVector1 onto the
    ## line segments normalized vector, "vVector2".  If the result of the dot product is
    ## 0, that means the vectors were perpendicular and had a 90 degree angle between them.
    ## The 0 part is the distance the new projected vector is from the starting of vVector2.
    ## If the result is a negative number, that means the angle between the 2 vectors
    ## is greater than 90 degrees, which means that the closest point must be "vA" because
    ## it's projected vector is on the outside of the line.  So, if the result is a positive
    ## number, the projected vector is on the right side of "vA", but could be past the right
    ## side of vB.  To test this, we make sure that the result of the dot product is NOT
    ## greater than the distance "d".  If it is, then the closest point on the plane is
    ## obviously vB.  
    ##
    ## So, we can find the closest point easily if it's one of the end points of the line
    ## segment, but how do we find the point between the 2 end points?  This is simple.
    ## Since we have the distance "t" from point "vA" (given to us from the dot product 
    ## of the 2 vectors), we just use our vector that is going the direction of the
    ## line segment, "vVector2", and multiply it by the distance scalar "t".  This will
    ## create a vector going in the direction of the line segment, with a distance
    ## (or magnitude) of the projected vector, "vVector1", is from from "vA".  We then add
    ## this vector to "vA", which gives us the point on the line that is closest to our
    ## point out in space, vPoint!  
    ##
    ## This is probably pretty hard to visualize with just comments, unless you have a good 
    ## grasp of linear algebra.  
	
    ## Create the vector from end point vA to our point vPoint.
    vVector1 = Subtract(vPoint, vA)

    ## Create a normalized direction vector from end point vA to end point vB
    vVector2 = Normalize(Subtract(vB, vA))

    ## Use the distance formula to find the distance of the line segment (or magnitude)
    d = Distance(vA, vB)

    ## Using the dot product, we project the vVector1 onto the vector vVector2.
    ## This essentially gives us the distance from our projected vector from vA.
    t = Dot(vVector2, vVector1)

    ## If our projected distance from vA, "t", is less than or equal to 0, it must
    ## be closest to the end point vA.  We want to return this end point.
    if (t <= 0): 
		return vA

    ## If our projected distance from vA, "t", is greater than or equal to the magnitude
    ## or distance of the line segment, it must be closest to the end point vB.  So, return vB.
    if (t >= d) :
		return vB
 
    ## Here we create a vector that is of length t and in the direction of vVector2
    vVector3 = Multiply(vVector2 ,t)

    ## To find the closest point on the line segment, we just add vVector3 to the original
    ## end point vA.  
    vClosestPoint = Add(vA, vVector3)

    ## Return the closest point on the line segment
    return vClosestPoint	
				 
####################################### OPERATIONS ###################################*
#####
##### theese are others varius operations, they returns vectors
#####
####################################### OPERATIONS ###################################*  
def Add(v1, v2):
  return CVector3(v1.x+v2.x, v1.y+v2.y, v1.z+v2.z)

def Subtract(v1, v2):
  return CVector3(v1.x-v2.x, v1.y-v2.y, v1.z-v2.z)

def Multiply(v1, m):
  return CVector3(v1.x*m, v1.y*m, v1.z*m)

def Divide( v1, m):
  return CVector3(v1.x/m, v1.y/m, v1.z/m)    
