#!/usr/bin/env python
####################################################################################*
####################################################################################*                       
######	                       MATH3D MODULE
######
######    Federico B. 2003 - dragonscraft@yahoo.it
######
################################# IMPORTS ##########################################*
####################################################################################*

import os
from math import sin, cos, sqrt
from OpenGL.GL import *
from OpenGL.GLU import *
import pygame
from pygame.locals import *
import pygame, pygame.key, pygame.mouse, pygame.time
from math import cos, acos, sin, asin
from math import pi as PI
from exceptions import *

BEHIND = 0	# This is returned if the sphere is completely behind the plane
INTERSECTS = 1	# This is returned if the sphere intersects the plane
FRONT = 2	# This is returned if the sphere is completely in front of the plane

###################################### CVECTOR3 #####################################*
######
###### This  is the vector class
######
###################################### CVECTOR#######################################*
class CVector3(object):

    def __init__(self,x,y,z):
      self.x = x
      self.y = y
      self.z = z  
      
####################################### ABSOLUTE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
#####
#####	This returns the absolute value of the number passed in
#####
####################################### ABSOLUTE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
def Absolute(num):

    # If num is less than zero, we want to return the absolute value of num.
    # This is simple, either we times num by -1 or subtract it from 0.
    if(num < 0):
      return (0 - num)

    # Return the original number because it was already positive
    return num
        
###################################### CROSS #######################################*
######
###### This returns a perpendicular vector from 2 given vectors by 
###### taking the cross product.
######
###################################### CROSS #######################################*
def Cross(vVector1, vVector2):

    vNormal = CVector3(0.0,0.0,0.0)
	
    # The X value for the vector is:  (V1.y * V2.z) - (V1.z * V2.y)													## Get the X value
    vNormal.x = ((vVector1.y * vVector2.z) - (vVector1.z * vVector2.y))															
    # The Y value for the vector is:  (V1.z * V2.x) - (V1.x * V2.z)
    vNormal.y = ((vVector1.z * vVector2.x) - (vVector1.x * vVector2.z))															
    # The Z value for the vector is:  (V1.x * V2.y) - (V1.y * V2.x)
    vNormal.z = ((vVector1.x * vVector2.y) - (vVector1.y * vVector2.x))
    # Return the cross product
    return vNormal	
  
####################################### VECTOR ###########################################*
#####
#####	This returns a vector between 2 points
#####
####################################### VECTOR ###########################################*

def Vector(vPoint1, vPoint2):

    vVector = CVector3(0,0,0)				# Initialize our variable to zero

    # In order to get a vector from 2 points (a direction) we need to
    # subtract the second point from the first point.

    vVector.x = vPoint1.x - vPoint2.x			# Get the X value of our new vector
    vVector.y = vPoint1.y - vPoint2.y			# Get the Y value of our new vector
    vVector.z = vPoint1.z - vPoint2.z			# Get the Z value of our new vector

    # Now that we have our new vector between the 2 points, we will return it.

    return vVector					# Return our new vector								 

####################################### MAGNITUDE ###################################*
#####
##### This returns the magnitude of a vector
#####
####################################### MAGNITUDE ###################################*
def Magnitude(vNormal):

    return (float(sqrt( (vNormal.x * vNormal.x) + 
		        (vNormal.y * vNormal.y) + 
		        (vNormal.z * vNormal.z) )))

####################################### NORMALIZE ###################################*
#####
#####	This returns a normalize vector (A vector exactly of length 1)
#####
####################################### NORMALIZE ###################################*
def Normalize(vVector):

    ## Get the magnitude of our normal
    magnitude = Magnitude(vVector)				
    vVector.x /= magnitude		
    vVector.y /= magnitude		
    vVector.z /= magnitude				
    ## Finally, return our normalized vector
    return vVector										

####################################### NORMAL #####################################*
#####
#####	This returns the normal of a polygon (The direction the polygon is facing)
#####
####################################### NORMAL #####################################*
def Normal(vTriangle):					# You might want to error check to make sure it's valid
							# Get 2 vectors from the polygon
    vVector1 = Vector(vTriangle[2], vTriangle[0])
    vVector2 = Vector(vTriangle[1], vTriangle[0])

    vNormal = Cross(vVector1, vVector2)
    vNormal = Normalize(vNormal)			# Use our function we created to normalize the normal (Makes it a length of one)
    return vNormal					# Return our normal at our desired length
 
#################################### PLANE DISTANCE ######################################*
#####
#####	This returns the distance between a plane and the origin
#####
#################################### PLANE DISTANCE ######################################*									
def PlaneDistance(Normal, Point):
	
    distance = 0.0				# This variable holds the distance from the plane tot he origin
						# Basically, the negated dot product of the normal of the plane and the point. (More about the dot product in another tutorial)
    distance = - ((Normal.x * Point.x) + (Normal.y * Point.y) + (Normal.z * Point.z))

    return distance				# Return the distance

################################### INTERSECTED PLANE ######################################*
#####
#####	This checks to see if a line intersects a plane
#####
################################### INTERSECTED PLANE ######################################*
											
def IntersectedPlane(vPoly, vLine, vNormal, originDistance):

    distance1=0.0
    distance2=0.0				# The distances from the 2 points of the line from the plane
			
    vNormal = Normal(vPoly)			# We need to get the normal of our plane to go any further

    # Let's find the distance our plane is from the origin.  We can find this value
    # from the normal to the plane (polygon) and any point that lies on that plane (Any vertice)
    originDistance = PlaneDistance(vNormal, vPoly[0])

    # Get the distance from point1 from the plane using: Ax + By + Cz + D = (The distance from the plane)

    distance1 = ((vNormal.x * vLine[0].x)  +			# Ax +
		(vNormal.y * vLine[0].y)  +			# Bx +
		(vNormal.z * vLine[0].z)) + originDistance	# Cz + D
	
    # Get the distance from point2 from the plane using Ax + By + Cz + D = (The distance from the plane)
	
    distance2 = ((vNormal.x * vLine[1].x)  +			# Ax +
		(vNormal.y * vLine[1].y)  +			# Bx +
		(vNormal.z * vLine[1].z)) + originDistance	# Cz + D

    if(distance1 * distance2 >= 0):			# Check to see if both point's distances are both negative or both positive
	return 0					# Return false if each point has the same sign.  -1 and 1 would mean each point is on either side of the plane.  -1 -2 or 3 4 wouldn't...
					
    return 1						# The line intersected the plane, Return TRUE

################################### DOT ######################################*
#####
#####	This computers the dot product of 2 vectors
#####
################################### DOT ######################################*
def Dot(vVector1, vVector2):
    
	 #    (V1.x * V2.x        +        V1.y * V2.y        +        V1.z * V2.z)
    return ( (vVector1.x * vVector2.x) + (vVector1.y * vVector2.y) + (vVector1.z * vVector2.z) )
		
################################### ANGLE BETWEEN VECTORS ######################################*
#####
#####	This checks to see if a point is inside the ranges of a polygon
#####
################################### ANGLE BETWEEN VECTORS ######################################*
    
def AngleBetweenVectors(Vector1, Vector2):
    
    ## Get the dot product of the vectors
    dotProduct = Dot(Vector1, Vector2)	

    ## Get the product of both of the vectors magnitudes
    vectorsMagnitude = Magnitude(Vector1) * Magnitude(Vector2)
    try:
      angle = acos( dotProduct / vectorsMagnitude)
    except (ValueError):
      return 0
    # This is quite a big problem, python doesn't work with NaN..so try to
    #chatch a exception when a Nan it's returned

    #if((angle)==NaN):
    # return 0
	
    ## Return the angle in radians
    return angle 
	
################################### INTERSECTION POINT ######################################*
#####
#####	This returns the intersection point of the line that intersects the plane
#####
################################### INTERSECTION POINT ######################################*											
def IntersectionPoint(vNormal, vLine, distance):
  
    vPoint = CVector3(0.0,0.0,0.0)
    vLineDir = CVector3(0.0,0.0,0.0) 		## Variables to hold the point and the line's direction
    Numerator = 0.0
    Denominator = 0.0
    dist = 0.0

    # Here comes the confusing part.  We need to find the 3D point that is actually
    # on the plane.  Here are some steps to do that:
	
    # 1)  First we need to get the vector of our line, Then normalize it so it's a length of 1
    vLineDir = Vector(vLine[1], vLine[0])		## Get the Vector of the line
    vLineDir = Normalize(vLineDir)			## Normalize the lines vector

    Numerator = - (vNormal.x * vLine[0].x +		## Use the plane equation with the normal and the line
		   vNormal.y * vLine[0].y +
		   vNormal.z * vLine[0].z + distance)

    Denominator = Dot(vNormal, vLineDir)		## Get the dot product of the line's vector and the normal of the plane
				  
    if( Denominator == 0):				## Check so we don't divide by zero
      return vLine[0]					## Return an arbitrary point on the line

    dist = Numerator / Denominator			## Divide to get the multiplying (percentage) factor
	
    vPoint.x = (float(vLine[0].x + (vLineDir.x * dist)))
    vPoint.y = (float(vLine[0].y + (vLineDir.y * dist)))
    vPoint.z = (float(vLine[0].z + (vLineDir.z * dist)))

    return vPoint				## Return the intersection point

################################### INSIDE POLYGON ######################################*
#####
#####	This checks to see if a point is inside the ranges of a polygon
#####
################################### INSIDE POLYGON ######################################*

def InsidePolygon(vIntersection, Poly, verticeCount):

    MATCH_FACTOR = 0.9999		## Used to cover up the error in floating point
    Angle = 0.0					## Initialize the angle
    vA=CVector3(0.0,0.0,0.0) 
    vB=CVector3(0.0,0.0,0.0) 					## Create temp vectors
	
    for i in xrange(0,verticeCount,1):		## Go in a circle to each vertex and get the angle between
	
      vA = Vector(Poly[i], vIntersection)	## Subtract the intersection point from the current vertex
												## Subtract the point from the next vertex
      vB = Vector(Poly[int((i + 1) % verticeCount)], vIntersection)
												
      Angle += AngleBetweenVectors(vA, vB)	## Find the angle between the 2 vectors and add them all up as we go along
											
    if(Angle >= (MATCH_FACTOR * (2.0 * PI)) ):	## If the angle is greater than 2 PI, (360 degrees)
      return 1							## The point is inside of the polygon
		
    return 0							## If you get here, it obviously wasn't inside the polygon, so Return FALSE

################################### INTERSECTED POLYGON ######################################*
#####
#####	This checks if a line is intersecting a polygon
#####
################################### INTERSECTED POLYGON ######################################*
def IntersectedPolygon(vPoly, vLine, verticeCount):

    vNormal = CVector3(0.0,0.0,0.0)
    originDistance = 0.0

    if( not IntersectedPlane( vPoly, vLine, vNormal, originDistance)):
      return 0	
    # Python seems suffer of the same problem than java
    # Java doesn't allow pass backs (let's recalculate them)
    vNormal = Normal(vPoly)
    originDistance = PlaneDistance(vNormal, vPoly[0])
	
    vIntersection = IntersectionPoint(vNormal, vLine, originDistance)

    ## Now that we have the intersection point, we need to test if it's inside the polygon.
    ## To do this, we pass in :
    ## (our intersection point, the polygon, and the number of vertices our polygon has)

    if(InsidePolygon(vIntersection, vPoly, verticeCount)):
      return 1							## We collided!	  Return success

    ## If we get here, we must have NOT collided

    return 0								## There was no collision, so return false

################################### DISTANCE #######################################*
#####
#####	This returns the distance between 2 3D points
#####
################################### DISTANCE #######################################*
def Distance( vPoint1, vPoint2):

    # This is the classic formula used in beginning algebra to return the
    # distance between 2 points.  Since it's 3D, we just add the z dimension:
    # 
    # Distance = sqrt(  (P2.x - P1.x)^2 + (P2.y - P1.y)^2 + (P2.z - P1.z)^2 )
    #
    distance = sqrt( (vPoint2.x - vPoint1.x) * (vPoint2.x - vPoint1.x) +
		    (vPoint2.y - vPoint1.y) * (vPoint2.y - vPoint1.y) +
		    (vPoint2.z - vPoint1.z) * (vPoint2.z - vPoint1.z) )

    # Return the distance between the 2 points
    return (float(distance))

############################## CLOSET POINT ON LINE ###################################*
#####
#####	This returns the point on the line vA_vB that is closest to the point vPoint
#####
############################## CLOSET POINT ON LINE ##################################*
def ClosestPointOnLine( vA, vB, vPoint):
	
    ## Create the vector from end point vA to our point vPoint.
    vVector1 = Subtract(vPoint, vA)

    ## Create a normalized direction vector from end point vA to end point vB
    vVector2 = Normalize(Subtract(vB, vA))

    ## Use the distance formula to find the distance of the line segment (or magnitude)
    d = Distance(vA, vB)

    ## Using the dot product, we project the vVector1 onto the vector vVector2.
    ## This essentially gives us the distance from our projected vector from vA.
    t = Dot(vVector2, vVector1)

    if (t <= 0): 
      return vA

    if (t >= d) :
      return vB
 
    ## Here we create a vector that is of length t and in the direction of vVector2
    vVector3 = Multiply(vVector2 ,t)

    ## To find the closest point on the line segment, we just add vVector3 to the original
    ## end point vA.  
    vClosestPoint = Add(vA, vVector3)

    ## Return the closest point on the line segment
    return vClosestPoint	
    
############################## SPHERE POLYGON COLLISION #############################*
#####
#####	This returns true if our sphere collides with the polygon passed in
#####
############################## SPHERE POLYGON COLLISION #############################*
def SpherePolygonCollision(vPolygon, vCenter, vertexCount, radius):

    # This function is the only function we need to call for testing if a sphere
    # collides with a polygon.  The rest are just helper functions called within here.
    # The theory is actually quite difficult to understand, especially if you are
    # a beginner to collision detection and are following the tutorials in order, but
    # I will try to be as gentle and descriptive as possible.  Let go!
    # Basically, here is the overview:  
    #
    # 1) First you want to check if the sphere collides with the polygon's plane.
    #    Remember, that planes are infinite and you could be 500 units from the
    #    polygon and it's still going to trigger this first test.  We want to 
    #    write a function that classifies the sphere.  Either it's completely 
    #    in front of the plane (the side the normal is on), intersecting the
    #    plane or completely behind the plane.  Got it so far?  We created a 
    #    function called ClassifySphere() that returns BEHIND, FRONT or INTERSECTS.
    #    If ClassifySphere() returns INTERSECTS, then we move on to step 2, otherwise
    #    we did not collide with the polygon.
    # 
    # 2) The second step is to get an intersection point right in front of the sphere.
    #    This one of the tricky parts.  We know that once we have an intersection point
    #    on the plane of the polygon, we just need to use the InsidePolygon() function
    #    to see if that point is inside the dimensions of the polygon, just like we
    #    did with the Ray to Polygon Collision tutorial.  So, how do we get the point
    #    of intersection?  It's not as simple as it might sound.  Since a sphere infinite
    #    points, there would be a million points that it collided at.  You can't just
    #    draw a ray in the direction the sphere was moving because it could have just
    #    nicked the bottom of the polygon and your ray would find an intersection
    #    point that is outside of the polygon.  Well, it turns out that we need to
    #    first try and give it a shot.  We will try the first attempt a different way though.
    #    We know that we can find the normal vector of the polygon, which in essence
    #    tells us the direction that the polygon is facing.  From ClassifyPoly(),
    #	  it also returns the distance the center our sphere is from the plane.  That
    #    means we have a distance our sphere center is from the plane, and the normal
    #    tells us the direction the plane is in.  If we multiply the normal by the
    #    distance from the plane we get an offset.  This offset can then be subtracted
    #    from the center of the sphere.  Believe it or not, but we now have a position
    #    on the plane in the direction of the plane.  Usually, this intersection points
    #    works fine, but if we get around the edges of the polygon, this does not work.
    #    What we just did is also called "projecting the center of the sphere onto the plane".
    #    Another way to do this is to shoot out a ray from the center of the sphere in
    #    the opposite direction of the normal, then we find the intersection of that line
    #    and the plane.  My way just takes 3 multiplies and a subtraction.  You choose.
    #
    # 3) Once we have our psuedo intersection point, we just pass it into InsidePolygon(),
    #    along with the polygon vertices and the vertex count.  This will then return
    #    true if the intersection point was inside of the polygon, otherwise false.
    #    Remember, just because this returns false doesn't mean we stop there!  If
    #    we didn't collide yet, we need to skip to step 4.
    #
    # 4) If we get here, it's assumed that we tried our intersection point and it
    #    wasn't in the polygon's perimeter.  No fear!  There is hope!  If we get to step
    #    4, that's means that our center point is outside of the polygon's perimeter. Since
    #    we are dealing with a sphere, we could still be colliding because of the sphere's radius.
    #	  This last check requires us to find the point on each of the polygon's edges that
    #    is closest to the sphere's center.  We have a tutorial on finding this, so make sure
    #    you have read it or are comfortable with the concept.  If we are dealing with a
    #    triangle, we go through every side and get an edge vector, and calculate the closest
    #    point on those lines to our sphere's center.  After getting each closest point, we
    #    calculate the distance that point is from our sphere center.  If the distance is
    #    less than the radius of the sphere, there was a collision.  This way is pretty fast.  
    #    You don't need to calculate all three sides evey time, since the first closest point's 
    #    distance could be less than the radius and you return "true".
    #
    # That's the overview, *phew!*.  I bet you are reading this just wanting to cry because
    # that seems like so much math and theory to digest, so the code must be atrocious!
    # Well, you are partially right :)  It's not that bad actually, quite straight forward.
    # I will label the steps in the code so you can go back and forth to the overview and code.
    # I might mention that by having our CVector3 class operator overloaded it cuts down the 
    # code tremendously.  If you are confused with this concept of C++, just create functions
    # to add, subtract and multiply vectors or scalars together.
    #

    # 1) STEP ONE - Finding the sphere's classification
	
    # Let's use our Normal() function to return us the normal to this polygon
    vNormal = Normal(vPolygon)

    # This will store the distance our sphere is from the plane
    distance = 0.0
    
    classification = ClassifySphere(vCenter, vNormal, vPolygon[0], radius, distance)

    d = (float(PlaneDistance(vNormal, vPolygon[0])))
    
    distance = (vNormal.x * vCenter.x + vNormal.y * vCenter.y + vNormal.z * vCenter.z + d)
    # This is where we determine if the sphere is in FRONT, BEHIND, or INTERSECTS the plane
    # of the polygon.  We pass is our sphere center, the polygon's normal, a point on
    # the plane (vertex), the sphere's radius and an empty float to fill the distance with.
  
    # If the sphere intersects the polygon's plane, then we need to check further,
    # otherwise the sphere did NOT intersect the polygon.  Pretty fast so far huh?
    if(classification == INTERSECTS):
	
      # 2) STEP TWO - Finding the psuedo intersection point on the plane

      # Now we want to project the sphere's center onto the polygon's plane,
      # in the direction of the normal.  This is done by multiplying the "normal"
      # by the "distance" the sphere center is from the plane.  We got the distance
      # from the ClassifySphere() function call up above.  2 return values were given
      # through the "distance" variable being passed in as a reference.  If projecting
      # is confusing to you, just think of it as this: "I am starting at the center
      # of the sphere and I am going to just run into the plane.  I will move in the 
      # direction that is reverse from the normal.  When do I know when to stop?  Well,
      # I just go in that direction until my distance from the center is the same as
      # the distance the center of the sphere is from the plane."  By doing this
      # we get an offset to subtract from the center of the sphere.
      vOffset = Multiply(vNormal ,distance)

      # Once we have the offset to the plane, we just subtract it from the center
      # of the sphere.  "vPosition" now a point that lies on the plane of the polygon.
      # Whether it is inside the polygon's perimeter is another story.  Usually it
      # is though, unless we get near the edges of the polygon.
      vPosition = Subtract(vCenter ,vOffset)

      # 3) STEP THREE - Check if the intersection point is inside the polygons perimeter

      # This is the same function used in our previous tutorial on Ray to Polygon Collision.
      # If the intersection point is inside the perimeter of the polygon, it returns true.
      # We pass in the intersection point, the list of vertices and vertex count of the poly.
      if(InsidePolygon(vPosition, vPolygon, vertexCount)):
        return 1	# We collided!
      # 4) STEP FOUR - Check the sphere to see if it intersects the polygon edges

      # If we get here, we didn't find an intersection point in the perimeter.
      # There is still one more chance to redeem our sphere that it can hit the mark.
      # If any part of the sphere intersects the edges of the polygon, we collided.  
      # This is only checked if the sphere's center point is outside the edges of the
      # polygon. We pass in the center of the sphere, the list of verts, the polygon 
      # vertex count and the sphere's radius.  If this returns true we have a collision.
      else:
        if(EdgeSphereCollision(vCenter, vPolygon, vertexCount, radius)):
          return 1	## We collided! "And you doubted me..." - Sphere

    # If we get here, there is obviously no collision happening up in this crib
    return 0

################################# CLASSIFY SPHERE ################################*
#####
#####	This tells if a sphere is BEHIND, in FRONT, or INTERSECTS a plane, also it's distance
#####
################################# CLASSIFY SPHERE ################################*
def ClassifySphere(vCenter, vNormal, vPoint, radius, distance):

    # First we need to find the distance our polygon plane is from the origin.
    # We need this for the distance formula below.
    d = (float(PlaneDistance(vNormal, vPoint)))

    # Here we use the famous distance formula to find the distance the center point
    # of the sphere is from the polygon's plane.  
    # Remember that the formula is Ax + By + Cz + d = 0 with ABC = Normal, XYZ = Point
    distance = ((((vNormal.x * vCenter.x) + (vNormal.y * vCenter.y)) + (vNormal.z * vCenter.z)) + d)
    # Now we query the information just gathered.  Here is how Sphere Plane Collision works:
    # If the distance the center is from the plane is less than the radius of the sphere,
    # we know that it must be intersecting the plane.  We take the absolute value of the
    # distance when we do this check because once the center of the sphere goes behind
    # the polygon, the distance turns into negative numbers (with 0 being that the center
    # is exactly on the plane).  What do I mean when I say "behind" the polygon?  How do
    # we know which side is the front or back side?  Well, the side with the normal pointing
    # out from it is the front side, the other side is the back side.  This is all dependant
    # on the direction the vertices stored.  I recommend drawing them counter-clockwise.
    # if you go clockwise the normal with then point the opposite way and will screw up
    # everything.
    # So, if we want to find if the sphere is in front of the plane, we just make sure
    # the distance is greater than or equal to the radius.  let's say we have a radius
    # of 5, and the distance the center is from the plane is 6; it's obvious that the
    # sphere is 1 unit away from the plane.
    # If the sphere isn't intersecting or in front of the plane, it HAS to be BEHIND it.

    # If the absolute value of the distance we just found is less than the radius, 
    # the sphere intersected the plane.
    if(Absolute(distance) < radius):
      return INTERSECTS
    # Else, if the distance is greater than or equal to the radius, the sphere is
    # completely in FRONT of the plane.
    elif(distance >= radius):
      return FRONT
	
    # If the sphere isn't intersecting or in FRONT of the plane, it must be BEHIND
    return BEHIND

################################# EDGE SPHERE COLLSIION \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
#####
#####	This returns true if the sphere is intersecting any of the edges of the polygon
#####
################################# EDGE SPHERE COLLSIION \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
def EdgeSphereCollision(vCenter, vPolygon, vertexCount, radius):

    vPoint=CVector3(0.0,0.0,0.0) 

    # This function takes in the sphere's center, the polygon's vertices, the vertex count
    # and the radius of the sphere.  We will return true from this function if the sphere
    # is intersecting any of the edges of the polygon.  How it works is, every edge line
    # segment finds the closest point on that line to the center of the sphere.  If the
    # distance from that closest point is less than the radius of the sphere, there was
    # a collision.  Otherwise, we are definately out of reach of the polygon.  This works
    # for triangles, quads, and any other convex polygons.

    # Go through all of the vertices in the polygon
    for i in xrange(0, vertexCount,1):

      # This returns the closest point on the current edge to the center of the sphere.
      # Notice that we mod the second point of the edge by our vertex count.  This is
      # so that when we get to the last edge of the polygon, the second vertex of the
      # edge is the first vertex that we starting with.  
      vPoint = ClosestPointOnLine(vPolygon[i], vPolygon[((i + 1) % vertexCount)], vCenter)
		
      # Now, we want to calculate the distance between the closest point and the center
      distance = Distance(vPoint, vCenter)
	
      # If the distance is less than the radius, there must be a collision so return true
      if(distance < radius):
	return 1


    # The was no intersection of the sphere and the edges of the polygon
    return 0
				 
####################################### OPERATIONS ###################################*
#####
##### theese are others varius operations, they returns vectors
#####
####################################### OPERATIONS ###################################*  
def Add(v1, v2):
    return CVector3(v1.x+v2.x, v1.y+v2.y, v1.z+v2.z)

def Subtract(v1, v2):
    return CVector3(v1.x-v2.x, v1.y-v2.y, v1.z-v2.z)

def Multiply(v1, m):
    return CVector3(v1.x*m, v1.y*m, v1.z*m)

def Divide( v1, m):
    return CVector3(v1.x/m, v1.y/m, v1.z/m)    
