#!/usr/bin/env python

##########################################################################*                       
######	                    GAME TUTORIAL 33 MODULE
######
######    Federico B. 2003 - dragonscraft@yahoo.it
######
################################# IMPORTS ################################*

from OpenGL.GL import *
from OpenGL.GLU import *
import pygame
from pygame.locals import *
import pygame, pygame.key, pygame.display, pygame.mouse
from string import *

############################# LOCAL IMPORTS ##############################*

from Math3D import *
from Camera import *
################################# MODULE #################################*

# This will hold the number of vertices that were found in "World.raw"
g_NumberOfVerts = 0

# This is our 3D point array that will store all of the world vertices
g_vWorld=[]

g_Camera= CCamera()

def LoadVertices():
    global g_vWorld, g_NumberOfVerts
    
    f = open(os.path.join("data", "world.raw"))
    lines = f.readlines()
    f.close()
    for line in lines:
      g_NumberOfVerts += 1
      vals = split(line)       
      g_vWorld.append(CVector3((float(vals[0])),(float(vals[1])),(float(vals[2]))))

def resize((width, height)):
    if height==0:
        height=1
    glViewport(0, 0, width, height)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(45, 1.0*width/height, 0.1, 150.0)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

def init():  
  
    glShadeModel(GL_SMOOTH)
      
    glEnable(GL_DEPTH_TEST)			# Enables Depth Testing
    glDepthFunc(GL_LEQUAL)
    
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)
    
    g_Camera.PositionCamera(10, 4, 12,   9, 4, 12,   0, 1, 0)
    # We need to specify our camera's radius in the beginning, I chose 1.
    g_Camera.SetCameraRadius(1)
    # This loads the vertices from the file into our global world array
    LoadVertices()

    # We are going to enable backface culling.  If you are unfamiliar with
    # this term, it basically means that we don't draw 2 side polygons.
    # The only side drawn is the side with the normal coming out of it.
    # by enabling backface culling, it cuts down our fill rate bottleneck.
    # Since our world is composed of 2D boxes, we will never see the inside of
    # the boxes, so there is no need to draw them.  We can change GL_BACK to
    # GL_FRONT for an interesting effect.
    glCullFace(GL_BACK)							# Don't draw the back sides of polygons
    glEnable(GL_CULL_FACE)					    # Turn on culling

    glClearColor(0.0, 0.0, 0.0, 1)			    # Turn the background color black

    fogColor = (0.0, 0.0, 0.0, 1.0)		        # Let's make the Fog Color black too

    glFogi(GL_FOG_MODE, GL_EXP2)			    # Set The Fog Mode
    glFogfv(GL_FOG_COLOR, fogColor)			    # Set The Fog Color
    glFogf(GL_FOG_DENSITY, 0.045)			    # Set How Dense Will The Fog Be
    glHint(GL_FOG_HINT, GL_DONT_CARE)			   # Set The Fog's calculation accuracy
    glFogf(GL_FOG_START, 0)						# Set The Fog's Start Depth
    glFogf(GL_FOG_END, 50.0)				    # Set The Fog's End Depth

    glEnable(GL_FOG)

def draw():
    global g_vWorld, gNumberOfVerts, textures

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)  # Clear The Screen And The Depth Buffer
    glLoadIdentity()
    
    g_Camera.CheckCameraCollision(g_vWorld, g_NumberOfVerts)

    # Position our camera
    g_Camera.Look()

    # Since we have the vertices for the world in the correct order, let's create
    # a loop that goes through all of the vertices and passes them in to be rendered.
    
    glBegin(GL_TRIANGLES)	    # This is our BEGIN to draw the polys
    # Go through all the vertices and draw them
    for i in xrange(0,g_NumberOfVerts,3):	
      glVertex3f(g_vWorld[i].x, g_vWorld[i].y, g_vWorld[i].z)
      glVertex3f(g_vWorld[i+1].x, g_vWorld[i+1].y, g_vWorld[i+1].z)
      glVertex3f(g_vWorld[i+2].x, g_vWorld[i+2].y, g_vWorld[i+2].z)	
    glEnd()					
   
def handleKeyPress(key):
    
    if key==K_ESCAPE:
      return 0
    g_Camera.CheckForMovement(key)
    return 1

def main():

    video_flags = OPENGL|DOUBLEBUF|RESIZABLE
    
    pygame.init()
    pygame.display.set_mode((640,480), video_flags)
    pygame.key.set_repeat(10,10) 

    resize((640,480))
    init()
    frames = 0
    done = 0
    ticks = pygame.time.get_ticks() 
    while not done:
      while 1:
        event = pygame.event.poll()
        if event.type == NOEVENT:
          break
        if event.type == KEYDOWN:
          if handleKeyPress(event.key) == 0:
            done = 1
        if event.type == QUIT:
          done = 1
      #Loop the scene
      draw()
      
      # We now just need to call the look from the camera object to
      # deal with the input and updating.   
      g_Camera.Update(640,480,pygame.mouse)	
      # Here we check for mouse and key movements every frame
      pygame.display.flip()
      frames = frames+1

    print "fps:  %d" % ((frames*1000)/(pygame.time.get_ticks()-ticks))

if __name__ == '__main__': main()
