#!/usr/bin/env python

#*****************************************************************************
# Pygame, PyOpenGL and Numpy version of Nehe's OpenGL lesson11                           
#
# Federico B. 2003 - dragonscraft@yahoo.it
#
# it runs around 17 fps (bleah!). Plz if someone optimize it and makes it runs
# in a decent framerate let me know it, thnkz.
#*****************************************************************************

import os
import Numeric
from math import sin
from OpenGL.GL import *
from OpenGL.GLU import *
from pygame.locals import *
import pygame, pygame.image

# The Array For The Points "On The Grid Of Our Wave", for performance we use half poly
points = Numeric.zeros((23,23,3), Numeric.Float) 
# Counter Used To Control How Fast Flag Waves
wiggle_count = 0
#hold = 0.0 # Temporarily Holds A Floating Point Value "It's no sense make it global"

textures = []

xrot=0.0
yrot=0.0
zrot=0.0

def loadGLtextures():
  global textures
  
  texturefile = os.path.join('data','tim.bmp')
  textureSurface = pygame.image.load(texturefile)
  textureData = pygame.image.tostring(textureSurface, "RGBX", 1)
  
  textures = glGenTextures(2)
 
  glBindTexture(GL_TEXTURE_2D, textures[0])
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST )
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR )
  gluBuild2DMipmaps( GL_TEXTURE_2D, GL_RGBA, textureSurface.get_width(), textureSurface.get_height(),GL_RGBA, GL_UNSIGNED_BYTE, textureData)
                
def handleKeyPress(key):
    
  if key==K_ESCAPE:
    return 0
  
  return 1     

def resize_window((width, height)):
  if height==0:
     height=1
  glViewport(0, 0, width, height)
  glMatrixMode(GL_PROJECTION)
  glLoadIdentity()
  gluPerspective(45, (float(width)/float(height)), 0.1, 100.0)
  glMatrixMode(GL_MODELVIEW)
  glLoadIdentity()  

def initGL():
  global points

  x,y = 0,0
  loadGLtextures() 
  glEnable(GL_TEXTURE_2D)           
  glClearColor(0.0, 0.0, 0.0, 0.5)
  glClearDepth(1.0)
  glDepthFunc(GL_LEQUAL)
  glEnable(GL_DEPTH_TEST)
  glShadeModel( GL_SMOOTH )   
  glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST )

  glPolygonMode( GL_BACK, GL_FILL )
  glPolygonMode( GL_FRONT, GL_LINE )

  #Fill the array, here we create the wave
  for y in xrange(0, 23, 1):
    for x in xrange(0, 23, 1):
      points[x,y,0]=((float(x)/5.0) - 4.5 )
      points[x,y,1]=((float(y)/5.0) - 4.5 )
      points[x,y,2]=(sin((((float(x)/ 5.0)* 40.0)/ 360.0) * 3.14159 * 2.0 ) )    
      
def DrawGLScene():
  global xrot, yrot, zrot
  global textures
  global wiggle_count
  global points
  
  x, y = 0, 0
  f_x, f_y, f_xb, f_yb = 0.0, 0.0, 0.0, 0.0

  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT )
  glLoadIdentity()
  
  glTranslatef( 0.0, 0.0, -6.0 )
  
  glRotatef( xrot, 1.0, 0.0, 0.0 )
  glRotatef( yrot, 0.0, 1.0, 0.0 )
  glRotatef( zrot, 0.0, 0.0, 1.0 )
  
  glBindTexture( GL_TEXTURE_2D, textures[0] )   
  
  #Start Drawing Our Quads
  glBegin( GL_QUADS ) 
  for y in xrange(0,22,1):
    for x in xrange(0,22,1):
    
      f_x = ( float(x)/ 22.0 )
      f_y = ( float(y)/ 22.0 )
      f_xb = ( float(x+1)/ 22.0 )
      f_yb = ( float(y+1)/ 22.0 )
	  
      # First Texture Coordinate (Bottom Left) */
      glTexCoord2f( f_x, f_y )
      glVertex3f( points[x,y,0], points[x,y,1],points[x,y,2] )

      # Second Texture Coordinate (Top Left) */
      glTexCoord2f( f_x, f_yb )
      glVertex3f( points[x,(y+1),0], points[x,(y+1),1],points[x,(y+1),2] )

      # Third Texture Coordinate (Top Right) */
      glTexCoord2f( f_xb, f_yb )
      glVertex3f( points[(x+1),(y+1),0],points[(x+1),y + 1,1],points[(x+1),(y+1),2] )

      # Fourth Texture Coordinate (Bottom Right) */
      glTexCoord2f( f_xb, f_y )
      glVertex3f( points[(x+1),y,0], points[(x+1),y,1],points[(x+1),y,2] )
  glEnd()
  
  # This is a NICER MOVMENT version kept by gcwin ver. Use this if u want
  #if wiggle_count==2:
    #for y in xrange(0,45,1):
      #points[44,y,2] = points[0,y,2]
    #for x in xrange(0,44,1):
      #for y in xrange(0,45,1):
        #points[x,y,2] = points[(x+1),y,2]
    #wiggle_count=0
  #wiggle_count += 1     

  #Every two times we slow down the wave
  if wiggle_count == 2:
    for y in xrange(0,23,1):
      hold = points[0,y,2]
      for x in xrange(0,22,1):
      	points[x,y,2] = points[(x+1),y,2]
      points[22,y,2] = hold
    wiggle_count = 0    
  wiggle_count += 1 # i can't find out another way to postincrement it

  # Make the flag rotating
  xrot += 0.3
  yrot += 0.2
  zrot += 0.4
  
def main():

  video_flags = OPENGL|DOUBLEBUF|RESIZABLE    
  pygame.init()
  surface = pygame.display.set_mode((640,480), video_flags)
  resize_window((640,480))

  initGL()
  
  frames = 0
  done = 0
  ticks = pygame.time.get_ticks()

  while not done:
    while 1:
      event = pygame.event.poll()
      if event.type == NOEVENT:
        break
      if event.type == KEYDOWN:
        if handleKeyPress(event.key) == 0:
          done = 1
      if event.type == QUIT:
        done = 1
      #Loop the scene
    DrawGLScene()
    pygame.display.flip()
    frames = frames+1

  print "fps:  %d" % ((frames*1000)/(pygame.time.get_ticks()-ticks))

if __name__ == '__main__': main()
