#!/usr/bin/env python

#*****************************************************************************
# Pygame, PyOpenGL and Numpy version of Nehe's OpenGL lesson12                           
#
# Federico B. 2003 - dragonscraft@yahoo.it
#
#*****************************************************************************

import os
import Numeric
from math import sin
from OpenGL.GL import *
from OpenGL.GLU import *
from pygame.locals import *
import pygame, pygame.image, pygame.key

texture = []

box = []
top = []
xloop = 0
yloop = 0

xrot = 0.0
yrot = 0.0

# Array For Box Colors  BRIGHT
boxcol = Numeric.array([[1.0,0.0,0.0],[1.0,0.5,0.0],[1.0,1.0,0.0],[0.0,1.0,0.0],[0.0,1.0,1.0]])   				   					 
# Array For Top Colors  DARK
topcol = Numeric.array([[0.5,0.0,0.0],[0.5,0.25,0.0],[0.5,0.5,0.0],[0.0,0.5,0.0],[0.0,0.5,0.5]])

def BuildList():  
  global box
  global top
  
  box = glGenLists(2)	
  
  glNewList( box, GL_COMPILE )
  glBegin( GL_QUADS )
  # Bottom Face
  glNormal3f( 0.0,-1.0, 0.0)
  glTexCoord2f( 0.0, 1.0 )
  glVertex3f( -1.0, -1.0, -1.0 )
  glTexCoord2f( 1.0, 1.0 )
  glVertex3f(  1.0, -1.0, -1.0 )
  glTexCoord2f( 1.0, 0.0 )
  glVertex3f(  1.0, -1.0,  1.0 )
  glTexCoord2f( 0.0, 0.0 )
  glVertex3f( -1.0, -1.0,  1.0 )

  # Front Face
  glNormal3f( 0.0, 0.0, 1.0)
  glTexCoord2f( 1.0, 0.0 )
  glVertex3f( -1.0, -1.0,  1.0 )
  glTexCoord2f( 0.0, 0.0 )
  glVertex3f(  1.0, -1.0,  1.0 )
  glTexCoord2f( 0.0, 1.0 )
  glVertex3f(  1.0,  1.0,  1.0 )
  glTexCoord2f( 1.0, 1.0 )
  glVertex3f( -1.0,  1.0,  1.0 )

  # Back Face
  glNormal3f( 0.0, 0.0,-1.0)
  glTexCoord2f( 0.0, 0.0 )
  glVertex3f( -1.0, -1.0, -1.0 )
  glTexCoord2f( 0.0, 1.0 )
  glVertex3f( -1.0,  1.0, -1.0 )
  glTexCoord2f( 1.0, 1.0 )
  glVertex3f(  1.0,  1.0, -1.0 )
  glTexCoord2f( 1.0, 0.0 )
  glVertex3f(  1.0, -1.0, -1.0 )

  # Right face 
  glNormal3f( 1.0, 0.0, 0.0)
  glTexCoord2f( 0.0, 0.0 )
  glVertex3f( 1.0, -1.0, -1.0 )
  glTexCoord2f( 0.0, 1.0 )
  glVertex3f( 1.0,  1.0, -1.0 )
  glTexCoord2f( 1.0, 1.0 )
  glVertex3f( 1.0,  1.0,  1.0 )
  glTexCoord2f( 1.0, 0.0 )
  glVertex3f( 1.0, -1.0,  1.0 )

  # Left Face 
  glNormal3f(-1.0, 0.0, 0.0)
  glTexCoord2f( 1.0, 0.0 )
  glVertex3f( -1.0, -1.0, -1.0 )
  glTexCoord2f( 0.0, 0.0 )
  glVertex3f( -1.0, -1.0,  1.0 )
  glTexCoord2f( 0.0, 1.0 )
  glVertex3f( -1.0,  1.0,  1.0 )
  glTexCoord2f( 1.0, 1.0 )
  glVertex3f( -1.0,  1.0, -1.0 )
  glEnd()
  glEndList()	
  
  top = box + 1
  
  # New compiled list, top
  glNewList( top, GL_COMPILE )
  glBegin( GL_QUADS ) # Start Drawing Quad 
  # Top Face 
  glNormal3f( 0.0, 1.0, 0.0);
  glTexCoord2f( 1.0, 1.0 )
  glVertex3f( -1.0,  1.0, -1.0 )
  glTexCoord2f( 1.0, 0.0 )
  glVertex3f( -1.0,  1.0,  1.0 )
  glTexCoord2f( 0.0, 0.0 )
  glVertex3f(  1.0,  1.0,  1.0 )
  glTexCoord2f( 0.0, 1.0 )
  glVertex3f(  1.0,  1.0, -1.0 )
  glEnd()
  glEndList()
  
def LoadGLtextures():
  global textures
  
  texturefile = os.path.join('data','cube.bmp')
  textureSurface = pygame.image.load(texturefile)
  textureData = pygame.image.tostring(textureSurface, "RGBX", 1)
  
  textures = glGenTextures(2)
 
  glBindTexture(GL_TEXTURE_2D, textures[0])
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
  glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, textureSurface.get_width(), textureSurface.get_height(), 0,
                GL_RGBA, GL_UNSIGNED_BYTE, textureData )	
 
def handleKeyPress(key):
  global xrot, yrot
    
  if key==K_ESCAPE:
    return 0
  if key==K_RIGHT:
    yrot += 0.2
  if key==K_LEFT:
    yrot -= 0.2
  if key==K_UP:
    xrot -= 0.2
  if key==K_DOWN:
    xrot += 0.2

  return 1     

def resize_window((width, height)):
  if height==0:
     height=1
  glViewport(0, 0, width, height)
  glMatrixMode(GL_PROJECTION)
  glLoadIdentity()
  gluPerspective(45, (float(width)/float(height)), 0.1, 100.0)
  glMatrixMode(GL_MODELVIEW)
  glLoadIdentity() 
  
def initGL():
  LoadGLtextures()
  BuildList()
  glEnable(GL_TEXTURE_2D)
  glShadeModel( GL_SMOOTH )            
  glClearColor(0.0, 0.0, 0.0, 0.5)
  glClearDepth(1.0)
  glEnable(GL_DEPTH_TEST)
  glDepthFunc(GL_LEQUAL) 
  # Enable lighting 
  glEnable( GL_LIGHT0 )         # Quick And Dirty Lighting 
  glEnable( GL_LIGHTING )       # Enable Lighting 
  glEnable( GL_COLOR_MATERIAL ) # Enable Material Coloring 
  glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST )
  
def DrawGLScene():
  global xrot, yrot
  global box, top
  global boxcol, topcol
  global textures
  global xloop, yloop

  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT )

  glBindTexture( GL_TEXTURE_2D, textures[0] )  
  
  for yloop in xrange(0,6,1):
    for xloop in xrange(0,yloop,1):
      
      glLoadIdentity()
      # Position The Cubes On The Screen 
      glTranslatef(((1.4+(float(xloop)*2.8))-(float(yloop)*1.4)),(((6.0-float(yloop))*2.4)-7.0),-20.0)  
      # Tilt the cubes 
      # Tilt The Cubes Up And Down 
      glRotatef( (45.0 - ( 2.0 * float(yloop) ) + xrot),1.0, 0.0, 0.0 )
      # Spin Cubes Left And Right 
      glRotatef( (45.0 + yrot), 0.0, 1.0, 0.0 )
      
      #This is the main difference with C version brcause the Numpy array
      #we use glColor3f insetad of glColor3fv
      glColor3f( (boxcol[yloop - 1,0]),(boxcol[yloop - 1,1]),(boxcol[yloop - 1,2]) ) # Select A Box Color 
      glCallList( box )              # Draw the box 
		    
      glColor3f( (topcol[yloop - 1,0]),(topcol[yloop - 1,1]),(topcol[yloop - 1,2]) ) # Select The Top Color
      glCallList( top )              # Draw The Top 	
	  
def main():

  video_flags = OPENGL|DOUBLEBUF | RESIZABLE
    
  pygame.init()
  surface = pygame.display.set_mode((640,480), video_flags)

  pygame.key.set_repeat(10,10)
  
  resize_window((640,480))

  initGL()
  
  frames = 0
  done = 0
  ticks = pygame.time.get_ticks()
  
  while not done:
    while 1:
      event = pygame.event.poll()
      if event.type == NOEVENT:
        break
      if event.type == KEYDOWN:
        if handleKeyPress(event.key) == 0:
          done = 1
      if event.type == QUIT:
        done = 1
      #Loop the scene
    DrawGLScene()
    pygame.display.flip()
    frames = frames+1

  print "fps:  %d" % ((frames*1000)/(pygame.time.get_ticks()-ticks))

if __name__ == '__main__': main()	  		                         	
