#!/usr/bin/env python

#*****************************************************************************
# Pygame, PyOpenGL and Numpy version of Nehe's OpenGL lesson13                           
#
# Federico B. 2003 - dragonscraft@yahoo.it
#
#*****************************************************************************

import os
import sys
import Numeric
from math import sin, cos
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from pygame.locals import *
import pygame, pygame.image, pygame.key, pygame.font, pygame.display


xpos = 0.0
ypos = 0.0
zpos = -2.0

cnt1 = 0
cnt2 = 0

def resize_window((width, height)):
  if height==0:
     height=1
  glViewport(0, 0, width, height)
  glMatrixMode(GL_PROJECTION)
  glLoadIdentity()
  gluPerspective(45, (float(width)/float(height)), 0.1, 100.0)
  glMatrixMode(GL_MODELVIEW)
  glLoadIdentity()   
      
  
def glPrint(s):     
  string = s
  for i in range(len(string)):
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, ord(string[i]))
  
def initGL():
  glClearColor(0.0, 0.0, 0.0, 0.0)
  glClearDepth(1.0)
  glEnable(GL_DEPTH_TEST)
  glDepthFunc(GL_LEQUAL)
  glShadeModel(GL_SMOOTH)
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)
  
def DrawGLScene():
  global xpos, ypos, zpos
  global cnt1, cnt2

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
  glLoadIdentity()
  glTranslatef(xpos, ypos, zpos)
  glColor3f(1.0*(float(cos(cnt1))),1.0*(float(sin(cnt2))),1.0-0.5*(float(cos(cnt1+cnt2))))
  glRasterPos2f((-0.65+0.05*(float(cos(cnt1)))), (0.32*(float(sin(cnt2)))))
  glPrint("Active OpenGL Text With NeHe - %s " % (cnt1))
  
  cnt1+=0.102;					
  cnt2+=0.010;
  

def handleKeyPress(key):
  global xpos, zpos, ypos
    
  if key==K_ESCAPE:
    return 0
  if key==K_RIGHT:
    xpos += 0.1
  if key==K_LEFT:
    xpos -= 0.1
  if key==K_UP:
    ypos += 0.1
  if key==K_DOWN:
    ypos -= 0.1

def main():
  height, width = 640, 480
  
  pygame.init()
  pygame.display.init()
  video_flags = OPENGL|DOUBLEBUF|RESIZABLE
  pygame.display.set_mode((height,width), video_flags)
  pygame.key.set_repeat(20,50) 
  initGL()
  resize_window((height, width))
  
  
  frames = 0
  done = 0
  ticks = pygame.time.get_ticks() 
  while not done:
    while 1:
      event = pygame.event.poll()
      if event.type == NOEVENT:
        break
      if event.type == KEYDOWN:
        if handleKeyPress(event.key) == 0:
          done = 1
      if event.type == QUIT:
        done = 1
    #Loop the scene
    DrawGLScene()
    pygame.display.flip()
    frames = frames+1

  print "fps:  %d" % ((frames*1000)/(pygame.time.get_ticks()-ticks))

if __name__ == '__main__': main()	  		                         	
