#!/usr/bin/env python

#*****************************************************************************
# Pygame, PyOpenGL and Numpy version of Nehe's OpenGL lesson15                           
#
# Federico B. 2003 - dragonscraft@yahoo.it
#
#Sigh! Texturing doesn't want work
#*****************************************************************************

import os
import sys
import Numeric
from math import sin, cos
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from pygame.locals import *
import pygame, pygame.image, pygame.key, pygame.font, pygame.display

g_rot = 0.0

textures = []

def loadGLtextures():
  global textures
  
  texturefile = os.path.join('data','Lights.bmp')
  textureSurface = pygame.image.load(texturefile)
  textureData = pygame.image.tostring(textureSurface, "RGBX", 1)
  
  textures = glGenTextures(2)
 
  glBindTexture(GL_TEXTURE_2D, textures[0])
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
  glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, textureSurface.get_width(), textureSurface.get_height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, textureData )
  # Texturing Contour Anchored To The Object
  glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR)
  # Texturing Contour Anchored To The Object
  glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR)
  glEnable(GL_TEXTURE_GEN_S)			
  glEnable(GL_TEXTURE_GEN_T)         

def resize_window((width, height)):
  if height==0:
     height=1
  glViewport(0, 0, width, height)
  glMatrixMode(GL_PROJECTION)
  glLoadIdentity()
  gluPerspective(45, (float(width)/float(height)), 0.1, 100.0)
  glMatrixMode(GL_MODELVIEW)
  glLoadIdentity()   
      

def render_stroke_string(font, string):

  glPushMatrix()
  for i in range(len(string)):
    glutStrokeCharacter(font, ord(string[i]))
  glPopMatrix()
  
def initGL():
  global textures
  
  loadGLtextures() 
  glShadeModel(GL_SMOOTH)
  glClearColor(0.0, 0.0, 0.0, 0.0)
  glClearDepth(1.0)  
  glEnable(GL_DEPTH_TEST)
  glDepthFunc(GL_LEQUAL)  
  glEnable(GL_LIGHT0)			
  glEnable(GL_LIGHTING)	
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)	
  glLineWidth(6.0) 
  glEnable(GL_TEXTURE_2D)
  glBindTexture(GL_TEXTURE_2D, textures[0])
  
def DrawGLScene():
  global g_rot

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
  glLoadIdentity()
  glTranslatef(1.1*(cos(g_rot/16.0)),0.8*(sin(g_rot/20.0)),-3.0)
  glRotatef(g_rot,1.0,0.0,0.0)				
  glRotatef(g_rot*1.0,0.0,1.0,0.0)				
  glRotatef(g_rot*1.4,0.0,0.0,1.0)			
  glScalef(0.005, 0.005, 0.0)
  glTranslatef(-0.35,-0.35,0.1)
  glColor3f(1.0*(cos(g_rot/20.0)),1.0*(sin(g_rot/25.0)),1.0-0.5*(cos(g_rot/17.0)))
  string= "CIAO"
  render_stroke_string(GLUT_STROKE_ROMAN, string)
  g_rot += 0.1

def handleKeyPress(key):
  global xpos, zpos, ypos
    
  if key==K_ESCAPE:
    return 0

def main():
  height, width = 640, 480
  
  pygame.init()
  pygame.display.init()
  video_flags = OPENGL|DOUBLEBUF|RESIZABLE
  pygame.display.set_mode((height,width), video_flags)
  pygame.key.set_repeat(20,50) 
  initGL()
  resize_window((height, width))
  
  
  frames = 0
  done = 0
  ticks = pygame.time.get_ticks() 
  while not done:
    while 1:
      event = pygame.event.poll()
      if event.type == NOEVENT:
        break
      if event.type == KEYDOWN:
        if handleKeyPress(event.key) == 0:
          done = 1
      if event.type == QUIT:
        done = 1
    #Loop the scene
    DrawGLScene()
    pygame.display.flip()
    frames = frames+1

  print "fps:  %d" % ((frames*1000)/(pygame.time.get_ticks()-ticks))

if __name__ == '__main__': main()	  		                         	
