#!/usr/bin/env python

#*****************************************************************************
# Pygame, PyOpenGL and Numpy version of Nehe's OpenGL lesson17                          
#
# Federico B. 2003 - dragonscraft@yahoo.it
#
#*****************************************************************************

import os
from math import sin, cos
from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image, pygame.key
from pygame.locals import *

textures = []

base = []

cnt1=0.0
cnt2=0.0

def handleKeyPress(key):
    
  if key==K_ESCAPE:
    return 0

def resize_window((width, height)):
    if height==0:
        height=1
    glViewport(0, 0, width, height)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(45, 1.0*width/height, 0.1, 100.0)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()   

def load_textures():
    global textures
    
    texturefile = os.path.join('data','Font.bmp')
    textureSurface = pygame.image.load(texturefile)
    textureData = pygame.image.tostring(textureSurface, "RGBX", 1)
    
    texturefile1 = os.path.join('data','Bumps.bmp')
    textureSurface1 = pygame.image.load(texturefile1)
    textureData1 = pygame.image.tostring(textureSurface1, "RGBX", 1)

    textures = glGenTextures(2)

    glBindTexture(GL_TEXTURE_2D, textures[0])
    glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, textureSurface.get_width(), textureSurface.get_height(), 0,
                  GL_RGBA, GL_UNSIGNED_BYTE, textureData )
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)

    glBindTexture(GL_TEXTURE_2D, textures[1])
    glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, textureSurface1.get_width(), textureSurface1.get_height(), 0,
                  GL_RGBA, GL_UNSIGNED_BYTE, textureData1 )
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)

def BuildFont():
  global textures
  global base

  cx = 0.0					# Holds Our X Character Coord
  cy = 0.0					# Holds Our Y Character Coord
  base=glGenLists(256)				# Creating 256 Display Lists
  glBindTexture(GL_TEXTURE_2D, textures[0])	# Select Our Font Texture
  for loop in xrange(0,256,1):			   # Loop Through All 256 Lists
  
    cx=float(loop%16)/16.0			# X Position Of Current Character
    cy=float(loop/16)/16.0			# Y Position Of Current Character
    
    glNewList(base+loop,GL_COMPILE)		# Start Building A List
    glBegin(GL_QUADS)				# Use A Quad For Each Character
    glTexCoord2f(cx,1-cy-0.0625)		# Texture Coord (Bottom Left)
    glVertex2i(0,0)				# Vertex Coord (Bottom Left)
    glTexCoord2f(cx+0.0625,1-cy-0.0625)	        # Texture Coord (Bottom Right)
    glVertex2i(16,0)				# Vertex Coord (Bottom Right)
    glTexCoord2f(cx+0.0625,1-cy)		# Texture Coord (Top Right)
    glVertex2i(16,16)				# Vertex Coord (Top Right)
    glTexCoord2f(cx,1-cy)			# Texture Coord (Top Left)
    glVertex2i(0,16)			        # Vertex Coord (Top Left)
    glEnd()					# Done Building Our Quad (Character)    
    glTranslated(10,0,0)			# Move To The Right Of The Character
    glEndList()	
	
def glPrint(x,y,string,set):
  global textures
  global base
  
  if set>1:
    set=1
    
  glBindTexture(GL_TEXTURE_2D, textures[0])	# Select Our Font Texture
  glDisable(GL_DEPTH_TEST)			# Disables Depth Testing
  glMatrixMode(GL_PROJECTION)			# Select The Projection Matrix
  glPushMatrix()				# Store The Projection Matrix
  glLoadIdentity()  				# Reset The Projection Matrix
  glOrtho(0,640,0,480,-1,1)			# Set Up An Ortho Screen
  glMatrixMode(GL_MODELVIEW)			# Select The Modelview Matrix
  glPushMatrix()				# Store The Modelview Matrix
  glLoadIdentity()				# Reset The Modelview Matrix
  glTranslated(x,y,0)				# Position The Text (0,0 - Bottom Left)
  glListBase(base-32+(128*set))			# Choose The Font Set (0 or 1)
  glCallLists(string)		                # Write The Text To The Screen
  glMatrixMode(GL_PROJECTION)			# Select The Projection Matrix
  glPopMatrix()					# Restore The Old Projection Matrix
  glMatrixMode(GL_MODELVIEW)			# Select The Modelview Matrix
  glPopMatrix()					# Restore The Old Projection Matrix
  glEnable(GL_DEPTH_TEST)	
  
def initGL():
  load_textures()
  BuildFont()
  glClearColor(0.0,0.0,0.0,0.0)
  glClearDepth(1.0)
  glDepthFunc(GL_LEQUAL)							
  glBlendFunc(GL_SRC_ALPHA,GL_ONE)					
  glShadeModel(GL_SMOOTH)							
  glEnable(GL_TEXTURE_2D)
  
def DrawGLScene():	
  global textures		
  global cnt1, cnt2						

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)	# Clear The Screen And The Depth Buffer
  glLoadIdentity()					# Reset The Modelview Matrix
  glBindTexture(GL_TEXTURE_2D, textures[1])			
  glTranslatef(0.0,0.0,-5.0)						
  glRotatef(45.0,0.0,0.0,1.0)					
  glRotatef(cnt1*30.0,1.0,1.0,0.0)				
  glDisable(GL_BLEND)								
  glColor3f(1.0,1.0,1.0)							
  
  glBegin(GL_QUADS)									
  glTexCoord2d(0.0,0.0)					# First Texture Coord
  glVertex2f(-1.0, 1.0)					# First Vertex
  glTexCoord2d(1.0,0.0)					# Second Texture Coord
  glVertex2f( 1.0, 1.0)					# Second Vertex
  glTexCoord2d(1.0,1.0)					# Third Texture Coord
  glVertex2f( 1.0,-1.0)					# Third Vertex
  glTexCoord2d(0.0,1.0)					# Fourth Texture Coord
  glVertex2f(-1.0,-1.0)					# Fourth Vertex
  glEnd()
  							# Done Drawing The First Quad
  glRotatef(90.0,1.0,1.0,0.0)                           # Rotate On The X & Y Axis By 90 Degrees (Left To Right)

  glBegin(GL_QUADS)					# Draw Our Second Texture Mapped Quad
  glTexCoord2d(0.0,0.0)					# First Texture Coord
  glVertex2f(-1.0, 1.0)					# First Vertex
  glTexCoord2d(1.0,0.0)					# Second Texture Coord
  glVertex2f( 1.0, 1.0)					# Second Vertex
  glTexCoord2d(1.0,1.0)					# Third Texture Coord
  glVertex2f( 1.0,-1.0)					# Third Vertex
  glTexCoord2d(0.0,1.0)					# Fourth Texture Coord
  glVertex2f(-1.0,-1.0)					# Fourth Vertex
  glEnd()
  							# Done Drawing Our Second Quad
  glEnable(GL_BLEND)					# Enable Blending

  glLoadIdentity()					# Reset The View
  # Pulsing Colors Based On Text Position
  glColor3f(1.0*float(cos(cnt1)),1.0*(sin(float(cnt2))),1.0-0.5*(cos(float(cnt1+cnt2))))
  glPrint(int((280+250*cos(cnt1))),int(235+200*sin(cnt2)),"NeHe",0)	# Print GL Text To The Screen

  glColor3f(1.0*(sin(float(cnt2))),1.0-0.5*(cos(float(cnt1+cnt2))),1.0*(cos(float(cnt1))))
  glPrint(int((280+230*cos(cnt2))),int(235+200*sin(cnt1)),"OpenGL",1)	# Print GL Text To The Screen

  glColor3f(0.0,0.0,1.0)						# Set Color To Blue
  glPrint(int(240+200*cos((cnt2+cnt1)/5)),2,"Giuseppe D'Agata",0)

  glColor3f(1.0,1.0,1.0)						# Set Color To White
  glPrint(int(242+200*cos((cnt2+cnt1)/5)),2,"Giuseppe D'Agata",0)

  cnt1+=0.01								# Increase The First Counter
  cnt2+=0.0081
  
def main():
  height, width = 640, 480
  
  pygame.init()
  pygame.display.init()
  video_flags = OPENGL|DOUBLEBUF|RESIZABLE
  pygame.display.set_mode((height,width), video_flags)
  pygame.key.set_repeat(20,50) 
  initGL()
  resize_window((height, width))
  
  
  frames = 0
  done = 0
  ticks = pygame.time.get_ticks() 
  while not done:
    while 1:
      event = pygame.event.poll()
      if event.type == NOEVENT:
        break
      if event.type == KEYDOWN:
        if handleKeyPress(event.key) == 0:
          done = 1
      if event.type == QUIT:
        done = 1
    #Loop the scene
    DrawGLScene()
    pygame.display.flip()
    frames = frames+1

  print "fps:  %d" % ((frames*1000)/(pygame.time.get_ticks()-ticks))

if __name__ == '__main__': main()	  		                         	
	


 
