#!/usr/bin/env python

#*****************************************************************************
# Pygame, PyOpenGL and Numpy version of Nehe's OpenGL lesson19                           
#
# Federico B. 2003 - dragonscraft@yahoo.it
#
#*****************************************************************************

import os
import Numeric
from random import *
from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image
from pygame.locals import *
import psyco; psyco.jit()
from psyco.classes import *

slowdown = 2.0     # Slow Down Particles
xspeed = 0.0       # Base X Speed (To Allow Keyboard Direction Of Tail)
yspeed = 0.0       # Base Y Speed (To Allow Keyboard Direction Of Tail)
zoom = -40.0       # Used To Zoom Out

MAX_PARTICLES = 1000 # Max number of particles (fro performance we use 500p)

loop = 0 			# Misc Loop Variable
col = 0				# Current Color Selection
delay =	0			# Rainbow Effect Delay
texture = []		# Storage For Our Particle Texture
particle = []		#We will store here our particles objects
		
colors=Numeric.array([[1.0,0.5,0.5],[1.0,0.75,0.5],[1.0,1.0,0.5],[0.75,1.0,0.5],
                      [0.5,1.0,0.5],[0.5,1.0,0.75],[0.5,1.0,1.0],[0.5,0.75,1.0],
                      [0.5,0.5,1.0],[0.75,0.5,1.0],[1.0,0.5,1.0],[1.0,0.5,0.75]])

class particles(object): #it would had more sense call this calss PARTICLE
  	
  def __init__(self, index):
    self.active ='true'										# Active (Yes/No)
    self.index = index										
    self.life = 1.0											# Particle Life
    self.fade = (float(100 * random())/1000.0+0.003)		# Fade Speed
    self.r = colors[self.index*(12/MAX_PARTICLES)][0]		# Red Value
    self.g = colors[self.index*(12/MAX_PARTICLES)][1]		# Green Value
    self.b = colors[self.index*(12/MAX_PARTICLES)][2]		# Blue Value
    self.xi = (float((50 * random())-26.0)*10.0)			# X Direction
    self.yi = (float((50 * random())-26.0)*10.0)			# Y Direction
    self.zi = (float((50 * random())-26.0)*10.0)			# Z Direction
    self.x = 0.0											# X Position
    self.y = 0.0											# Y Position
    self.z = 0.0											# Z Position
    self.xg = 0.0											# X Gravity
    self.yg = -0.8											# Y Gravity
    self.zg = 0.0											# Z Gravity

def load_textures():
  global  texture
    
  texturefile = os.path.join('data','Particle.bmp')
  textureSurface = pygame.image.load(texturefile)
  textureData = pygame.image.tostring(textureSurface, "RGBX", 1)

  texture = glGenTextures(2)
    
  glBindTexture(GL_TEXTURE_2D, texture[0])
  glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, textureSurface.get_width(), textureSurface.get_height(), 0,
                GL_RGBA, GL_UNSIGNED_BYTE, textureData )
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)                
                      	
def resize((width, height)):
    if height==0:
        height=1
    glViewport(0, 0, width, height)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(45, 1.0*width/height, 0.1, 100.0)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

def initGL():
  global texture, particle, particles
   
  load_textures()
  glShadeModel(GL_SMOOTH)
  glClearColor(0.0, 0.0, 0.0, 0.0)
  glClearDepth(1.0)
  glDisable(GL_DEPTH_TEST)
  glEnable(GL_BLEND)								# Enable Blending
  glBlendFunc(GL_SRC_ALPHA,GL_ONE)					# Type Of Blending To Perform
  glHint(GL_POINT_SMOOTH_HINT, GL_NICEST)	
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)
  glEnable(GL_TEXTURE_2D)
  glBindTexture(GL_TEXTURE_2D,texture[0])
  
  append = particle.append							#this is the list of 1000 particle
  for loop in xrange(0,MAX_PARTICLES,1):			#span 1000 particle
    append(particles(loop))   						#Add particle to the list

def DrawGLScene():
  global particle, xspeed, yspeed, slowdown, zoom

  glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)
  glLoadIdentity()
    
  for loop in xrange(0,MAX_PARTICLES,1):		# Loop Through All The Particles
    if (particle[loop].active=='true'):			# If The Particle Is Active
      x=particle[loop].x						# Grab Our Particle X Position
      y=particle[loop].y						# Grab Our Particle Y Position
      z=particle[loop].z+zoom			        # Particle Z Pos + Zoom
            
      # Draw The Particle Using Our RGB Values, Fade The Particle Based On It's Life
      glColor4f(particle[loop].r,particle[loop].g,particle[loop].b,particle[loop].life)
            
      glBegin(GL_TRIANGLE_STRIP)# Build Quad From A Triangle Strip
      glTexCoord2d(1,1)
      glVertex3f(x+0.5,y+0.5,z) # Top Right
      glTexCoord2d(0,1)
      glVertex3f(x-0.5,y+0.5,z) # Top Left
      glTexCoord2d(1,0) 
      glVertex3f(x+0.5,y-0.5,z) # Bottom Right
      glTexCoord2d(0,0) 
      glVertex3f(x-0.5,y-0.5,z) # Bottom Left
      glEnd()			# Done Building Triangle Strip
            
      particle[loop].x+=particle[loop].xi/(slowdown*1000)# Move On The X Axis By X Speed
      particle[loop].y+=particle[loop].yi/(slowdown*1000)# Move On The Y Axis By Y Speed
      particle[loop].z+=particle[loop].zi/(slowdown*1000)# Move On The Z Axis By Z Speed
            
      particle[loop].xi+=particle[loop].xg			# Take Pull On X Axis Into Account
      particle[loop].yi+=particle[loop].yg			# Take Pull On Y Axis Into Account
      particle[loop].zi+=particle[loop].zg			# Take Pull On Z Axis Into Account
      particle[loop].life-=particle[loop].fade		        # Reduce Particles Life By 'Fade'
            
      if (particle[loop].life<0.0):					# If Particle Is Burned Out
        particle[loop].life=1.0						# Give It New Life
        particle[loop].fade=((float(100 * random()))/1000.0+0.003)	# randomom Fade Value
        particle[loop].x=0.0						# Center On X Axis
        particle[loop].y=0.0						# Center On Y Axis
        particle[loop].z=0.0						# Center On Z Axis
        particle[loop].xi=xspeed+(float((60 * random()))-32.0)	# X Axis Speed And Direction
        particle[loop].yi=yspeed+(float((60 * random()))-30.0)	# Y Axis Speed And Direction
        particle[loop].zi=(float((60 * random()))-30.0)         # Z Axis Speed And Direction
        particle[loop].r=colors[col][0]				# Select Red From Color Table
        particle[loop].g=colors[col][1]				# Select Green From Color Table
        particle[loop].b=colors[col][2]				# Select Blue From Color Table

        
def handleKeyPress(key):
  global particle, slowdown, xspeed, yspeed, zoom
  # We have to change all the particles so loop (even here) inside them
  for loop in xrange(0,MAX_PARTICLES,1):
    if key==K_ESCAPE:
      return 0         
    # If Number Pad 8 And Y Gravity Is Less Than 1.5 Increase Pull Upwards
    elif (key==K_UP and (particle[loop].yg<1.5)):
      particle[loop].yg+=1.01          
    # If Number Pad 2 And Y Gravity Is Greater Than -1.5 Increase Pull Downwards
    elif (key==K_DOWN and (particle[loop].yg>-1.5)): 
      particle[loop].yg-=1.01       
    # If Number Pad 6 And X Gravity Is Less Than 1.5 Increase Pull Right
    elif (key==K_RIGHT and (particle[loop].xg<1.5)): 
      particle[loop].xg+=1.01       
    # If Number Pad 4 And X Gravity Is Greater Than -1.5 Increase Pull Left
    elif (key==K_LEFT and (particle[loop].xg>-1.5)): 
      particle[loop].xg-=1.01            
    elif key==K_TAB:						# Tab Key Causes A Burst
      particle[loop].x=0.0					# Center On X Axis
      particle[loop].y=0.0					# Center On Y Axis
      particle[loop].z=0.0					# Center On Z Axis
      particle[loop].xi=((float((50 * random()))-26.0)*10.0)	# randomom Speed On X Axis
      particle[loop].yi=((float((50 * random()))-25.0)*10.0)	# randomom Speed On Y Axis
      particle[loop].zi=((float((50 * random()))-25.0)*10.0)    # randomom Speed On Z Axis
  if (key==K_PLUS and (slowdown>1.0)):
    slowdown-=1.0	                                # Speed Up Particles
  elif (key==K_MINUS and (slowdown<4.0)):
    slowdown+=1.0	                                # Slow Down Particles      
  elif (key==K_PAGEUP):
  	zoom+=1.0		                                # Zoom In
  elif (key==K_PAGEDOWN):
  	zoom-=1.0	                                    # Zoom Out         
  # If Up Arrow And Y Speed Is Less Than 200 Increase Upward Speed
  elif (key==K_UP and (yspeed<200)):
    yspeed+=1.0
  # If Down Arrow And Y Speed Is Greater Than -200 Increase Downward Speed
  elif (key==K_DOWN and (yspeed>-200)):
    yspeed-=1.0
  # If Right Arrow And X Speed Is Less Than 200 Increase Speed To The Right
  elif (key==K_RIGHT and (xspeed<200)):
    xspeed+=1.0
  # If Left Arrow And X Speed Is Greater Than -200 Increase Speed To The Left
  elif (key==K_LEFT and (xspeed>-200)):
    xspeed-=1.0
       
def main():

  video_flags = OPENGL|DOUBLEBUF
    
  pygame.init()
  pygame.display.set_mode((640,480), video_flags)
  pygame.key.set_repeat(0,1) 

  resize((640,480))
  initGL()

  frames = 0
  done = 0
  ticks = pygame.time.get_ticks() 
  while not done:
    while 1:
      event = pygame.event.poll()
      if event.type == NOEVENT:
        break
      if event.type == KEYDOWN:
        if handleKeyPress(event.key) == 0:
          done = 1
      if event.type == QUIT:
        done = 1
    #Loop the scene
    DrawGLScene()
    pygame.display.flip()
      
    frames = frames+1

  print "fps:  %d" % ((frames*1000)/(pygame.time.get_ticks()-ticks))


if __name__ == '__main__': main()
