#!/usr/bin/env python

#*****************************************************************************
# Pygame, PyOpenGL and Numpy version of Nehe's OpenGL lesson20                           
#
# Federico B. 2003 - dragonscraft@yahoo.it
#
#*****************************************************************************

import os
from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image, pygame.key
from pygame.locals import *


textures=[]
roll=0.0
masking=0
scene=0

def load_textures():
  global textures
  texturefile=[]
  textureSurface=[]
  textureData=[]
    
  texturefile.append(os.path.join('data','logo.bmp'))
  texturefile.append(os.path.join('data','mask1.bmp'))
  texturefile.append(os.path.join('data','image1.bmp'))
  texturefile.append(os.path.join('data','mask2.bmp'))
  texturefile.append(os.path.join('data','image2.bmp'))
    
  for image_n in range(5):								
    textureSurface.append(pygame.image.load(texturefile[image_n]))
    textureData.append(pygame.image.tostring(textureSurface[image_n], "RGBX", 1))
      
  textures = glGenTextures(5)
    
  for texture_n in range(5):	# We bind all 5 texture with this loop
    glBindTexture(GL_TEXTURE_2D, textures[texture_n])
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
    glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, textureSurface[texture_n].get_width(), textureSurface[texture_n].get_height(), 0,
                  GL_RGBA, GL_UNSIGNED_BYTE, textureData[texture_n] )
                  
def resize((width, height)):
    if height==0:
        height=1
    glViewport(0, 0, width, height)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(45, 1.0*width/height, 0.1, 100.0)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()
    
def initGL():

    load_textures()
    glClearColor(0.0,0.,0.0,0.0)
    glClearDepth(1.0)
    glEnable(GL_DEPTH_TEST) 
    glShadeModel(GL_SMOOTH)
    glEnable(GL_TEXTURE_2D)
    
def DrawGLScene():										# Here's Where We Do All The Drawing
	global roll, textures, masking, scene

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)	# Clear The Screen And The Depth Buffer
	glLoadIdentity()									# Reset The Modelview Matrix
	glTranslatef(0.0,0.0,-2.0)							# Move Into The Screen 2 Units

	glBindTexture(GL_TEXTURE_2D, textures[0])			# Select Our Logo Texture
	glBegin(GL_QUADS)									# Start Drawing A Textured Quad
	glTexCoord2f(0.0, -roll+0.0) 
	glVertex3f(-1.1, -1.1,  0.0)						# Bottom Left
	glTexCoord2f(3.0, -roll+0.0) 
	glVertex3f( 1.1, -1.1 , 0.0)						# Bottom Right
	glTexCoord2f(3.0, -roll+3.0) 
	glVertex3f( 1.1,  1.1,  0.0)						# Top Right
	glTexCoord2f(0.0, -roll+3.0) 
	glVertex3f(-1.1,  1.1,  0.0)						# Top Left
	glEnd()												# Done Drawing The Quad

	glEnable(GL_BLEND)									# Enable Blending
	glDisable(GL_DEPTH_TEST)							# Disable Depth Testing

	if (masking==1):									# Is Masking Enabled?
	  glBlendFunc(GL_DST_COLOR,GL_ZERO)					# Blend Screen Color With Zero (Black)

	if (scene==1):										# Are We Drawing The Second Scene?
	  glTranslatef(0.0,0.0,-1.0)						# Translate Into The Screen One Unit
	  glRotatef(roll*360.0,0.0,0.0,1.0)					# Rotate On The Z Axis 360 Degrees.
	  
	  if (masking==1):									# Is Masking On?
	    glBindTexture(GL_TEXTURE_2D, textures[3])		# Select The Second Mask Texture
	    glBegin(GL_QUADS)								# Start Drawing A Textured Quad
	    glTexCoord2f(0.0, 0.0) 
	    glVertex3f(-1.1, -1.1, 0.0)						# Bottom Left
	    glTexCoord2f(1.0, 0.0) 
	    glVertex3f( 1.1, -1.1, 0.0)						# Bottom Right
	    glTexCoord2f(1.0, 1.0) 
	    glVertex3f( 1.1,  1.1, 0.0)						# Top Right
	    glTexCoord2f(0.0, 1.0) 
	    glVertex3f(-1.1,  1.1, 0.0)						# Top Left
	    glEnd()											# Done Drawing The Quad

	  glBlendFunc(GL_ONE, GL_ONE)						# Copy Image 2 Color To The Screen
	  glBindTexture(GL_TEXTURE_2D, textures[4])			# Select The Second Image Texture
	  glBegin(GL_QUADS)									# Start Drawing A Textured Quad
	  glTexCoord2f(0.0, 0.0)
	  glVertex3f(-1.1, -1.1, 0.0)						# Bottom Left
	  glTexCoord2f(1.0, 0.0)
	  glVertex3f( 1.1, -1.1, 0.0)						# Bottom Right
	  glTexCoord2f(1.0, 1.0)
	  glVertex3f( 1.1,  1.1, 0.0)						# Top Right
	  glTexCoord2f(0.0, 1.0)
	  glVertex3f(-1.1,  1.1, 0.0)						# Top Left
	  glEnd()											# Done Drawing The Quad
	else:												# Otherwise
	  if (masking==1):									# Is Masking On?
	    glBindTexture(GL_TEXTURE_2D, textures[1])		# Select The First Mask Texture
	    glBegin(GL_QUADS)								# Start Drawing A Textured Quad
	    glTexCoord2f(roll+0.0, 0.0) 
	    glVertex3f(-1.1, -1.1,  0.0)					# Bottom Left
	    glTexCoord2f(roll+4.0, 0.0) 
	    glVertex3f( 1.1, -1.1,  0.0)					# Bottom Right
	    glTexCoord2f(roll+4.0, 4.0) 
	    glVertex3f( 1.1,  1.1,  0.0)					# Top Right
	    glTexCoord2f(roll+0.0, 4.0) 
	    glVertex3f(-1.1,  1.1,  0.0)					# Top Left
	    glEnd()											# Done Drawing The Quad

	  glBlendFunc(GL_ONE, GL_ONE)						# Copy Image 1 Color To The Screen
	  glBindTexture(GL_TEXTURE_2D, textures[2])			# Select The First Image Texture
	  glBegin(GL_QUADS)									# Start Drawing A Textured Quad
	  glTexCoord2f(roll+0.0, 0.0) 
	  glVertex3f(-1.1, -1.,  0.0)						# Bottom Left
	  glTexCoord2f(roll+4.0, 0.0) 
	  glVertex3f( 1.1, -1.1,  0.0)						# Bottom Right
	  glTexCoord2f(roll+4.0, 4.0) 
	  glVertex3f( 1.1,  1.1,  0.0)						# Top Right
	  glTexCoord2f(roll+0.0, 4.0) 
	  glVertex3f(-1.1,  1.1,  0.0)						# Top Left
	  glEnd()											# Done Drawing The Quad

	  glEnable(GL_DEPTH_TEST)							# Enable Depth Testing
	  glDisable(GL_BLEND)								# Disable Blending

	roll+=0.002											# Increase Our Texture Roll Variable
	if (roll>1.0):										# Is Roll Greater Than One
		roll-=1.0										# Subtract 1 From Roll

def handleKeyPress(key):
  global masking, scene
    
  if key==K_ESCAPE:
    return 0
  elif key==K_m:
    if masking==1:
      masking=0
    elif masking==0:
      masking=1
  elif key==K_SPACE:
    if scene==1:  
      scene=0
    elif scene==0: 
      scene=1    
  return 1 		
				
def main():

  video_flags = OPENGL|DOUBLEBUF
    
  pygame.init()
  pygame.display.set_mode((640,480), video_flags)
  pygame.key.set_repeat(0,1) 

  resize((640,480))
  initGL()

  frames = 0
  done = 0
  ticks = pygame.time.get_ticks() 
  while not done:
    while 1:
      event = pygame.event.poll()
      if event.type == NOEVENT:
        break
      if event.type == KEYDOWN:
        if handleKeyPress(event.key) == 0:
          done = 1
      if event.type == QUIT:
        done = 1
    #Loop the scene
    DrawGLScene()
    pygame.display.flip()
      
    frames = frames+1

  print "fps:  %d" % ((frames*1000)/(pygame.time.get_ticks()-ticks))


if __name__ == '__main__': main()
		
