#!/usr/bin/env python

#*****************************************************************************
# Pygame, PyOpenGL and Numpy version of Nehe's OpenGL lesson17                          
#
# Federico B. 2003 - dragonscraft@yahoo.it
#
#*****************************************************************************

import os
from math import sin, cos
from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image, pygame.key
from pygame.locals import *
from string import *

textures = []

base = []

swidth=0.0
sheight=0.0

maxtokens=0

scroll = 0

def handleKeyPress(key):
  global scroll, maxtokens
    
  if key==K_ESCAPE:
    return 0
  if ((key==K_UP) & (scroll>0)):				
    scroll-=2										
  elif ((key==K_DOWN) & (scroll<32*(maxtokens-9))):
    scroll+=2	
  return 1							

def resize_window((width, height)):
    if height==0:
        height=1
    glViewport(0, 0, width, height)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    glOrtho(0.0,width,height,0.0,-1.0,1.0) 
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()   
   
def load_textures():
    global textures
    
    texturefile = os.path.join('data','Font.tga')
    textureSurface = pygame.image.load(texturefile)
    textureData = pygame.image.tostring(textureSurface, "RGBX", 1)

    textures = glGenTextures(2)

    glBindTexture(GL_TEXTURE_2D, textures[0])
    gluBuild2DMipmaps(GL_TEXTURE_2D, 4 ,textureSurface.get_width() , 
		 textureSurface.get_height() , GL_RGBA, GL_UNSIGNED_BYTE, textureData )            
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)


def BuildFont():
  global textures
  global base

  cx = 0.0					# Holds Our X Character Coord
  cy = 0.0					# Holds Our Y Character Coord
  base=glGenLists(256)				# Creating 256 Display Lists
  glBindTexture(GL_TEXTURE_2D, textures[0])	# Select Our Font Texture
  for loop in xrange(0,256,1):			   # Loop Through All 256 Lists
  
    cx=float(loop%16)/16.0			# X Position Of Current Character
    cy=float(loop/16)/16.0			# Y Position Of Current Character
    
    glNewList(base+loop,GL_COMPILE)		# Start Building A List
    glBegin(GL_QUADS)				# Use A Quad For Each Character
    glTexCoord2f(cx,1.0-cy-0.0625)		# Texture Coord (Bottom Left)
    glVertex2i(0,16)				# Vertex Coord (Bottom Left)
    glTexCoord2f(cx+0.0625,1.0-cy-0.0625)	# Texture Coord (Bottom Right)
    glVertex2i(16,16)				# Vertex Coord (Bottom Right)
    glTexCoord2f(cx+0.0625,1.0-cy)		# Texture Coord (Top Right)
    glVertex2i(16,0)				# Vertex Coord (Top Right)
    glTexCoord2f(cx,1.0-cy)			# Texture Coord (Top Left)
    glVertex2i(0,0)			        # Vertex Coord (Top Left)
    glEnd()					# Done Building Our Quad (Character)    
    glTranslated(10,0,0)			# Move To The Right Of The Character
    glEndList()	
	
def glPrint(x,y,set,string):
  global textures
  global base
  
  if set>1:
    set=1
    
  glEnable(GL_TEXTURE_2D)			# Enable Texture Mapping
  glLoadIdentity()					# Reset The Modelview Matrix
  glTranslated(x,y,0)				# Position The Text (0,0 - Top Left)
  glListBase(base-32+(128*set))		# Choose The Font Set (0 or 1)

  glScalef(1.0,2.0,1.0)			    # Make The Text 2X Taller

  glCallLists(string)	# Write The Text To The Screen
  glDisable(GL_TEXTURE_2D)			# Disable Texture Mapping
  
def initGL():
  global textures

  load_textures()
  BuildFont()
  glClearColor(0.0,0.0,0.0,0.0)
  glClearDepth(1.0)											
  glShadeModel(GL_SMOOTH)
  glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA) # We Disable them because I prefer follow
  glEnable(GL_BLEND)				                # C version, we enable alpha by adding a 
                                                    # flag when creating the window
def DrawGLScene():	
  global textures, maxtokens, scroll								

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)	# Clear The Screen And The Depth Buffer
  glLoadIdentity()					                    # Reset The Modelview Matrix
  glColor3f(1.0,0.5,0.5)								# Set Color To Bright Red
  glPrint(50,16,1,"Renderer")							# Display Renderer
  glPrint(80,48,1,"Vendor")								# Display Vendor Name
  glPrint(66,80,1,"Version")							# Display Version

  glColor3f(1.0,0.7,0.4)								# Set Color To Orange
  glPrint(200,16,1,(glGetString(GL_RENDERER)))	    # Display Renderer
  glPrint(200,48,1,(glGetString(GL_VENDOR)))	    # Display Vendor Name
  glPrint(200,80,1,(glGetString(GL_VERSION)))	    # Display Version

  glColor3f(0.5,0.5,1.0)								# Set Color To Bright Blue
  glPrint(192,432,1,"NeHe Productions")					# Write NeHe Productions At The Bottom Of The Screen

  glLoadIdentity()										# Reset The ModelView Matrix
  glColor3f(1.0,1.0,1.0)								# Set The Color To White
  glBegin(GL_LINE_STRIP)								# Start Drawing Line Strips (Something New)
  glVertex2d(639,417)								    # Top Right Of Bottom Box
  glVertex2d(  0,417)								    # Top Left Of Bottom Box
  glVertex2d(  0,480)								    # Lower Left Of Bottom Box
  glVertex2d(639,480)								    # Lower Right Of Bottom Box
  glVertex2d(639,128)								    # Up To Bottom Right Of Top Box
  glEnd()												# Done First Line Strip
  glBegin(GL_LINE_STRIP)								# Start Drawing Another Line Strip
  glVertex2d(  0,128)								    # Bottom Left Of Top Box
  glVertex2d(639,128)								    # Bottom Right Of Top Box								
  glVertex2d(639,  1)								    # Top Right Of Top Box
  glVertex2d(  0,  1)								    # Top Left Of Top Box
  glVertex2d(  0,417)								    # Down To Top Left Of Bottom Box
  glEnd()												# Done Second Line Strip

  glScissor(1 ,int(0.135416*sheight),swidth-2,int(0.597916*sheight))	# Define Scissor Region
  glEnable(GL_SCISSOR_TEST)
  
  tok = (glGetString(GL_EXTENSIONS))
  token = split(tok) 			   # Why don't use the python string functions?
  maxtokens=len(token)
  
  for cnt in xrange(0,maxtokens,1):     # We go thorught the token list untile the end
    glColor3f(0.5,1.0,0.5)			        		# Set Color To Bright Green
    glPrint(0,(32+(96+(cnt*32)-scroll)),0,("%d" % (cnt+1)) ) # Print Current Extension Number
    glColor3f(1.0,1.0,0.5)			        		# Set Color To Yellow
    glPrint(50,(32+(96+(cnt*32)-scroll)),0,token[cnt])# Print The Current Token (Parsed Extension Name)
    
  glDisable(GL_SCISSOR_TEST)			      		# Disable Scissor Testing

  
def main():
  global swidth, sheight
  height, width = 480, 640
  sheight=height
  swidth=width
  
  pygame.init()
  pygame.display.init()
  video_flags = OPENGL|DOUBLEBUF|RESIZABLE|SRCALPHA # This enable alpha from source
  pygame.display.set_mode((width,height), video_flags)
  pygame.key.set_repeat(1,0) 
  initGL()
  resize_window((width, height))
    
  frames = 0
  done = 0
  ticks = pygame.time.get_ticks() 
  while not done:
    while 1:
      event = pygame.event.poll()
      if event.type == NOEVENT:
        break
      if event.type == KEYDOWN:
        if handleKeyPress(event.key) == 0:
          done = 1
      if event.type == QUIT:
        done = 1
    #Loop the scene
    DrawGLScene()
    pygame.display.flip()
    frames = frames+1

  print "fps:  %d" % ((frames*1000)/(pygame.time.get_ticks()-ticks))

if __name__ == '__main__': main()	  		                         	
	


 
