#!/usr/bin/env python

#*****************************************************************************
# Pygame, PyOpenGL and Numpy version of Nehe's OpenGL lesson17                          
#
# Federico B. 2003 - dragonscraft@yahoo.it
#
#*****************************************************************************

import os
import sys
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from string import *
import array
import struct

class TextureImage(object): 
 
   def __init__(self):
 	self.imageData = []	       # Image Data (Up To 32 Bits)
	self.bpp = 0			   # Image Color Depth In Bits Per Pixel.
	self.width = 0		       # Image Width
	self.height = 0		       # Image Height
	self.texID = 0		       # Texture ID Used To Select A Texture

Tex = TextureImage()

base = []

swidth=0.0
sheight=0.0

maxtokens=0

scroll = 0

ESCAPE = '\033'

def handleKeyPress(key,x,y):
  global scroll, maxtokens
    
  if key==ESCAPE:
    sys.exit()
  if ((key==GLUT_KEY_UP) & (scroll>0)):				
    scroll-=2										
  if ((key==GLUT_KEY_DOWN) & (scroll<32*(maxtokens-9))):
    scroll+=2											

def resize_window((width, height)):
    if height==0:
        height=1
    glViewport(0, 0, width, height)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    glOrtho(0.0,width,height,0.0,-1.0,1.0) # I really don't like this impostation, anyway..
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()   

def load_textures( texture, filename):
    
    h = h1 = h2 = ()        # Tuples to hold header data
    ImgCMD = ImgCMD = ()    # Tuples to hold image color map data if present
    bytesPerPixel = 0		# Holds Number Of Bytes Per Pixel Used In The TGA File
    imageSize = 0		    # Used To Store The Image Size When Setting Aside Ram
    Ttype=GL_RGBA			# Set The Default GL Mode To RBGA (32 BPP)

    tgafile = open(filename, 'rb')
    ############################ TGA FILE HEADER ########################
    h = struct.unpack('bbb', tgafile.read(3)) 
    # ID Length
    # Color Map Type
    # Image Type :
      # 0 No Image Data Included
      # 1 Uncompressed, Color-mapped Image
      # 2 Uncompressed, True-color Image
      # 3 Uncompressed, Black-and-white Image
      # 9 Run-length encoded, Color-mapped Image
      # 10 Run-length encoded, True-color Image
      #11 Run-length encoded, Black-and-white Image
    h1= struct.unpack('hhb', tgafile.read(5)) 
    # Color Map Specification :
       # First Entry Index
       # Color map Length
       # Color map Entry Size
    h2 = struct.unpack("hhhhbb", tgafile.read(10)) 
    # Image Specification :
      # X-origin of Image
      # Y-origin of Image
      # Image Width
      # Image Height
      # Pixel Depth
      # Image Descriptor
    ######################### IMAGE/COLOR MAP DATA #######################    
    if (h[0] != 0):  # Is there an Image ID?
      ImgCMD= struct.unpack('hhb', tgafile.read(h[0])) # Yes, get it (lenght is in header pos 0)
      
    if (h[1] != 0):  # Is there Color Map Data?
      ImgCMD1= struct.unpack('hhb', tgafile.read(h[0])) # Yes, get it (lenght is in header pos 0)  
        
    texture.width  = h2[2]			# Determine The TGA Width	(highbyte*256+lowbyte)
    texture.height = h2[3]			# Determine The TGA Height	(highbyte*256+lowbyte)
   
    texture.bpp   = h2[4]						# Grab The TGA's Bits Per Pixel (24 or 32)
    bytesPerPixel = texture.bpp/8					# Divide By 8 To Get The Bytes Per Pixel
    imageSize     = texture.width*texture.height*bytesPerPixel	# Calculate The Memory Required For The TGA Data

    if (h[2]==0): # It has no image so close the file
      tgafile.close()
      return
    elif (h[2]>=9): # It'rle encoded not yet supported ;)
      tgafile.close()
      return    
    ######################### COLOR DEPTH ########################

             #################### 16 ##################    
    if (texture.bpp==16):
      texture.ImageData = array('b', imageSize ) 
      b = g = r = 0
      for i in xrange(0,imageSize,1):
        pixels = struct.unpack('H', tgafile.read(2))
        b = (pixels & 0x1f) << 3
        g = ((pixels >> 5) & 0x1f) << 3
        r = ((pixels >> 10) & 0x1f) << 3				
        # This essentially assigns the color to our array and swaps the
        # B and R values at the same time.
        texture.imageData[i * 3 + 0] = r
        texture.imageData[i * 3 + 1] = g
        texture.imageData[i * 3 + 2] = b
             ################# 32 - 24 ################
    else:         
      texture.imageData = tgafile.read(imageSize) # Read image data in byte format
      	
    tgafile.close()  # done
	
    ########################  OPENGL BINDING ##############################
    if(texture.bpp==24):					# Was The TGA 24 Bits	
      Ttype=GL_RGB							# If So Set The 'type' To GL_RGB
      
    texture.texID = glGenTextures(1)    

    glBindTexture(GL_TEXTURE_2D, texture.texID)    
    glTexImage2D(GL_TEXTURE_2D, 0, Ttype, texture.width, texture.height, 0,
                Ttype, GL_UNSIGNED_BYTE, texture.imageData)
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)

def BuildFont():
  global base

  cx = 0.0					# Holds Our X Character Coord
  cy = 0.0					# Holds Our Y Character Coord
  base=glGenLists(256)				# Creating 256 Display Lists
  glBindTexture(GL_TEXTURE_2D, Tex.texID)	# Select Our Font Texture
  for loop in xrange(0,256,1):			   # Loop Through All 256 Lists
  
    cx=float(loop%16)/16.0			# X Position Of Current Character
    cy=float(loop/16)/16.0			# Y Position Of Current Character
    
    glNewList(base+loop,GL_COMPILE)		# Start Building A List
    glBegin(GL_QUADS)				# Use A Quad For Each Character
    glTexCoord2f(cx,1.0-cy-0.0625)		# Texture Coord (Bottom Left)
    glVertex2i(0,16)				# Vertex Coord (Bottom Left)
    glTexCoord2f(cx+0.0625,1.0-cy-0.0625)	        # Texture Coord (Bottom Right)
    glVertex2i(16,16)				# Vertex Coord (Bottom Right)
    glTexCoord2f(cx+0.0625,1.0-cy)		# Texture Coord (Top Right)
    glVertex2i(16,0)				# Vertex Coord (Top Right)
    glTexCoord2f(cx,1.0-cy)			# Texture Coord (Top Left)
    glVertex2i(0,0)			        # Vertex Coord (Top Left)
    glEnd()					# Done Building Our Quad (Character)    
    glTranslated(14,0,0)			# Move To The Right Of The Character
    glEndList()	
	
def glPrint(x,y,set,string):
  global base
  
  if set>1:
    set=1
    
  glEnable(GL_TEXTURE_2D)			# Enable Texture Mapping
  glLoadIdentity()					# Reset The Modelview Matrix
  glTranslated(x,y,0)				# Position The Text (0,0 - Top Left)
  glListBase(base-32+(128*set))		# Choose The Font Set (0 or 1)

  glScalef(1.0,2.0,1.0)			    # Make The Text 2X Taller

  glCallLists(string)	# Write The Text To The Screen
  glDisable(GL_TEXTURE_2D)			# Disable Texture Mapping
  
def initGL():

  load_textures( Tex,'data\Font.tga')
  BuildFont()
  glClearColor(0.0,0.0,0.0,0.0)
  glClearDepth(1.0)											
  glShadeModel(GL_SMOOTH)
  glEnable(GL_DEPTH_TEST)						# Enable Depth Testing
  #glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)# Enable Alpha Blending (disable alpha testing)
  #glEnable(GL_BLEND)							# Enable Blending       (disable alpha testing)
  #glAlphaFunc(GL_GREATER,0.1)				    # Set Alpha Testing     (disable blending)
  #glEnable(GL_ALPHA_TEST)	 					# Enable Alpha Testing  (disable blending)
   
def DrawGLScene():	
  global textures, maxtokens, scroll, sheight, swidth								

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)	# Clear The Screen And The Depth Buffer
  glLoadIdentity()					                    # Reset The Modelview Matrix
  glColor3f(1.0,0.5,0.5)								# Set Color To Bright Red
  glPrint(50,16,1,"Renderer")							# Display Renderer
  glPrint(80,48,1,"Vendor")								# Display Vendor Name
  glPrint(66,80,1,"Version")							# Display Version

  glColor3f(1.0,0.7,0.4)								# Set Color To Orange
  glPrint(200,16,1,(glGetString(GL_RENDERER)))	    # Display Renderer
  glPrint(200,48,1,(glGetString(GL_VENDOR)))	    # Display Vendor Name
  glPrint(200,80,1,(glGetString(GL_VERSION)))	    # Display Version

  glColor3f(0.5,0.5,1.0)								# Set Color To Bright Blue
  glPrint(192,432,1,"NeHe Productions")					# Write NeHe Productions At The Bottom Of The Screen

  glLoadIdentity()										# Reset The ModelView Matrix
  glColor3f(1.0,1.0,1.0)								# Set The Color To White
  glBegin(GL_LINE_STRIP)								# Start Drawing Line Strips (Something New)
  glVertex2d(639,417)								    # Top Right Of Bottom Box
  glVertex2d(  0,417)								    # Top Left Of Bottom Box
  glVertex2d(  0,480)								    # Lower Left Of Bottom Box
  glVertex2d(639,480)								    # Lower Right Of Bottom Box
  glVertex2d(639,128)								    # Up To Bottom Right Of Top Box
  glEnd()												# Done First Line Strip
  glBegin(GL_LINE_STRIP)								# Start Drawing Another Line Strip
  glVertex2d(  0,128)								    # Bottom Left Of Top Box
  glVertex2d(639,128)								    # Bottom Right Of Top Box								
  glVertex2d(639,  1)								    # Top Right Of Top Box
  glVertex2d(  0,  1)								    # Top Left Of Top Box
  glVertex2d(  0,417)								    # Down To Top Left Of Bottom Box
  glEnd()												# Done Second Line Strip
  # We need to know the current size of the window to mantain aspect ratio
  sheight = glutGet(GLUT_WINDOW_HEIGHT)
  swidth = glutGet(GLUT_WINDOW_WIDTH)
  
  glScissor(1 ,(int(0.135416*sheight)),(swidth-2),(int(0.597916*sheight)))	# Define Scissor Region
  glEnable(GL_SCISSOR_TEST)
  
  tok = (glGetString(GL_EXTENSIONS))
  token = split(tok) 			   # Why don't use the python string functions?
  maxtokens=len(token)
  for cnt in range(0,maxtokens,1):     # We go thorught the token list untile the end

    glColor3f(0.5,1.0,0.5)			        		# Set Color To Bright Green
    glPrint(0,(32+(96+(cnt*32)-scroll)),0,("%d" % (cnt+1)) ) # Print Current Extension Number
    glColor3f(1.0,1.0,0.5)			        		# Set Color To Yellow
    glPrint(50,(32+(96+(cnt*32)-scroll)),0,token[cnt])# Print The Current Token (Parsed Extension Name)
  glDisable(GL_SCISSOR_TEST)			      		# Disable Scissor Testing
  
  glutSwapBuffers()

def main():
  global swidth, sheight
  height, width = 480, 640
  sheight=float(height)
  swidth=float(width)
  
  glutInit(sys.argv)
  glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_ALPHA | GLUT_DEPTH)	
  glutInitWindowSize(width,height)
  glutInitWindowPosition(0, 0)	
  window = glutCreateWindow("Scissor and TGA loading")
  glutDisplayFunc(DrawGLScene)	
  glutIdleFunc(DrawGLScene)	
  glutReshapeFunc(resize_window((width,height)))	
  glutKeyboardFunc(handleKeyPress)
  glutSpecialFunc(handleKeyPress)
  initGL()
  glutMainLoop()


if __name__ == '__main__': main()	  		                         	
	


 
