#!/usr/bin/env python

#*****************************************************************************
# Pygame, PyOpenGL and Numpy version of Nehe's OpenGL lesson25                           
#
# Federico B. 2003 - dragonscraft@yahoo.it
#
#*****************************************************************************

import os
from OpenGL.GL import *
from OpenGL.GLU import *
import pygame, pygame.image, pygame.key
from pygame.locals import *
from string import *
from random import random

################################################# 
xrot,yrot,zrot = 0.0,0.0,0.0			# X, Y & Z Rotation
xspeed,yspeed,zspeed =	0.0,0.0,0.0		# X, Y & Z Spin Speed
cx,cy,cz = 0.0,0.0,-15.0

step, steps = 0, 200

MAX_VERT = 486

################################################# 
class CVertex(object):

  def __init__(self, x, y ,z):
    self.x = x
    self.y = y
    self.z = z	
    
#################################################    
class CObject(object):

  def __init__(self):
    self.verts = 0
    self.points  = []
   
  def objload(self, name):  
    global MAX_VERT
     
    f = open(os.path.join("data", name))
    lines = f.readlines()
    f.close()
    for line in lines:
      vals = split(line)
      if (vals[0] == "Vertices:"):
        self.verts=int(vals[1])
        MAX_VERT=self.verts
        continue
      else:
        x = float(vals[0])
        y = float(vals[1])
        z = float(vals[2])
        self.points.append(CVertex(x,y,z))
        
#################################################
helper = CObject()
sour = dest = 0
morph_list = [CObject(),CObject(),CObject(),CObject()]
morph = 0

#################################################
def vertex_calculate(i):
  global morph_list,sour,dest, steps
  
  x = float((morph_list[sour].points[i].x - morph_list[dest].points[i].x)/steps)
  y = float((morph_list[sour].points[i].y - morph_list[dest].points[i].y)/steps)
  z = float((morph_list[sour].points[i].z - morph_list[dest].points[i].z)/steps)
  return CVertex(x,y,z)
  
#################################################      
def resize((width, height)):
    if height==0:
        height=1
    glViewport(0, 0, width, height)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(45, 1.0*width/height, 0.1, 100.0)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()
    
#################################################    
def initGL():
    global morph, helper, sour,dest, MAX_VERT

    glBlendFunc(GL_SRC_ALPHA,GL_ONE)	  # Set The Blending Function For Translucency
    glClearColor(0.0, 0.0, 0.0, 0.0)	  # This Will Clear The Background Color To Black
    glClearDepth(1.0)				      # Enables Clearing Of The Depth Buffer
    glDepthFunc(GL_LESS)			      # The Type Of Depth Test To Do
    glEnable(GL_DEPTH_TEST)			      # Enables Depth Testing
    glShadeModel(GL_SMOOTH)			      # Enables Smooth Color Shading
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST) # Really Nice Perspective Calculations
		    
    morph_list[0].objload('sphere.txt')
    morph_list[1].objload('torus.txt')
    morph_list[2].objload('tube.txt')
    
    for i in range(0,MAX_VERT,1):	# Loop through all 486 vertices
      x=(float((random()*14)-7))	# morph4 x point becomes a random float value from -7 to 7
      y=(float((random()*14)-7))	# morph4 y point becomes a random float value from -7 to 7
      z=(float((random()*14)-7))	# morph4 z point becomes a random float value from -7 to 7
      morph_list[3].points.append(CVertex(x,y,z))
    
    helper.objload('sphere.txt')
    
    sour = dest = 0
	      
#################################################   
def DrawGLScene():										# Here's Where We Do All The Drawing
    global cx,cy,cz, xrot,yrot,zrot, xspeed,yspeed,zspeed 
    global MAX_VERT, morph, helper, step, steps, sour, dest

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)	# Clear The Screen And The Depth Buffer
    glLoadIdentity()					# Reset The Modelview Matrix
    glTranslatef(0.0,0.0,-2.0)				# Move Into The Screen 2 Units
	
    glTranslatef(cx,cy,cz)		# Translate the current position to start drawing
    glRotatef(xrot,1.0,0.0,0.0)		# Rotate on the X axis by xrot
    glRotatef(yrot,0.0,1.0,0.0)		# Rotate on the Y axis by yrot
    glRotatef(zrot,0.0,0.0,1.0)		# Rotate on the Z axis by zrot

    xrot+=xspeed 
    yrot+=yspeed
    zrot+=zspeed		# Increase xrot,yrot & zrot by xspeed, yspeed & zspeed
	
    tx,ty,tz = 0.0,0.0,0.0						# Temp X, Y & Z Variables
    q = CVertex(0.0,0.0,0.0)						# Holds Returned Calculated Values For One Vertex

    glBegin(GL_POINTS)							# Begin Drawing Points
    for i in xrange(0,MAX_VERT,1):					# Loop Through All The Verts Of morph0 (All Objects Have
									# The Same Amount Of Verts For Simplicity, Could Use maxver Also)
      if(morph==1): 
        q=vertex_calculate(i)
      else:
        q.x=q.y=q.z=0.0						# If morph Is True Calculate Movement Otherwise Movement=0
      helper.points[i].x-=q.x						# Subtract q.x Units From helper.points[i].x (Move On X Axis)
      helper.points[i].y-=q.y						# Subtract q.y Units From helper.points[i].y (Move On Y Axis)
      helper.points[i].z-=q.z						# Subtract q.z Units From helper.points[i].z (Move On Z Axis)
      tx=helper.points[i].x						# Make Temp X Variable Equal To Helper's X Variable
      ty=helper.points[i].y						# Make Temp Y Variable Equal To Helper's Y Variable
      tz=helper.points[i].z						# Make Temp Z Variable Equal To Helper's Z Variable
      glColor3f(0.0,1.0,1.0)							# Set Color To A Bright Shade Of Off Blue
      glVertex3f(tx,ty,tz)						# Draw A Point At The Current Temp Values (Vertex)
      glColor3f(0.0,0.5,1.0)						# Darken Color A Bit
      tx-=2*q.x
      ty-=2*q.y
      ty-=2*q.y								# Calculate Two Positions Ahead
      glVertex3f(tx,ty,tz)						# Draw A Second Point At The Newly Calculate Position
      glColor3f(0.0,0.0,1.0)							# Set Color To A Very Dark Blue
      tx-=2*q.x
      ty-=2*q.y
      ty-=2*q.y								# Calculate Two More Positions Ahead
      glVertex3f(tx,ty,tz)						# Draw A Third Point At The Second New Position													
	  								# This Creates A Ghostly Tail As Points Move	  
    glEnd()	
	
    if((morph==1) & (step<=steps)):
      step+=1
    else:
      morph=0
      sour=dest
      step=0

################################################ 
def handleKeyPress(key):
  global cx,cy,cz, dest,morph, xspeed,yspeed,zspeed, morph
  k = 0
    
  if key==K_ESCAPE:
    return 0
  elif key==K_UP:
    xspeed+=0.1
  elif key==K_DOWN:
    xspeed-=0.1
  elif key==K_LEFT:
    yspeed-=0.1
  elif key==K_RIGHT:
    yspeed+=0.1
  elif key==K_PAGEDOWN:
    zspeed+=0.1
  elif key==K_PAGEUP:
    zspeed+=0.1
  elif key==K_q:
    cz-=0.1
  elif key==K_z:
    cz-=0.1
  elif key==K_w:
    cy-=0.1
  elif key==K_s:
    cy-=0.1
  elif key==K_d:
    cx-=0.1
  elif key==K_a:
    cx-=0.1
  elif ((key==K_1) & (k!=1) & (morph==0)):
    morph=1	
    dest=0
    k=1
  elif ((key==K_2) & (k!=2) & (morph==0)): 
    morph=1	
    dest=1
    k = 2
  elif ((key==K_3) & (k!=3) & (morph==0)):
    morph=1
    dest=2
    k = 3
  elif ((key==K_4) & (k!=4) & (morph==0)):
    morph=1
    dest=3
    k = 4
  return 1 		
  
################################################# 				
def main():

  video_flags = OPENGL|DOUBLEBUF
    
  pygame.init()
  pygame.display.set_mode((640,480), video_flags)
  pygame.key.set_repeat(10,10) 

  resize((640,480))
  initGL()

  frames = 0
  done = 0
  ticks = pygame.time.get_ticks() 
  while not done:
    while 1:
      event = pygame.event.poll()
      if event.type == NOEVENT:
        break
      if event.type == KEYDOWN:
        if handleKeyPress(event.key) == 0:
          done = 1
      if event.type == QUIT:
        done = 1
    #Loop the scene
    DrawGLScene()
    pygame.display.flip()
      
    frames = frames+1

  print "fps:  %d" % ((frames*1000)/(pygame.time.get_ticks()-ticks))

################################################# 
if __name__ == '__main__': main()
		
