"""
Program		:	Acid Blur (not that I'd know)
Author		:	Gareth "Korruptor" Noyce (<a href=http://www.korruptor.demon.co.uk/>Home</a>)
Depend		:	Numeric, Pygame with Surfarray
Description	:

    This is probably familiar to anyone who uses iTunes or Winamp visualisations, 
and it's another *really* easy one.

Basically, a four step process:

1) Scale the display surface
2) Blur the scaled surface
3) Draw something onto the blurred, scaled surface
3) Blit the central portion of this to the display

I'll leave out the goto 1 ;-)

The blur is a basic convolution, as demonstrated by Pete in his excellent Surfarray
tutorial. The only thing here that's new is our use of pygame's transform functions 
to scale the surface.

This is dedicated to slm, who seems to think I'm writing a visualisation library. :)
"""

import pygame, pygame.transform, pygame.image
from pygame.surfarray import *
from pygame.locals import *
from Numeric import *


# -----------------------------------------------------------------------------------
def main():
    RES = array((320,200))
    PI 	= 3.14159
    DEG2RAD = PI/180
    
    pygame.init()

    # Setup an 8bit screen...
    screen = pygame.display.set_mode(RES,0,8)
    pygame.display.set_caption('Acid Blur')
    
    # Load a sprite and setup the palettes and colorkey info
    sprite = pygame.image.load("sprite.gif")
    sprite.set_colorkey((0,0,0))
    screen.set_palette(sprite.get_palette())

    # This is our "working surface"...
    worksurf = pygame.Surface(RES,0,8)
    #just make an integer array the same as the screen
    workarr = zeros(RES, 'i')

    # Angle deltas for sin/cos
    xd = 0
    yd = 0
 
    # Fruity loops...
    while 1:
        # Have we received an event to quit the program?
        for e in pygame.event.get():
            if e.type in (QUIT,KEYDOWN,MOUSEBUTTONDOWN):
                return

        # Calculate a new x/y position for the sprite...
        x    = 50 * cos((xd*DEG2RAD) * 2.75)
        y    = 50 * sin((yd*DEG2RAD) * 1.13)

        # Increment the 'angles'
        xd  += 1.5
        yd  += 3

        # Scale the working surface up by 16px x 16px -- obviously
        # smaller values == a slower spreading effect.
        worksurf = pygame.transform.scale(screen, RES+16)

        # Copy the surf array of the scaled surface to a temporary array
	# you can make the blur skew in any direction by slicing a different
	# part of the array, like [6:-10,6:-10] to move down and right
        tmp = pygame.surfarray.array2d(worksurf)[8:-8,8:-8]
	
        # simplified blur, averages 4 neighboring pixels
        workarr[1:,:]  = tmp[:-1,:]
        workarr[:-1,:] += tmp[1:,:]
        workarr[:,1:]  += tmp[:,:-1]
        workarr[:,:-1] += tmp[:,1:]
        workarr >>= 2
        
        # send the blurred surface and original sprite to the screen
        blit_array(screen, workarr)
        screen.blit(sprite,(x+110,y+50))
	pygame.display.update()


if __name__ == '__main__': main()

